@@ -10,8 +10,10 @@
 
 #include <lttng/domain.h>
 #include <lttng/event.h>
+#include <lttng/lttng-error.h>
 #include <lttng/lttng-export.h>
 
+#include <stdbool.h>
 #include <stdint.h>
 
 #ifdef __cplusplus
@@ -33,6 +35,38 @@ enum lttng_channel_allocation_policy {
 	LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CHANNEL = 1,
 };
 
+/*!
+@brief
+    \ref api-channel-buf-prealloc-policy "Buffer preallocation policy"
+    of a channel.
+
+@ingroup api_channel
+*/
+enum lttng_channel_preallocation_policy {
+	/// Preallocate sub-buffers.
+	LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE = 0,
+
+	/// Allocate sub-buffers as needed.
+	LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND = 1,
+};
+
+/*!
+@brief
+    Status code for \lt_obj_channel property accessors.
+
+@ingroup api_channel
+*/
+enum lttng_channel_status {
+	/// Success.
+	LTTNG_CHANNEL_STATUS_OK = 0,
+
+	/// Property is unset.
+	LTTNG_CHANNEL_STATUS_UNSET = 1,
+
+	/// Unsatisfied precondition.
+	LTTNG_CHANNEL_STATUS_INVALID = -1,
+};
+
 /*
  * Tracer channel attributes. For both kernel and user-space.
  *
@@ -216,6 +250,11 @@ following ones have their own dedicated accessors:
     - lttng_channel_get_monitor_timer_interval()
     - lttng_channel_set_monitor_timer_interval()
 
+  <dt>\ref api-channel-watchdog-timer "Watchdog timer" period
+  <dd>
+    - lttng_channel_get_watchdog_timer_interval()
+    - lttng_channel_set_watchdog_timer_interval()
+
   <dt>\ref api-channel-blocking-timeout "Blocking timeout"
   <dd>
     - lttng_channel_get_blocking_timeout()
@@ -340,7 +379,7 @@ LTTNG_EXPORT extern void lttng_channel_destroy(struct lttng_channel *channel);
       #lttng_domain.
     @lt_pre_not_null{channels}
 */
-LTTNG_EXPORT extern int lttng_list_channels(struct lttng_handle *handle,
+LTTNG_EXPORT extern int lttng_list_channels(const struct lttng_handle *handle,
 					    struct lttng_channel **channels);
 
 /*!
@@ -513,7 +552,7 @@ channel summaries of a given \lt_obj_session and \lt_obj_domain.
 @sa lttng_channel_get_lost_packet_count() --
     Returns the number of discarded packets (sub-buffers) of a channel.
 */
-LTTNG_EXPORT extern int lttng_channel_get_discarded_event_count(struct lttng_channel *channel,
+LTTNG_EXPORT extern int lttng_channel_get_discarded_event_count(const struct lttng_channel *channel,
 								uint64_t *count);
 
 /*!
@@ -556,7 +595,7 @@ channel summaries of a given \lt_obj_session and \lt_obj_domain.
 @sa lttng_channel_get_discarded_event_count() --
     Returns the number of discarded event records of a channel.
 */
-LTTNG_EXPORT extern int lttng_channel_get_lost_packet_count(struct lttng_channel *channel,
+LTTNG_EXPORT extern int lttng_channel_get_lost_packet_count(const struct lttng_channel *channel,
 							    uint64_t *count);
 
 /*!
@@ -617,7 +656,6 @@ lttng_channel_get_monitor_timer_interval(const struct lttng_channel *channel, ui
 
 @pre
     @lt_pre_not_null{channel}
-    - \lt_p{period}&nbsp;≥&nbsp;1
 
 @sa lttng_channel_get_monitor_timer_interval() --
     Returns the monitor timer period property of a channel summary.
@@ -625,6 +663,68 @@ lttng_channel_get_monitor_timer_interval(const struct lttng_channel *channel, ui
 LTTNG_EXPORT extern int lttng_channel_set_monitor_timer_interval(struct lttng_channel *channel,
 								 uint64_t period);
 
+/*!
+@brief
+    Sets \lt_p{*period} to the
+    \ref api-channel-watchdog-timer "watchdog timer" period (µs)
+    property of the \lt_obj_channel summary \lt_p{channel}.
+
+@ingroup api_channel
+
+@param[in] channel
+    Summary of the channel of which to get the watchdog timer period.
+@param[out] period
+    <strong>On success</strong>, this function sets \lt_p{*period} to
+    the watchdog timer period (µs) property of \lt_p{channel}.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS_INVALID
+    Unsatisfied precondition.
+@retval #LTTNG_CHANNEL_STATUS_UNSET
+    Watchdog timer isn't set.
+
+@pre
+    @lt_pre_not_null{channel}
+    @lt_pre_not_null{period}
+
+@sa lttng_channel_get_watchdog_timer_interval() --
+    Returns the watchdog timer period property of a channel summary.
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_get_watchdog_timer_interval(const struct lttng_channel *channel, uint64_t *period);
+
+/*!
+@brief
+    Sets the \ref api-channel-watchdog-timer "watchdog timer" period
+    property of the channel summary \lt_p{channel} to
+    \lt_p{period}&nbsp;µs.
+
+@ingroup api_channel
+
+@param[in] channel
+    Channel summary of which to set the watchdog timer period
+    to \lt_p{period}&nbsp;µs.
+@param[in] period
+    Watchdog timer period property to set.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS__INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{channel}
+    - The \lt_obj_domain of \lt_p{channel} is #LTTNG_DOMAIN_UST.
+    - The \ref api-channel-buf-ownership-model "buffer ownership model"
+      of \lt_p{channel} is #LTTNG_BUFFER_PER_UID.
+
+@sa lttng_channel_set_watchdog_timer_interval() --
+    Sets the watchdog timer period property of a channel summary.
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_set_watchdog_timer_interval(struct lttng_channel *channel, uint64_t period);
+
 /*!
 @brief
     Sets \lt_p{*timeout} to the
@@ -800,6 +900,134 @@ LTTNG_EXPORT extern enum lttng_error_code
 lttng_channel_set_allocation_policy(struct lttng_channel *channel,
 				    enum lttng_channel_allocation_policy policy);
 
+/*!
+@brief
+    Sets \lt_p{*policy} to the
+    \ref api-channel-buf-prealloc-policy "buffer preallocation policy"
+    of the \lt_obj_channel summary \lt_p{channel}.
+
+@ingroup api_channel
+
+This property only applies to \link #LTTNG_DOMAIN_UST user space\endlink
+channels.
+
+@param[in] channel
+    Summary of the channel of which to get the
+    buffer preallocation policy.
+@param[out] policy
+    <strong>On success</strong>, this function sets \lt_p{*policy} to
+    the buffer preallocation policy of \lt_p{channel}.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS_INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{channel}
+    - The \lt_obj_domain type of \lt_p{channel} is #LTTNG_DOMAIN_UST.
+    @lt_pre_not_null{policy}
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_get_preallocation_policy(const struct lttng_channel *channel,
+				       enum lttng_channel_preallocation_policy *policy);
+
+/*!
+@brief
+    Sets the
+    \ref api-channel-buf-prealloc-policy "buffer preallocation policy"
+    property of the channel summary \lt_p{channel} to \lt_p{policy}.
+
+@ingroup api_channel
+
+This property only applies to \link #LTTNG_DOMAIN_UST user space\endlink
+channels.
+
+@param[in] channel
+    Channel summary of which to set the buffer preallocation policy to
+    \lt_p{policy}.
+@param[in] policy
+    Buffer preallocation policy to set.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS_INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{channel}
+    - The \lt_obj_domain type of \lt_p{channel} is #LTTNG_DOMAIN_UST.
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_set_preallocation_policy(struct lttng_channel *channel,
+				       enum lttng_channel_preallocation_policy policy);
+
+/*!
+@brief
+    Sets \lt_p{*older_than_us} to the age threshold (µs) of the
+    \ref api-channel-auto-reclaim "automatic memory reclaim"
+    of the channel summary \lt_p{channel}.
+
+@ingroup api_channel
+
+This property only applies to \link #LTTNG_DOMAIN_UST user space\endlink
+channels.
+
+@param[in] channel
+    Summary of the channel of which to get the automatic memory
+    reclaim maximal age.
+@param[out] older_than_us
+    <strong>On success</strong>, this function sets
+    \lt_p{*older_than_us} to the age threshold (µs) of the
+    automatic memory reclaim.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS_UNSET
+    The automatic memory reclaim age threshold is not
+    set for this channel.
+@retval #LTTNG_CHANNEL_STATUS_INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{channel}
+    - The \lt_obj_domain type of \lt_p{channel} is #LTTNG_DOMAIN_UST.
+    @lt_pre_not_null{older_than_us}
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_get_automatic_memory_reclamation_policy(const struct lttng_channel *channel,
+						      uint64_t *older_than_us);
+
+/*!
+@brief
+    Sets the \ref api-channel-auto-reclaim "automatic memory reclaim"
+    policy of the
+    \lt_obj_channel summary \lt_p{channel} to reclaim buffered data
+    older than \lt_p{older_than_us}&nbsp;µs.
+
+@ingroup api_channel
+
+This property only applies to \link #LTTNG_DOMAIN_UST user space\endlink
+channels.
+
+@param[in] channel
+    Channel summary of which to set the automatic memory reclaim policy.
+@param[in] older_than_us
+    Minimum age (µs) of buffered data to consider to reclaim memory.
+
+@retval #LTTNG_CHANNEL_STATUS_OK
+    Success.
+@retval #LTTNG_CHANNEL_STATUS_INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{channel}
+    - The \lt_obj_domain type of \lt_p{channel} is #LTTNG_DOMAIN_UST.
+*/
+LTTNG_EXPORT extern enum lttng_channel_status
+lttng_channel_set_automatic_memory_reclamation_policy(struct lttng_channel *channel,
+						      uint64_t older_than_us);
+
 #ifdef __cplusplus
 }
 #endif
