@@ -686,6 +686,12 @@ enum lttng_error_code {
 	/// Failed to flush a buffer.
 	LTTNG_ERR_BUFFER_FLUSH_FAILED = 170,
 
+	/// Invalid reclamation policy.
+	LTTNG_ERR_INVALID_RECLAMATION_POLICY = 171,
+
+	/// Trace format is not supported.
+	LTTNG_ERR_UNSUPPORTED_TRACE_FORMAT = 172,
+
 	/* MUST be last element of the manually-assigned section of the enum */
 	/// @cond INTERNAL_ERR_CODES
 	LTTNG_ERR_NR,
