@@ -64,11 +64,13 @@
 #include <lttng/lttng-error.h>
 #include <lttng/notification/channel.h>
 #include <lttng/notification/notification.h>
+#include <lttng/reclaim.h>
 #include <lttng/rotation.h>
 #include <lttng/save.h>
 #include <lttng/session-descriptor.h>
 #include <lttng/session.h>
 #include <lttng/snapshot.h>
+#include <lttng/stream-info.h>
 #include <lttng/tracker.h>
 #include <lttng/trigger/trigger.h>
 #include <lttng/userspace-probe.h>
