@@ -29,7 +29,7 @@ struct lttng_session_descriptor;
 
 /*!
 @brief
-    Return type of recording session descriptor fuctions.
+    Return type of recording session descriptor functions.
 
 Error status enumerators have a negative value.
 */
@@ -44,6 +44,28 @@ enum lttng_session_descriptor_status {
 	LTTNG_SESSION_DESCRIPTOR_STATUS_UNSET = 1,
 };
 
+/*!
+@brief
+    Trace format of a recording session.
+*/
+enum lttng_trace_format {
+	/*!
+	@brief
+	    Use the default trace format of the
+	    \ref api-gen-sessiond-conn "connected session daemon".
+
+	See the <code>\--default-trace-format</code> option of
+	\lt_man{lttng-sessiond,8}.
+	*/
+	LTTNG_TRACE_FORMAT_DEFAULT = 0,
+
+	/// <a href="https://diamon.org/ctf/v1.8.3/">Common Trace Format version&nbsp;1.8</a>.
+	LTTNG_TRACE_FORMAT_CTF_1_8 = 1,
+
+	/// <a href="https://diamon.org/ctf/">Common Trace Format version&nbsp;2</a>.
+	LTTNG_TRACE_FORMAT_CTF_2 = 2,
+};
+
 /*!
 @brief
     Creates a recording session descriptor to create a no-output,
@@ -660,6 +682,32 @@ LTTNG_EXPORT extern enum lttng_session_descriptor_status
 lttng_session_descriptor_get_session_name(const struct lttng_session_descriptor *session_descriptor,
 					  const char **session_name);
 
+/*!
+@brief
+    Sets the trace format of the recording session descriptor
+    \lt_p{session_descriptor} to \lt_p{format}.
+
+Call this to set the trace format of the recording session to create
+from \lt_p{session_descriptor} before you call
+lttng_create_session_ext().
+
+@param[in] session_descriptor
+    Recording session descriptor of which to set the trace format.
+@param[in] format
+    Trace format of \lt_p{session_descriptor}.
+
+@retval #LTTNG_SESSION_DESCRIPTOR_STATUS_OK
+    Success.
+@retval #LTTNG_SESSION_DESCRIPTOR_STATUS_INVALID
+    Unsatisfied precondition.
+
+@pre
+    @lt_pre_not_null{session_descriptor}
+*/
+LTTNG_EXPORT extern enum lttng_session_descriptor_status
+lttng_session_descriptor_set_trace_format(struct lttng_session_descriptor *descriptor,
+					  enum lttng_trace_format format);
+
 /*!
 @brief
     Destroys the recording session descriptor \lt_p{session_descriptor}.
