@@ -23,10 +23,9 @@ struct lttng_session_extended {
 	struct {
 		char value[LTTNG_PATH_MAX];
 	} LTTNG_PACKED shm_path;
+
+	/* enum lttng_trace_format */
+	uint8_t trace_format;
 } LTTNG_PACKED;
 
-using lttng_session_uptr = std::unique_ptr<
-	lttng_session[],
-	lttng::memory::create_deleter_class<lttng_session, lttng::memory::free>::deleter>;
-
 #endif /* LTTNG_SESSION_INTERNAL_H */
