@@ -21,11 +21,13 @@
 #include "viewer-session.hpp"
 #include "viewer-stream.hpp"
 
+#include <common/buffer-view.hpp>
 #include <common/common.hpp>
 #include <common/compat/endian.hpp>
 #include <common/compat/poll.hpp>
 #include <common/compat/socket.hpp>
 #include <common/defaults.hpp>
+#include <common/dynamic-buffer.hpp>
 #include <common/fd-tracker/utils.hpp>
 #include <common/fs-handle.hpp>
 #include <common/futex.hpp>
@@ -142,6 +144,18 @@ lttng_viewer_next_index_return_code_str(enum lttng_viewer_next_index_return_code
 	}
 }
 
+static lttng_live_trace_format trace_format_to_live_format(const lttng_trace_format format)
+{
+	switch (format) {
+	case LTTNG_TRACE_FORMAT_CTF_1_8:
+		return LTTNG_LIVE_TRACE_FORMAT_CTF_1_8;
+	case LTTNG_TRACE_FORMAT_CTF_2:
+		return LTTNG_LIVE_TRACE_FORMAT_CTF_2;
+	default:
+		abort();
+	}
+}
+
 static const char *lttng_viewer_attach_return_code_str(enum lttng_viewer_attach_return_code code)
 {
 	switch (code) {
@@ -817,18 +831,14 @@ static struct lttcomm_sock *init_socket(struct lttng_uri *uri, const char *name)
 		LTTNG_THROW_ERROR(lttng::format("Failed to create \"{}\" socket", formatted_name));
 	}
 
-	auto fd_tracker_remove_on_error =
-		lttng::make_scope_exit([&sock, &sock_fd, &relayd_live_port_path]() noexcept {
-			if (sock.get() != nullptr) {
-				if (sock->fd >= 0) {
-					fd_tracker_close_unsuspendable_fd(the_fd_tracker,
-									  &sock_fd,
-									  1,
-									  close_sock,
-									  sock.get());
-				}
+	auto fd_tracker_remove_on_error = lttng::make_scope_exit([&sock, &sock_fd]() noexcept {
+		if (sock.get() != nullptr) {
+			if (sock->fd >= 0) {
+				fd_tracker_close_unsuspendable_fd(
+					the_fd_tracker, &sock_fd, 1, close_sock, sock.get());
 			}
-		});
+		}
+	});
 
 	DBG("Listening on %s socket %d", name, sock->fd);
 	ret = sock->ops->bind(sock.get());
@@ -1192,6 +1202,64 @@ end:
 	return ret;
 }
 
+/* Serialize a single session entry for protocol 2.4 (fixed-size strings). */
+static int serialize_viewer_session_2_4(const struct relay_session *session,
+					struct lttng_dynamic_buffer *out)
+{
+	struct lttng_viewer_session_2_4 s = {};
+
+	s.common.id = htobe64(session->id);
+	s.common.live_timer = htobe32(session->live_timer);
+	s.common.clients = htobe32(session->viewer_attached ? 1U : 0U);
+	s.common.streams = htobe32(session->stream_count);
+
+	if (lttng_strncpy(s.hostname, session->hostname, sizeof(s.hostname))) {
+		return -1;
+	}
+
+	if (lttng_strncpy(s.session_name, session->session_name, sizeof(s.session_name))) {
+		return -1;
+	}
+
+	return lttng_dynamic_buffer_append(out, &s, sizeof(s));
+}
+
+/* Serialize a single session entry for protocol 2.15 (variable-length strings). */
+static int serialize_viewer_session_2_15(const struct relay_session *session,
+					 struct lttng_dynamic_buffer *out)
+{
+	int ret;
+	struct lttng_viewer_session_2_15 s = {};
+	const size_t hostname_len = strlen(session->hostname);
+	const size_t session_name_len = strlen(session->session_name);
+
+	s.common.id = htobe64(session->id);
+	s.common.live_timer = htobe32(session->live_timer);
+	s.common.clients = htobe32(session->viewer_attached ? 1U : 0U);
+	s.common.streams = htobe32(session->stream_count);
+	s.hostname_len = htobe32(static_cast<uint32_t>(hostname_len));
+	s.session_name_len = htobe32(static_cast<uint32_t>(session_name_len));
+	s.trace_format =
+		htobe32(static_cast<uint32_t>(trace_format_to_live_format(session->trace_format)));
+
+	ret = lttng_dynamic_buffer_append(out, &s, sizeof(s));
+	if (ret) {
+		return ret;
+	}
+
+	ret = lttng_dynamic_buffer_append(out, session->hostname, hostname_len);
+	if (ret) {
+		return ret;
+	}
+
+	ret = lttng_dynamic_buffer_append(out, session->session_name, session_name_len);
+	if (ret) {
+		return ret;
+	}
+
+	return 0;
+}
+
 /*
  * Send the viewer the list of current sessions.
  * We need to create a copy of the hash table content because otherwise
@@ -1204,96 +1272,68 @@ static int viewer_list_sessions(struct relay_connection *conn)
 {
 	int ret = 0;
 	struct lttng_viewer_list_sessions session_list;
-	struct lttng_viewer_session *send_session_buf = nullptr;
-	uint32_t buf_count = SESSION_BUF_DEFAULT_COUNT;
 	uint32_t count = 0;
+	struct lttng_dynamic_buffer payload;
+	const auto proto_supports_ctf2 = conn->minor >= 15;
+	const auto serialize_viewer_session = proto_supports_ctf2 ? serialize_viewer_session_2_15 :
+								    serialize_viewer_session_2_4;
 
-	send_session_buf = calloc<lttng_viewer_session>(SESSION_BUF_DEFAULT_COUNT);
-	if (!send_session_buf) {
-		return -1;
-	}
+	lttng_dynamic_buffer_init(&payload);
 
 	for (auto *session :
 	     lttng::urcu::lfht_iteration_adapter<relay_session,
 						 decltype(relay_session::session_n),
 						 &relay_session::session_n>(*sessions_ht->ht)) {
-		struct lttng_viewer_session *send_session;
-
 		health_code_update();
 
 		pthread_mutex_lock(&session->lock);
 		if (session->connection_closed) {
-			/* Skip closed session */
-			goto next_session;
+			pthread_mutex_unlock(&session->lock);
+			continue;
 		}
 
-		if (count >= buf_count) {
-			struct lttng_viewer_session *newbuf;
-			const uint32_t new_buf_count = buf_count << 1;
+		/*
+		 * Skip CTF 2 sessions when the viewer is using protocol < 2.15.
+		 * Protocol 2.4 doesn't support communicating the trace format,
+		 * so viewers would incorrectly decode CTF 2 data as CTF 1.8.
+		 */
+		if (!proto_supports_ctf2 && session->trace_format == LTTNG_TRACE_FORMAT_CTF_2) {
+			pthread_mutex_unlock(&session->lock);
+			continue;
+		}
 
-			newbuf = (lttng_viewer_session *) realloc(
-				send_session_buf, new_buf_count * sizeof(*send_session_buf));
-			if (!newbuf) {
-				ret = -1;
-				goto break_loop;
-			}
-			send_session_buf = newbuf;
-			buf_count = new_buf_count;
+		ret = serialize_viewer_session(session, &payload);
+
+		pthread_mutex_unlock(&session->lock);
+
+		if (ret) {
+			goto end;
 		}
-		send_session = &send_session_buf[count];
-		if (lttng_strncpy(send_session->session_name,
-				  session->session_name,
-				  sizeof(send_session->session_name))) {
-			ret = -1;
-			goto break_loop;
-		}
-		if (lttng_strncpy(send_session->hostname,
-				  session->hostname,
-				  sizeof(send_session->hostname))) {
-			ret = -1;
-			goto break_loop;
-		}
-		send_session->id = htobe64(session->id);
-		send_session->live_timer = htobe32(session->live_timer);
-		if (session->viewer_attached) {
-			send_session->clients = htobe32(1);
-		} else {
-			send_session->clients = htobe32(0);
-		}
-		send_session->streams = htobe32(session->stream_count);
+
 		count++;
-	next_session:
-		pthread_mutex_unlock(&session->lock);
-		continue;
-	break_loop:
-		pthread_mutex_unlock(&session->lock);
-		break;
-	}
-
-	if (ret < 0) {
-		goto end_free;
 	}
 
 	session_list.sessions_count = htobe32(count);
 
 	health_code_update();
-
 	ret = send_response(conn->sock, &session_list, sizeof(session_list));
 	if (ret < 0) {
-		goto end_free;
+		goto end;
 	}
 
 	health_code_update();
-
-	ret = send_response(conn->sock, send_session_buf, count * sizeof(*send_session_buf));
-	if (ret < 0) {
-		goto end_free;
+	if (payload.size > 0) {
+		ret = send_response(conn->sock, payload.data, payload.size);
+		if (ret < 0) {
+			goto end;
+		}
 	}
 	health_code_update();
 
 	ret = 0;
-end_free:
-	free(send_session_buf);
+
+end:
+	lttng_dynamic_buffer_reset(&payload);
 	return ret;
 }
 
@@ -1487,6 +1527,20 @@ static int viewer_attach_session(struct relay_connection *conn)
 		goto send_reply;
 	}
 
+	/*
+	 * Reject CTF 2 sessions when the viewer is using protocol < 2.15.
+	 * Protocol 2.4 doesn't support communicating the trace format,
+	 * so viewers would incorrectly decode CTF 2 data as CTF 1.8.
+	 */
+	if (conn->minor < 15 && session->trace_format == LTTNG_TRACE_FORMAT_CTF_2) {
+		viewer_attach_status = LTTNG_VIEWER_ATTACH_UNK;
+		DBG_FMT("Relay session ID {} uses CTF 2 format which is not supported by viewer protocol 2.{}, returning status={}",
+			session_id,
+			conn->minor,
+			lttng_viewer_attach_return_code_str(viewer_attach_status));
+		goto send_reply;
+	}
+
 	switch (be32toh(request.seek)) {
 	case LTTNG_VIEWER_SEEK_BEGINNING:
 	case LTTNG_VIEWER_SEEK_LAST:
@@ -1933,16 +1987,36 @@ static int viewer_get_next_index(struct relay_connection *conn)
 	 */
 	if (vstream->stream_file.trace_chunk) {
 		uint64_t id;
+
 		status = lttng_trace_chunk_get_id(vstream->stream_file.trace_chunk, &id);
 		if (status == LTTNG_TRACE_CHUNK_STATUS_OK) {
-			stream_file_chunk_id = std::to_string(id);
+			try {
+				stream_file_chunk_id = std::to_string(id);
+			} catch (const std::exception& e) {
+				ERR_FMT("Failed to convert stream file trace chunk id to string: {}",
+					e.what());
+				ret = -1;
+				goto end;
+			}
+		} else {
+			LTTNG_ASSERT(status == LTTNG_TRACE_CHUNK_STATUS_NONE);
 		}
 	}
 	if (conn->viewer_session->current_trace_chunk) {
 		uint64_t id;
+
 		status = lttng_trace_chunk_get_id(conn->viewer_session->current_trace_chunk, &id);
 		if (status == LTTNG_TRACE_CHUNK_STATUS_OK) {
-			viewer_session_chunk_id = std::to_string(id);
+			try {
+				viewer_session_chunk_id = std::to_string(id);
+			} catch (const std::exception& e) {
+				ERR_FMT("Failed to convert viewer session trace chunk id to string: {}",
+					e.what());
+				ret = -1;
+				goto end;
+			}
+		} else {
+			LTTNG_ASSERT(status == LTTNG_TRACE_CHUNK_STATUS_NONE);
 		}
 	}
 
@@ -1952,7 +2026,7 @@ static int viewer_get_next_index(struct relay_connection *conn)
 		vstream->last_seen_rotation_count + 1;
 
 	if (viewer_stream_and_session_in_same_chunk) {
-		DBG_FMT("Transition to latest chunk check ({} -> {}) chunk, no need to rotate",
+		DBG_FMT("Transition to latest chunk check ({} -> {}): Same chunk, no need to rotate",
 			vstream->stream_file.trace_chunk ?
 				stream_file_chunk_id.value_or("anonymous") :
 				"None",
