@@ -1183,8 +1183,9 @@ int stream_update_index(struct relay_stream *stream,
 		tracefile_array_file_rotate(stream->tfa, TRACEFILE_ROTATE_READ);
 		tracefile_array_commit_seq(stream->tfa, stream->index_received_seqcount);
 		stream->index_received_seqcount++;
-		LTTNG_OPTIONAL_SET(&stream->received_packet_seq_num,
-				   be64toh(index->index_data.packet_seq_num));
+
+		const uint64_t packet_seq_num = be64toh(index->index_data.packet_seq_num);
+		LTTNG_OPTIONAL_SET(&stream->received_packet_seq_num, packet_seq_num);
 		*flushed = true;
 	} else if (ret > 0) {
 		index->total_size = total_size;
@@ -1245,13 +1246,14 @@ int stream_add_index(struct relay_stream *stream, const struct lttcomm_relayd_in
 
 	/* Live beacon handling */
 	if (index_info->packet_size == 0) {
-		DBG("Received live beacon for stream %" PRIu64, stream->stream_handle);
+		DBG_FMT("Received live beacon for stream {}, indexes_in_flight={}",
+			stream->stream_handle,
+			stream->indexes_in_flight);
 
 		/*
-		 * Only flag a stream inactive when it has already
-		 * received data and no indexes are in flight.
+		 * Only flag a stream inactive when no indexes are in flight.
 		 */
-		if (stream->index_received_seqcount > 0 && stream->indexes_in_flight == 0) {
+		if (stream->indexes_in_flight == 0) {
 			stream->beacon_ts_end = index_info->timestamp_end;
 		}
 		ret = 0;
