@@ -27,25 +27,18 @@
 /*
  * Return allocated channel attributes.
  */
-struct lttng_channel *channel_new_default_attr(int dom, enum lttng_buffer_type type)
+lttng::ctl::lttng_channel_uptr channel_new_default_attr(lttng_domain_type dom,
+							enum lttng_buffer_type type)
 {
-	struct lttng_channel *chan;
 	const char *channel_name = DEFAULT_CHANNEL_NAME;
-	struct lttng_channel_extended *extended_attr = nullptr;
 
-	chan = zmalloc<lttng_channel>();
-	if (chan == nullptr) {
-		PERROR("zmalloc channel init");
-		goto error_alloc;
+	lttng::ctl::lttng_channel_uptr chan(lttng_channel_create_internal());
+	if (!chan) {
+		LTTNG_THROW_POSIX("Failed to allocate channel with default attributes", -ENOMEM);
 	}
 
-	extended_attr = zmalloc<lttng_channel_extended>();
-	if (!extended_attr) {
-		PERROR("zmalloc channel extended init");
-		goto error;
-	}
-
-	chan->attr.extended.ptr = extended_attr;
+	struct lttng_channel_extended *extended_attr =
+		static_cast<lttng_channel_extended *>(chan->attr.extended.ptr);
 
 	/* Same for all domains. */
 	chan->attr.overwrite = DEFAULT_CHANNEL_OVERWRITE;
@@ -64,6 +57,7 @@ struct lttng_channel *channel_new_default_attr(int dom, enum lttng_buffer_type t
 		extended_attr->blocking_timeout = DEFAULT_KERNEL_CHANNEL_BLOCKING_TIMEOUT;
 		extended_attr->monitor_timer_interval = DEFAULT_KERNEL_CHANNEL_MONITOR_TIMER;
 		extended_attr->allocation_policy = DEFAULT_CHANNEL_ALLOCATION_POLICY;
+		extended_attr->preallocation_policy = DEFAULT_CHANNEL_PREALLOCATION_POLICY;
 		break;
 	case LTTNG_DOMAIN_JUL:
 		channel_name = DEFAULT_JUL_CHANNEL_NAME;
@@ -90,7 +84,11 @@ struct lttng_channel *channel_new_default_attr(int dom, enum lttng_buffer_type t
 			extended_attr->blocking_timeout = DEFAULT_UST_UID_CHANNEL_BLOCKING_TIMEOUT;
 			extended_attr->monitor_timer_interval =
 				DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER;
+			LTTNG_OPTIONAL_SET(
+				&extended_attr->watchdog_timer_interval,
+				static_cast<uint64_t>(DEFAULT_UST_UID_CHANNEL_WATCHDOG_TIMER));
 			extended_attr->allocation_policy = DEFAULT_CHANNEL_ALLOCATION_POLICY;
+			extended_attr->preallocation_policy = DEFAULT_CHANNEL_PREALLOCATION_POLICY;
 			break;
 		case LTTNG_BUFFER_PER_PID:
 		default:
@@ -104,33 +102,20 @@ struct lttng_channel *channel_new_default_attr(int dom, enum lttng_buffer_type t
 			extended_attr->monitor_timer_interval =
 				DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER;
 			extended_attr->allocation_policy = DEFAULT_CHANNEL_ALLOCATION_POLICY;
+			extended_attr->preallocation_policy = DEFAULT_CHANNEL_PREALLOCATION_POLICY;
 			break;
 		}
 		break;
 	default:
-		goto error; /* Not implemented */
+		return nullptr;
 	}
 
 	if (snprintf(chan->name, sizeof(chan->name), "%s", channel_name) < 0) {
 		PERROR("snprintf default channel name");
-		goto error;
+		return nullptr;
 	}
+
 	return chan;
-
-error:
-	free(extended_attr);
-	free(chan);
-error_alloc:
-	return nullptr;
-}
-
-void channel_attr_destroy(struct lttng_channel *channel)
-{
-	if (!channel) {
-		return;
-	}
-	free(channel->attr.extended.ptr);
-	free(channel);
 }
 
 /*
@@ -233,19 +218,14 @@ enum lttng_error_code channel_kernel_create(struct ltt_kernel_session *ksession,
 					    struct lttng_channel *attr,
 					    int kernel_pipe)
 {
-	enum lttng_error_code ret_code;
-	struct lttng_channel *defattr = nullptr;
+	lttng::ctl::lttng_channel_uptr defattr;
 
 	LTTNG_ASSERT(ksession);
 
 	/* Creating channel attributes if needed */
 	if (attr == nullptr) {
 		defattr = channel_new_default_attr(LTTNG_DOMAIN_KERNEL, LTTNG_BUFFER_GLOBAL);
-		if (defattr == nullptr) {
-			ret_code = LTTNG_ERR_FATAL;
-			goto error;
-		}
-		attr = defattr;
+		attr = defattr.get();
 	}
 
 	/*
@@ -258,31 +238,24 @@ enum lttng_error_code channel_kernel_create(struct ltt_kernel_session *ksession,
 
 	/* Validate common channel properties. */
 	if (channel_validate(attr) < 0) {
-		ret_code = LTTNG_ERR_INVALID;
-		goto error;
+		return LTTNG_ERR_INVALID;
 	}
 
 	if (channel_validate_kernel(attr) < 0) {
-		ret_code = LTTNG_ERR_INVALID;
-		goto error;
+		return LTTNG_ERR_INVALID;
 	}
 
 	/* Channel not found, creating it. */
 	if (kernel_create_channel(ksession, attr) < 0) {
-		ret_code = LTTNG_ERR_KERN_CHAN_FAIL;
-		goto error;
+		return LTTNG_ERR_KERN_CHAN_FAIL;
 	}
 
 	/* Notify kernel thread that there is a new channel */
 	if (notify_thread_pipe(kernel_pipe) < 0) {
-		ret_code = LTTNG_ERR_FATAL;
-		goto error;
+		return LTTNG_ERR_FATAL;
 	}
 
-	ret_code = LTTNG_OK;
-error:
-	channel_attr_destroy(defattr);
-	return ret_code;
+	return LTTNG_OK;
 }
 
 /*
@@ -340,7 +313,7 @@ enum lttng_error_code channel_ust_create(struct ltt_ust_session *usess,
 {
 	enum lttng_error_code ret_code = LTTNG_OK;
 	struct ltt_ust_channel *uchan = nullptr;
-	struct lttng_channel *defattr = nullptr;
+	lttng::ctl::lttng_channel_uptr defattr;
 	enum lttng_domain_type domain = LTTNG_DOMAIN_UST;
 	bool chan_published = false;
 	const lttng::urcu::read_lock_guard read_lock;
@@ -350,11 +323,7 @@ enum lttng_error_code channel_ust_create(struct ltt_ust_session *usess,
 	/* Creating channel attributes if needed */
 	if (attr == nullptr) {
 		defattr = channel_new_default_attr(LTTNG_DOMAIN_UST, type);
-		if (defattr == nullptr) {
-			ret_code = LTTNG_ERR_FATAL;
-			goto error;
-		}
-		attr = defattr;
+		attr = defattr.get();
 	} else {
 		/*
 		 * HACK: Set the channel's subdomain (JUL, Log4j, Python, etc.)
@@ -498,7 +467,6 @@ enum lttng_error_code channel_ust_create(struct ltt_ust_session *usess,
 		}
 	}
 
-	channel_attr_destroy(defattr);
 	return LTTNG_OK;
 
 error_remove_chan:
@@ -508,7 +476,6 @@ error_remove_chan:
 error_free_chan:
 	trace_ust_destroy_channel(uchan);
 error:
-	channel_attr_destroy(defattr);
 	return ret_code;
 }
 
@@ -558,6 +525,25 @@ error:
 struct lttng_channel *trace_ust_channel_to_lttng_channel(const struct ltt_ust_channel *uchan)
 {
 	struct lttng_channel *channel = nullptr, *ret = nullptr;
+	const int type = uchan->attr.type;
+	enum lttng_channel_allocation_policy allocation_policy;
+	enum lttng_channel_preallocation_policy preallocation_policy;
+
+	/* Convert from internal enum to external enum */
+	switch (uchan->preallocation_policy) {
+	case lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t::
+		PREALLOCATE:
+		preallocation_policy = LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE;
+		break;
+	case lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t::
+		ON_DEMAND:
+		preallocation_policy = LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND;
+		break;
+	default:
+		ERR_FMT("Unknown preallocation policy during conversion from ltt_ust_channel to lttng_channel: policy={}",
+			(int) uchan->preallocation_policy);
+		goto end;
+	}
 
 	channel = lttng_channel_create_internal();
 	if (!channel) {
@@ -595,12 +581,29 @@ struct lttng_channel *trace_ust_channel_to_lttng_channel(const struct ltt_ust_ch
 		break;
 	}
 
-	lttng_channel_set_blocking_timeout(channel, uchan->attr.u.s.blocking_timeout);
+	lttng_channel_set_blocking_timeout(channel, uchan->attr.blocking_timeout);
 	lttng_channel_set_monitor_timer_interval(channel, uchan->monitor_timer_interval);
 
-	enum lttng_channel_allocation_policy allocation_policy;
+	if (uchan->watchdog_timer_interval.is_set) {
+		switch (type) {
+		case LTTNG_UST_ABI_CHAN_PER_CPU:
+			/* fall through */
+		case LTTNG_UST_ABI_CHAN_PER_CHANNEL:
+			if (lttng_channel_set_watchdog_timer_interval(
+				    channel, LTTNG_OPTIONAL_GET(uchan->watchdog_timer_interval)) !=
+			    LTTNG_CHANNEL_STATUS_OK) {
+				ERR_FMT("Failed to set channel watchdog timer during conversion from ltt_ust_channel to lttng_channel");
+				goto end;
+			}
+			break;
+		default:
+			ERR_FMT("Watchdog timer is invalid for UST buffer type: buff_type={}",
+				type);
+			goto end;
+		}
+	}
 
-	switch (uchan->attr.u.s.type) {
+	switch (type) {
 	case LTTNG_UST_ABI_CHAN_PER_CPU:
 		allocation_policy = LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU;
 		break;
@@ -609,7 +612,7 @@ struct lttng_channel *trace_ust_channel_to_lttng_channel(const struct ltt_ust_ch
 		break;
 	default:
 		ERR_FMT("Unknown channel allocation policy: {}",
-			static_cast<int>(uchan->attr.u.s.type));
+			static_cast<int>(uchan->attr.type));
 		goto end;
 	}
 
@@ -619,6 +622,23 @@ struct lttng_channel *trace_ust_channel_to_lttng_channel(const struct ltt_ust_ch
 		goto end;
 	}
 
+	if (lttng_channel_set_preallocation_policy(channel, preallocation_policy) !=
+	    LTTNG_CHANNEL_STATUS_OK) {
+		ERR("Failed to set channel preallocation policy "
+		    "during conversion from ltt_ust_channel to lttng_channel");
+		goto end;
+	}
+
+	if (uchan->automatic_memory_reclamation_maximal_age) {
+		if (lttng_channel_set_automatic_memory_reclamation_policy(
+			    channel, uchan->automatic_memory_reclamation_maximal_age->count()) !=
+		    LTTNG_CHANNEL_STATUS_OK) {
+			ERR("Failed to set automatic memory reclamation maximal age "
+			    "during conversion from ltt_ust_channel to lttng_channel");
+			goto end;
+		}
+	}
+
 	ret = channel;
 	channel = nullptr;
 
