@@ -11,10 +11,14 @@
 #include "clear.hpp"
 #include "client.hpp"
 #include "cmd.hpp"
+#include "commands/get-channel-memory-usage.hpp"
+#include "commands/reclaim-channel-memory.hpp"
+#include "domain.hpp"
 #include "health-sessiond.hpp"
 #include "kernel.hpp"
 #include "lttng-sessiond.hpp"
 #include "manage-consumer.hpp"
+#include "pending-memory-reclamation-request.hpp"
 #include "save.hpp"
 #include "testpoint.hpp"
 #include "utils.hpp"
@@ -33,6 +37,7 @@
 #include <common/pthread-lock.hpp>
 #include <common/scope-exit.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
+#include <common/stream-info.hpp>
 #include <common/tracker.hpp>
 #include <common/unix.hpp>
 #include <common/utils.hpp>
@@ -41,17 +46,22 @@
 #include <lttng/event-internal.hpp>
 #include <lttng/event-rule/event-rule-internal.hpp>
 #include <lttng/lttng.h>
+#include <lttng/reclaim-internal.hpp>
 #include <lttng/session-descriptor-internal.hpp>
 #include <lttng/session-internal.hpp>
 #include <lttng/userspace-probe-internal.hpp>
 
+#include <array>
 #include <fcntl.h>
+#include <grp.h>
 #include <pthread.h>
+#include <pwd.h>
 #include <signal.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <vector>
 
 namespace ls = lttng::sessiond;
 
@@ -63,16 +73,115 @@ struct thread_state {
 	bool running;
 	int client_sock;
 } thread_state;
-} /* namespace */
 
-static void set_thread_status(bool running)
+/*
+ * Maximum number of retries for group list resizing.
+ * 5 retries is chosen as a reasonable upper bound to avoid infinite loops in case
+ * of pathological group database changes or errors. This value can be adjusted if needed.
+ */
+constexpr unsigned int GROUP_LIST_RESIZE_MAX_RETRIES = 5;
+
+bool is_user_part_of_group(uid_t uid, gid_t primary_group, gid_t target_group)
+{
+	if (primary_group == target_group) {
+		return true;
+	}
+
+	/* Get number of groups. getgrouplist() returns -1 when fetching the number of groups. */
+	int ngroups = 0;
+	std::vector<gid_t> groups;
+	unsigned int retry_count = GROUP_LIST_RESIZE_MAX_RETRIES;
+
+	while (retry_count-- > 0) {
+		std::array<char, 1024> pwuid_string_buf;
+		passwd pwd, *pw_result = nullptr;
+
+		const auto getpwuid_ret = getpwuid_r(
+			uid, &pwd, pwuid_string_buf.data(), pwuid_string_buf.size(), &pw_result);
+		if (getpwuid_ret < 0) {
+			LTTNG_THROW_POSIX(
+				fmt::format("Failed to get password file entry of user: uid={}",
+					    uid),
+				errno);
+		} else if (!pw_result) {
+			ERR_FMT("No matching password file entry for user: uid={}", uid);
+			return false;
+		}
+
+		DBG_FMT("Validating user membership of group: uid={}, user_name=`{}`, primary_group={}, target_group={}",
+			uid,
+			pw_result->pw_name,
+			primary_group,
+			target_group);
+
+		if (pw_result->pw_gid != primary_group) {
+			ERR_FMT("Primary group of user does not match the expected primary group: uid={}, user_name=`{}`, primary_group={}, expected_primary_group={}",
+				uid,
+				pw_result->pw_name,
+				pw_result->pw_gid,
+				primary_group);
+			return false;
+		}
+
+		(void) getgrouplist(pw_result->pw_name, pw_result->pw_gid, nullptr, &ngroups);
+		if (ngroups == 0) {
+			DBG_FMT("User is not a member of any groups: uid={}, user_name={}",
+				uid,
+				pw_result->pw_name);
+			return false;
+		}
+
+		groups.resize(ngroups);
+
+		DBG_FMT("Fetching the list of groups for user: uid={}, user_name=`{}`, ngroups={}",
+			uid,
+			pw_result->pw_name,
+			ngroups);
+		const auto getgrouplist_ret = getgrouplist(
+			pw_result->pw_name, pw_result->pw_gid, groups.data(), &ngroups);
+		if (getgrouplist_ret < 0) {
+			/* Group list got resized, retry. */
+			DBG_FMT("Group list of user got resized, retrying: uid={}, user_name={}, previous_ngroups={}, ngroups={}",
+				uid,
+				pw_result->pw_name,
+				groups.size(),
+				ngroups);
+			continue;
+		}
+
+		const auto it = std::find(groups.cbegin(), groups.cend(), target_group);
+		return it != groups.cend();
+	}
+
+	return false;
+}
+
+bool is_user_in_tracing_group(uid_t uid, gid_t primary_group)
+{
+	gid_t tracing_group_id;
+	const auto get_group_id_ret =
+		utils_get_group_id(the_config.tracing_group_name.value, true, &tracing_group_id);
+
+	if (get_group_id_ret < 0) {
+		return false;
+	}
+
+	DBG_FMT("Validating user membership of tracing group: uid={}, primary_group={}, "
+		"tracing_group_id={}",
+		uid,
+		primary_group,
+		tracing_group_id);
+	return is_user_part_of_group(uid, primary_group, tracing_group_id);
+}
+
+void set_thread_status(bool running)
 {
 	DBG("Marking client thread's state as %s", running ? "running" : "error");
 	thread_state.running = running;
 	sem_post(&thread_state.ready);
 }
 
-static bool wait_thread_status()
+bool wait_thread_status()
 {
 	DBG("Waiting for client thread to be ready");
 	sem_wait(&thread_state.ready);
@@ -90,18 +199,17 @@ static bool wait_thread_status()
  * right amount of memory and copying the original information from the lsm
  * structure.
  */
-static void setup_lttng_msg(struct command_ctx *cmd_ctx,
-			    const void *payload_buf,
-			    size_t payload_len,
-			    const void *cmd_header_buf,
-			    size_t cmd_header_len)
+void setup_lttng_msg(struct command_ctx *cmd_ctx,
+		     const void *payload_buf,
+		     size_t payload_len,
+		     const void *cmd_header_buf,
+		     size_t cmd_header_len)
 {
 	const auto header_len = sizeof(struct lttcomm_lttng_msg);
 	const auto total_msg_size = header_len + cmd_header_len + payload_len;
 	lttcomm_lttng_msg llm{};
 
 	llm.cmd_type = cmd_ctx->lsm.cmd_type;
-	llm.pid = (uint32_t) cmd_ctx->lsm.domain.attr.pid;
 	llm.cmd_header_size = (uint32_t) cmd_header_len;
 	llm.data_size = (uint32_t) payload_len;
 
@@ -138,7 +246,7 @@ static void setup_lttng_msg(struct command_ctx *cmd_ctx,
 	}
 }
 
-static void setup_empty_lttng_msg(struct command_ctx *cmd_ctx)
+void setup_empty_lttng_msg(struct command_ctx *cmd_ctx)
 {
 	const struct lttcomm_lttng_msg llm = {};
 
@@ -154,7 +262,7 @@ static void setup_empty_lttng_msg(struct command_ctx *cmd_ctx)
 	cmd_ctx->lttng_msg_size = sizeof(llm);
 }
 
-static void update_lttng_msg(struct command_ctx *cmd_ctx, size_t cmd_header_len, size_t payload_len)
+void update_lttng_msg(struct command_ctx *cmd_ctx, size_t cmd_header_len, size_t payload_len)
 {
 	const size_t header_len = sizeof(struct lttcomm_lttng_msg);
 	const size_t total_msg_size = header_len + cmd_header_len + payload_len;
@@ -162,7 +270,6 @@ static void update_lttng_msg(struct command_ctx *cmd_ctx, size_t cmd_header_len,
 	lttcomm_lttng_msg llm{};
 
 	llm.cmd_type = cmd_ctx->lsm.cmd_type;
-	llm.pid = (uint32_t) cmd_ctx->lsm.domain.attr.pid;
 	llm.cmd_header_size = (uint32_t) cmd_header_len;
 	llm.data_size = (uint32_t) payload_len;
 
@@ -180,7 +287,7 @@ static void update_lttng_msg(struct command_ctx *cmd_ctx, size_t cmd_header_len,
  * Start the thread_manage_consumer. This must be done after a lttng-consumerd
  * exec or it will fail.
  */
-static int spawn_consumer_thread(struct consumer_data *consumer_data)
+int spawn_consumer_thread(struct consumer_data *consumer_data)
 {
 	return launch_consumer_management_thread(consumer_data) ? 0 : -1;
 }
@@ -190,7 +297,7 @@ static int spawn_consumer_thread(struct consumer_data *consumer_data)
  *
  * Return pid if successful else -1.
  */
-static pid_t spawn_consumerd(struct consumer_data *consumer_data)
+pid_t spawn_consumerd(struct consumer_data *consumer_data)
 {
 	int ret;
 	pid_t pid;
@@ -364,7 +471,7 @@ error:
 /*
  * Spawn the consumerd daemon and session daemon thread.
  */
-static int start_consumerd(struct consumer_data *consumer_data)
+int start_consumerd(struct consumer_data *consumer_data)
 {
 	int ret;
 
@@ -424,7 +531,7 @@ error:
  * function also applies the right modification on a per domain basis for the
  * trace files destination directory.
  */
-static int copy_session_consumer(int domain, const ltt_session::locked_ref& session)
+int copy_session_consumer(int domain, const ltt_session::locked_ref& session)
 {
 	int ret;
 	const char *dir_name;
@@ -484,8 +591,7 @@ error:
 /*
  * Create an UST session and add it to the session ust list.
  */
-static int create_ust_session(const ltt_session::locked_ref& session,
-			      const struct lttng_domain *domain)
+int create_ust_session(const ltt_session::locked_ref& session, const struct lttng_domain *domain)
 {
 	int ret;
 	struct ltt_ust_session *lus = nullptr;
@@ -527,6 +633,16 @@ static int create_ust_session(const ltt_session::locked_ref& session,
 		lus->shm_path[sizeof(lus->shm_path) - 1] = '\0';
 		strncat(lus->shm_path, "/ust", sizeof(lus->shm_path) - strlen(lus->shm_path) - 1);
 	}
+
+	if (!lus->supports_madv_remove()) {
+		WARN_FMT(
+			"File-system containing shared memory location for UST session does not support "
+			"MADV_REMOVE; memory reclamation won't be possible for this session: "
+			"session_name=`{}`, shm_path=`{}`",
+			session->name,
+			session->shm_path);
+	}
+
 	/* Copy session output to the newly created UST session */
 	ret = copy_session_consumer(domain->type, session);
 	if (ret != LTTNG_OK) {
@@ -544,7 +660,7 @@ error:
 /*
  * Create a kernel tracer session then create the default channel.
  */
-static int create_kernel_session(const ltt_session::locked_ref& session)
+int create_kernel_session(const ltt_session::locked_ref& session)
 {
 	int ret;
 
@@ -552,7 +668,14 @@ static int create_kernel_session(const ltt_session::locked_ref& session)
 
 	ret = kernel_create_session(session);
 	if (ret < 0) {
-		ret = LTTNG_ERR_KERN_SESS_FAIL;
+		/* Handle specific errno values */
+		if (ret == -ENOSYS && session->trace_format == LTTNG_TRACE_FORMAT_CTF_2) {
+			/* Kernel doesn't support CTF 2 format */
+			ret = LTTNG_ERR_UNSUPPORTED_TRACE_FORMAT;
+		} else {
+			/* Generic kernel session failure */
+			ret = LTTNG_ERR_KERN_SESS_FAIL;
+		}
 		goto error_create;
 	}
 
@@ -583,7 +706,7 @@ error_create:
 /*
  * Count number of session permitted by uid/gid.
  */
-static unsigned int lttng_sessions_count(uid_t uid, gid_t gid __attribute__((unused)))
+unsigned int lttng_sessions_count(uid_t uid, gid_t gid __attribute__((unused)))
 {
 	unsigned int i = 0;
 	const struct ltt_session_list *session_list = session_get_list();
@@ -607,8 +730,7 @@ static unsigned int lttng_sessions_count(uid_t uid, gid_t gid __attribute__((unu
 	return i;
 }
 
-static lttng::ctl::trigger
-receive_lttng_trigger(struct command_ctx *cmd_ctx, int sock, int *sock_error)
+lttng::ctl::trigger receive_lttng_trigger(struct command_ctx *cmd_ctx, int sock, int *sock_error)
 {
 	int ret;
 	size_t trigger_len;
@@ -672,10 +794,10 @@ receive_lttng_trigger(struct command_ctx *cmd_ctx, int sock, int *sock_error)
 	return lttng::ctl::trigger(trigger);
 }
 
-static enum lttng_error_code receive_lttng_error_query(struct command_ctx *cmd_ctx,
-						       int sock,
-						       int *sock_error,
-						       struct lttng_error_query **_query)
+enum lttng_error_code receive_lttng_error_query(struct command_ctx *cmd_ctx,
+						int sock,
+						int *sock_error,
+						struct lttng_error_query **_query)
 {
 	int ret;
 	size_t query_len;
@@ -741,14 +863,14 @@ end:
 	return ret_code;
 }
 
-static enum lttng_error_code receive_lttng_event(struct command_ctx *cmd_ctx,
-						 int sock,
-						 int *sock_error,
-						 struct lttng_event **out_event,
-						 char **out_filter_expression,
-						 struct lttng_bytecode **out_bytecode,
-						 struct lttng_event_exclusion **out_exclusion,
-						 lttng::event_rule_uptr& event_rule)
+enum lttng_error_code receive_lttng_event(struct command_ctx *cmd_ctx,
+					  int sock,
+					  int *sock_error,
+					  struct lttng_event **out_event,
+					  char **out_filter_expression,
+					  struct lttng_bytecode **out_bytecode,
+					  struct lttng_event_exclusion **out_exclusion,
+					  lttng::ctl::event_rule_uptr& event_rule)
 {
 	int ret;
 	size_t event_len;
@@ -868,11 +990,10 @@ end:
 	return ret_code;
 }
 
-static enum lttng_error_code
-receive_lttng_event_context(const struct command_ctx *cmd_ctx,
-			    int sock,
-			    int *sock_error,
-			    struct lttng_event_context **out_event_context)
+enum lttng_error_code receive_lttng_event_context(const struct command_ctx *cmd_ctx,
+						  int sock,
+						  int *sock_error,
+						  struct lttng_event_context **out_event_context)
 {
 	int ret;
 	const size_t event_context_len = (size_t) cmd_ctx->lsm.u.context.length;
@@ -934,8 +1055,9 @@ end:
 /*
  * Version of setup_lttng_msg() without command header.
  */
-static void
-setup_lttng_msg_no_cmd_header(struct command_ctx *cmd_ctx, void *payload_buf, size_t payload_len)
+void setup_lttng_msg_no_cmd_header(struct command_ctx *cmd_ctx,
+				   const void *payload_buf,
+				   size_t payload_len)
 {
 	setup_lttng_msg(cmd_ctx, payload_buf, payload_len, nullptr, 0);
 }
@@ -944,7 +1066,7 @@ setup_lttng_msg_no_cmd_header(struct command_ctx *cmd_ctx, void *payload_buf, si
  * Check if the current kernel tracer supports the session rotation feature.
  * Return 1 if it does, 0 otherwise.
  */
-static int check_rotate_compatible()
+int check_rotate_compatible()
 {
 	int ret = 1;
 
@@ -961,7 +1083,7 @@ static int check_rotate_compatible()
  *
  * Return lttcomm error code.
  */
-static int send_unix_sock(int sock, struct lttng_payload_view *view)
+int send_unix_sock(int sock, struct lttng_payload_view *view)
 {
 	int ret;
 	const int fd_count = lttng_payload_view_get_fd_handle_count(view);
@@ -988,12 +1110,73 @@ end:
 	return ret;
 }
 
-static void command_ctx_set_status_code(command_ctx& cmd_ctx, enum lttng_error_code status_code)
+void command_ctx_set_status_code(command_ctx& cmd_ctx, enum lttng_error_code status_code)
 {
 	LTTNG_ASSERT(cmd_ctx.reply_payload.buffer.size >= sizeof(lttcomm_lttng_msg));
 	((struct lttcomm_lttng_msg *) (cmd_ctx.reply_payload.buffer.data))->ret_code = status_code;
 }
 
+lttng_data_stream_info_sets lttng_data_stream_info_sets_create_from_memory_usage_groups(
+	const std::vector<lttng::sessiond::commands::stream_memory_usage_group>& groups)
+{
+	lttng_data_stream_info_sets sets;
+
+	sets.sets.reserve(groups.size());
+
+	for (const auto& group : groups) {
+		lttng_data_stream_info_set set{};
+
+		/* Set owner information */
+		switch (group.owner.owner_type) {
+		case lttng::sessiond::commands::stream_group_owner::type::USER:
+			set.is_per_pid = false;
+			set.owner.uid = group.owner.id.uid;
+			break;
+		case lttng::sessiond::commands::stream_group_owner::type::PROCESS:
+			set.is_per_pid = true;
+			set.owner.pid = group.owner.id.pid;
+			break;
+		default:
+			/* Skip unsupported types */
+			continue;
+		}
+
+		/* Set bitness */
+		switch (group.owner.bitness) {
+		case lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness::ABI_32:
+			set.bitness = LTTNG_APP_BITNESS_32;
+			break;
+		case lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness::ABI_64:
+			set.bitness = LTTNG_APP_BITNESS_64;
+			break;
+		default:
+			std::abort();
+		}
+
+		set.streams.reserve(group.streams_memory_usage.size());
+
+		/* Convert each stream */
+		for (const auto& stream : group.streams_memory_usage) {
+			lttng_data_stream_info stream_info;
+
+			/* Set CPU ID if available */
+			if (stream.id.cpu_id.has_value()) {
+				stream_info.cpu_id = *stream.id.cpu_id;
+			}
+
+			/* Set memory usage (use logical size) */
+			stream_info.memory_usage = stream.size_bytes.physical;
+			stream_info.max_memory_usage = stream.size_bytes.logical;
+
+			set.streams.emplace_back(std::move(stream_info));
+		}
+
+		sets.sets.emplace_back(std::move(set));
+	}
+
+	return sets;
+}
+
 /*
  * Process the command requested by the lttng client within the command
  * context structure. This function make sure that the return structure (llm)
@@ -1005,7 +1188,7 @@ static void command_ctx_set_status_code(command_ctx& cmd_ctx, enum lttng_error_c
  * A command may assume the ownership of the socket, in which case its value
  * should be set to -1.
  */
-static int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_error)
+int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_error)
 {
 	int ret = LTTNG_OK;
 	bool need_tracing_session = true;
@@ -1049,6 +1232,7 @@ static int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_
 	case LTTCOMM_SESSIOND_COMMAND_LIST_TRIGGERS:
 	case LTTCOMM_SESSIOND_COMMAND_EXECUTE_ERROR_QUERY:
 	case LTTCOMM_SESSIOND_COMMAND_KERNEL_TRACER_STATUS:
+	case LTTCOMM_SESSIOND_COMMAND_RECLAIM_CHANNEL_MEMORY:
 		need_domain = false;
 		break;
 	default:
@@ -1113,6 +1297,8 @@ static int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_
 	case LTTCOMM_SESSIOND_COMMAND_REGISTER_TRIGGER:
 	case LTTCOMM_SESSIOND_COMMAND_LIST_TRIGGERS:
 	case LTTCOMM_SESSIOND_COMMAND_EXECUTE_ERROR_QUERY:
+	case LTTCOMM_SESSIOND_COMMAND_RECLAIM_CHANNEL_MEMORY:
+	case LTTCOMM_SESSIOND_COMMAND_GET_CHANNEL_DATA_STREAM_INFO_SETS:
 		break;
 	default:
 		/* Setup lttng message with no payload */
@@ -1171,10 +1357,13 @@ static int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_
 		try {
 			target_session.emplace(
 				ltt_session::find_locked_session(cmd_ctx->lsm.session.name));
+		} catch (const lttng::sessiond::exceptions::session_not_found_error& ex) {
+			return LTTNG_ERR_SESS_NOT_FOUND;
 		} catch (...) {
 			std::throw_with_nested(lttng::ctl::error(
 				fmt::format(
-					"Target session of command doesn't exist: command='{}'",
+					"Failed to get target session: session='{}', command='{}'",
+					(const char *) cmd_ctx->lsm.session.name,
 					lttcomm_sessiond_command_str(
 						(lttcomm_sessiond_command) cmd_ctx->lsm.cmd_type)),
 				LTTNG_ERR_SESS_NOT_FOUND,
@@ -1252,7 +1441,9 @@ static int process_client_msg(struct command_ctx *cmd_ctx, int *sock, int *sock_
 			if ((*target_session)->kernel_session == nullptr) {
 				ret = create_kernel_session(*target_session);
 				if (ret != LTTNG_OK) {
-					ret = LTTNG_ERR_KERN_SESS_FAIL;
+					if (ret != LTTNG_ERR_UNSUPPORTED_TRACE_FORMAT) {
+						ret = LTTNG_ERR_KERN_SESS_FAIL;
+					}
 					goto error;
 				}
 			}
@@ -1661,7 +1852,7 @@ skip_domain:
 		char *filter_expression;
 		struct lttng_event_exclusion *exclusions;
 		struct lttng_bytecode *bytecode;
-		lttng::event_rule_uptr event_rule;
+		lttng::ctl::event_rule_uptr event_rule;
 		const enum lttng_error_code ret_code = receive_lttng_event(cmd_ctx,
 									   *sock,
 									   sock_error,
@@ -2056,13 +2247,11 @@ skip_domain:
 		ret = LTTNG_OK;
 		break;
 	}
+
 	case LTTCOMM_SESSIOND_COMMAND_SNAPSHOT_RECORD:
 	{
 		const lttng_snapshot_output output = cmd_ctx->lsm.u.snapshot_record.output;
-		ret = cmd_snapshot_record(*target_session, &output, 0); // RFC: set to zero
-									// since it's ignored
-									// by
-									// cmd_snapshot_record
+		ret = cmd_snapshot_record(*target_session, &output);
 		break;
 	}
 	case LTTCOMM_SESSIOND_COMMAND_CREATE_SESSION_EXT:
@@ -2194,6 +2383,44 @@ skip_domain:
 		ret = LTTNG_OK;
 		break;
 	}
+	case LTTCOMM_SESSIOND_COMMAND_RECLAIM_CHANNEL_MEMORY:
+		cmd_reclaim_channel_memory(*target_session, cmd_ctx, sock);
+		ret = LTTNG_OK;
+		break;
+	case LTTCOMM_SESSIOND_COMMAND_GET_CHANNEL_DATA_STREAM_INFO_SETS:
+	{
+		DBG("Client get channel data stream info sets \"%s\"", (*target_session)->name);
+
+		/* Validate that channel_name is null-terminated */
+		const auto channel_name =
+			cmd_ctx->lsm.u.get_channel_data_stream_info_sets.channel_name;
+		if (strnlen(channel_name,
+			    sizeof(cmd_ctx->lsm.u.get_channel_data_stream_info_sets.channel_name)) ==
+		    sizeof(cmd_ctx->lsm.u.get_channel_data_stream_info_sets.channel_name)) {
+			LTTNG_THROW_INVALID_ARGUMENT_ERROR("Channel name is not null-terminated");
+		}
+
+		const auto domain =
+			lttng::get_domain_class_from_lttng_domain_type(cmd_ctx->lsm.domain.type);
+
+		const auto results = lttng::sessiond::commands::get_channel_memory_usage(
+			*target_session, domain, lttng::c_string_view(channel_name));
+
+		/* Convert results to lttng_data_stream_info_sets using helper function */
+		lttng_data_stream_info_sets sets =
+			lttng_data_stream_info_sets_create_from_memory_usage_groups(results);
+
+		setup_empty_lttng_msg(cmd_ctx);
+		const auto original_payload_size = cmd_ctx->reply_payload.buffer.size;
+		sets.serialize(cmd_ctx->reply_payload);
+		const auto payload_size =
+			cmd_ctx->reply_payload.buffer.size - original_payload_size;
+
+		update_lttng_msg(cmd_ctx, 0, payload_size);
+
+		ret = LTTNG_OK;
+		break;
+	}
 	case LTTCOMM_SESSIOND_COMMAND_ROTATION_GET_INFO:
 	{
 		struct lttng_rotation_get_info_return get_info_return;
@@ -2347,7 +2574,7 @@ error:
 	return ret;
 }
 
-static int create_client_sock()
+int create_client_sock()
 {
 	int ret, client_sock;
 
@@ -2383,19 +2610,22 @@ end:
 	return ret;
 }
 
-static void cleanup_client_thread(void *data)
+void cleanup_client_thread(void *data)
 {
 	struct lttng_pipe *quit_pipe = (lttng_pipe *) data;
 
 	lttng_pipe_destroy(quit_pipe);
 }
 
-static void thread_init_cleanup(void *data __attribute__((unused)))
+void thread_init_cleanup(void *data __attribute__((unused)))
 {
 	set_thread_status(false);
 }
 
-// Helper function to log the source_location if the exception is derived from lttng::runtime_error
+/*
+ * Helper function to log the source_location if the exception is derived from
+ * lttng::runtime_error.
+ */
 template <typename ExceptionType>
 typename std::enable_if<std::is_base_of<lttng::runtime_error, ExceptionType>::value,
 			std::string>::type
@@ -2413,30 +2643,35 @@ formatted_source_location(const ExceptionType&)
 }
 
 template <class ExceptionType>
-static void log_nested_exceptions(const ExceptionType& ex, unsigned int level = 0)
+void log_nested_exceptions(const ExceptionType& ex,
+			   lttng_error_level log_level = PRINT_WARN,
+			   unsigned int level = 0)
 {
 	const auto location = formatted_source_location(ex);
 
 	if (level == 0) {
 		if (location.size()) {
-			WARN_FMT("Client request failed: {}, location='{}'", ex.what(), location);
+			LOG_FMT(log_level,
+				"Client request failed: {}, location='{}'",
+				ex.what(),
+				location);
 		} else {
-			WARN_FMT("Client request failed: {}", ex.what());
+			LOG_FMT(log_level, "Client request failed: {}", ex.what());
 		}
 	} else {
 		if (location.size()) {
-			WARN_FMT("\t{}, location='{}'", ex.what(), location);
+			LOG_FMT(log_level, "\t{}, location='{}'", ex.what(), location);
 		} else {
-			WARN_FMT("\t{}", ex.what());
+			LOG_FMT(log_level, "\t{}", ex.what());
 		}
 	}
 
 	try {
 		std::rethrow_if_nested(ex);
 	} catch (const lttng::runtime_error& nested_ex) {
-		log_nested_exceptions(nested_ex, level + 1);
+		log_nested_exceptions(nested_ex, log_level, level + 1);
 	} catch (const std::exception& nested_ex) {
-		log_nested_exceptions(nested_ex, level + 1);
+		log_nested_exceptions(nested_ex, log_level, level + 1);
 	}
 }
 
@@ -2444,7 +2679,7 @@ static void log_nested_exceptions(const ExceptionType& ex, unsigned int level =
  * This thread manage all clients request using the unix client socket for
  * communication.
  */
-static void *thread_manage_clients(void *data)
+void *thread_manage_clients(void *data)
 {
 	int sock = -1, ret, i, err = -1;
 	int sock_error;
@@ -2607,10 +2842,8 @@ static void *thread_manage_clients(void *data)
 
 		health_code_update();
 
-		// TODO: Validate cmd_ctx including sanity check for
-		// security purpose.
-
 		rcu_thread_online();
+
 		/*
 		 * This function dispatch the work to the kernel or userspace tracer
 		 * libs and fill the lttcomm_lttng_msg data structure of all the needed
@@ -2618,8 +2851,22 @@ static void *thread_manage_clients(void *data)
 		 * everything this function may needs.
 		 */
 		try {
-			ret = process_client_msg(&cmd_ctx, &sock, &sock_error);
-			rcu_thread_offline();
+			/*
+			 * Check if the client has the right to execute this command.
+			 * If the client is root, it can do anything. If the client is not
+			 * root, it must be in the tracing group or have the same UID as the
+			 * sessiond's UID.
+			 */
+			if ((is_root &&
+			     is_user_in_tracing_group(cmd_ctx.creds.uid, cmd_ctx.creds.gid)) ||
+			    (getuid() == cmd_ctx.creds.uid) || cmd_ctx.creds.uid == 0) {
+				ret = process_client_msg(&cmd_ctx, &sock, &sock_error);
+			} else {
+				WARN_FMT(
+					"Client doesn't have permission to interact with this instance: uid={}",
+					cmd_ctx.creds.uid);
+				ret = LTTNG_ERR_EPERM;
+			}
 		} catch (const std::bad_alloc& ex) {
 			log_nested_exceptions(ex);
 			ret = LTTNG_ERR_NOMEM;
@@ -2629,6 +2876,9 @@ static void *thread_manage_clients(void *data)
 		} catch (const lttng::invalid_argument_error& ex) {
 			log_nested_exceptions(ex);
 			ret = LTTNG_ERR_INVALID;
+		} catch (const lttng::unsupported_error& ex) {
+			log_nested_exceptions(ex);
+			ret = LTTNG_ERR_NOT_SUPPORTED;
 		} catch (const lttng::sessiond::exceptions::session_not_found_error& ex) {
 			log_nested_exceptions(ex);
 			ret = LTTNG_ERR_SESS_NOT_FOUND;
@@ -2643,6 +2893,8 @@ static void *thread_manage_clients(void *data)
 			ret = LTTNG_ERR_UNK;
 		}
 
+		rcu_thread_offline();
+
 		if (ret < LTTNG_OK || ret >= LTTNG_ERR_NR) {
 			WARN("Command returned an invalid status code, returning unknown error: "
 			     "command type = %s (%d), ret = %d",
@@ -2655,8 +2907,8 @@ static void *thread_manage_clients(void *data)
 
 		if (ret != LTTNG_OK) {
 			/*
-			 * Reset the payload contents as the command may have left them in an
-			 * inconsistent state.
+			 * Reset the payload contents as the command may have left them in
+			 * an inconsistent state.
 			 */
 			setup_empty_lttng_msg(&cmd_ctx);
 		}
@@ -2738,13 +2990,14 @@ error_create_poll:
 	return nullptr;
 }
 
-static bool shutdown_client_thread(void *thread_data)
+bool shutdown_client_thread(void *thread_data)
 {
 	struct lttng_pipe *client_quit_pipe = (lttng_pipe *) thread_data;
 	const int write_fd = lttng_pipe_get_writefd(client_quit_pipe);
 
 	return notify_thread_pipe(write_fd) == 1;
 }
+} /* namespace */
 
 struct lttng_thread *launch_client_thread()
 {
