@@ -7,13 +7,17 @@
  */
 
 #define _LGPL_SOURCE
+#include "common/exception.hpp"
+#include "common/runas.hpp"
 #include "consumer-output.hpp"
 #include "consumer.hpp"
 #include "health-sessiond.hpp"
 #include "lttng-sessiond.hpp"
+#include "recording-channel-configuration.hpp"
 #include "ust-app.hpp"
 #include "utils.hpp"
 
+#include <common/binary-view.hpp>
 #include <common/common.hpp>
 #include <common/defaults.hpp>
 #include <common/relayd/relayd.hpp>
@@ -29,6 +33,62 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+namespace lsc = lttng::sessiond::consumer;
+
+namespace {
+std::vector<std::uint8_t> consumer_request(consumer_socket& socket,
+					   lttng_consumer_command command_id,
+					   std::vector<std::uint8_t> payload)
+{
+	const auto send_ret = consumer_socket_send(&socket, payload.data(), payload.size());
+	if (send_ret < 0) {
+		LTTNG_THROW_POSIX("Failed to send command payload to consumer daemon", errno);
+	}
+
+	auto reply = std::move(payload);
+	reply.resize(0);
+
+	lttcomm_consumer_status_msg reply_header;
+
+	const auto recv_reply_header_ret =
+		consumer_socket_recv(&socket, &reply_header, sizeof(reply_header));
+	if (recv_reply_header_ret < 0) {
+		if (errno == 0) {
+			/* Our peer closed the connection unexpectedly. */
+			errno = ECONNRESET;
+		}
+
+		LTTNG_THROW_POSIX("Failed to receive command reply header from consumer daemon",
+				  errno);
+	}
+
+	const auto status_code = static_cast<lttcomm_return_code>(reply_header.ret_code);
+	const auto payload_size = reply_header.payload_size;
+	DBG_FMT("Received command reply from consumer daemon: status_code={}, payload_size={}",
+		status_code,
+		payload_size);
+
+	if (status_code != LTTCOMM_CONSUMERD_SUCCESS) {
+		LTTNG_THROW_CONSUMERD_COMMAND_ERROR(
+			fmt::format("Consumer command failed: command={}", command_id),
+			status_code);
+	}
+
+	if (reply_header.payload_size == 0) {
+		return reply;
+	}
+
+	reply.resize(reply_header.payload_size);
+	const auto recv_payload_ret = consumer_socket_recv(&socket, reply.data(), reply.size());
+	if (recv_payload_ret < 0) {
+		LTTNG_THROW_POSIX("Failed to receive command reply payload from consumer daemon",
+				  errno);
+	}
+
+	return reply;
+}
+} /* namespace */
+
 /*
  * Return allocated full pathname of the session using the consumer trace path
  * and subdir if available.
@@ -140,7 +200,7 @@ error:
  *
  * Return 0 on success else a negative value on error.
  */
-int consumer_socket_recv(struct consumer_socket *socket, void *msg, size_t len)
+int consumer_socket_recv(consumer_socket *socket, void *msg, size_t len)
 {
 	int fd;
 	ssize_t size;
@@ -892,34 +952,41 @@ error:
  * Populate the given consumer msg structure with the ask_channel command
  * information.
  */
-void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
-					uint64_t subbuf_size,
-					uint64_t num_subbuf,
-					int overwrite,
-					unsigned int switch_timer_interval,
-					unsigned int read_timer_interval,
-					unsigned int live_timer_interval,
-					bool is_in_live_session,
-					unsigned int monitor_timer_interval,
-					int output,
-					int type,
-					uint64_t session_id,
-					const char *pathname,
-					const char *name,
-					uint64_t relayd_id,
-					uint64_t key,
-					const lttng_uuid& uuid,
-					uint32_t chan_id,
-					uint64_t tracefile_size,
-					uint64_t tracefile_count,
-					uint64_t session_id_per_pid,
-					unsigned int monitor,
-					uint32_t ust_app_uid,
-					int64_t blocking_timeout,
-					const char *root_shm_path,
-					const char *shm_path,
-					struct lttng_trace_chunk *trace_chunk,
-					const struct lttng_credentials *buffer_credentials)
+void consumer_init_ask_channel_comm_msg(
+	struct lttcomm_consumer_msg *msg,
+	uint64_t subbuf_size,
+	uint64_t num_subbuf,
+	int overwrite,
+	unsigned int switch_timer_interval,
+	unsigned int read_timer_interval,
+	unsigned int live_timer_interval,
+	bool is_in_live_session,
+	bool continuously_reclaimed,
+	unsigned int monitor_timer_interval,
+	nonstd::optional<uint64_t> watchdog_timer_interval,
+	int output,
+	int type,
+	uint64_t session_id,
+	const char *pathname,
+	const char *name,
+	uint64_t relayd_id,
+	uint64_t key,
+	const lttng_uuid& uuid,
+	uint32_t chan_id,
+	uint64_t tracefile_size,
+	uint64_t tracefile_count,
+	uint64_t session_id_per_pid,
+	unsigned int monitor,
+	uint32_t ust_app_uid,
+	int64_t blocking_timeout,
+	lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t
+		preallocation_policy,
+	const nonstd::optional<std::chrono::microseconds>& automatic_memory_reclamation_maximal_age,
+	const char *root_shm_path,
+	const char *shm_path,
+	struct lttng_trace_chunk *trace_chunk,
+	const struct lttng_credentials *buffer_credentials,
+	enum lttng_trace_format trace_format)
 {
 	LTTNG_ASSERT(msg);
 
@@ -947,7 +1014,14 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.ask_channel.read_timer_interval = read_timer_interval;
 	msg->u.ask_channel.live_timer_interval = live_timer_interval;
 	msg->u.ask_channel.is_live = is_in_live_session;
+	msg->u.ask_channel.continuously_reclaimed = continuously_reclaimed;
 	msg->u.ask_channel.monitor_timer_interval = monitor_timer_interval;
+
+	if (watchdog_timer_interval) {
+		LTTNG_OPTIONAL_SET(&msg->u.ask_channel.watchdog_timer_interval,
+				   watchdog_timer_interval.value());
+	}
+
 	msg->u.ask_channel.output = output;
 	msg->u.ask_channel.type = type;
 	msg->u.ask_channel.session_id = session_id;
@@ -960,6 +1034,15 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.ask_channel.monitor = monitor;
 	msg->u.ask_channel.ust_app_uid = ust_app_uid;
 	msg->u.ask_channel.blocking_timeout = blocking_timeout;
+	msg->u.ask_channel.preallocate_backing = preallocation_policy ==
+		lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t::
+			PREALLOCATE;
+
+	if (automatic_memory_reclamation_maximal_age) {
+		LTTNG_OPTIONAL_SET(
+			&msg->u.ask_channel.automatic_memory_reclamation_maximal_age_us,
+			static_cast<uint64_t>(automatic_memory_reclamation_maximal_age->count()));
+	}
 
 	std::copy(uuid.begin(), uuid.end(), msg->u.ask_channel.uuid);
 
@@ -986,6 +1069,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 			sizeof(msg->u.ask_channel.shm_path) - 1);
 		msg->u.ask_channel.shm_path[sizeof(msg->u.ask_channel.shm_path) - 1] = '\0';
 	}
+	msg->u.ask_channel.trace_format = static_cast<uint8_t>(trace_format);
 }
 
 /*
@@ -1006,7 +1090,8 @@ void consumer_init_add_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 					unsigned int live_timer_interval,
 					bool is_in_live_session,
 					unsigned int monitor_timer_interval,
-					struct lttng_trace_chunk *trace_chunk)
+					struct lttng_trace_chunk *trace_chunk,
+					enum lttng_trace_format trace_format)
 {
 	LTTNG_ASSERT(msg);
 
@@ -1036,6 +1121,7 @@ void consumer_init_add_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.channel.live_timer_interval = live_timer_interval;
 	msg->u.channel.is_live = is_in_live_session;
 	msg->u.channel.monitor_timer_interval = monitor_timer_interval;
+	msg->u.channel.trace_format = static_cast<uint8_t>(trace_format);
 
 	strncpy(msg->u.channel.pathname, pathname, sizeof(msg->u.channel.pathname));
 	msg->u.channel.pathname[sizeof(msg->u.channel.pathname) - 1] = '\0';
@@ -1124,7 +1210,8 @@ int consumer_send_relayd_socket(struct consumer_socket *consumer_sock,
 				int session_live_timer,
 				const uint64_t *current_chunk_id,
 				time_t session_creation_time,
-				bool session_name_contains_creation_time)
+				bool session_name_contains_creation_time,
+				enum lttng_trace_format trace_format)
 {
 	int ret;
 	int fd;
@@ -1158,6 +1245,7 @@ int consumer_send_relayd_socket(struct consumer_socket *consumer_sock,
 					    current_chunk_id,
 					    session_creation_time,
 					    session_name_contains_creation_time,
+					    trace_format,
 					    output_path);
 		if (ret < 0) {
 			/* Close the control socket. */
@@ -1771,6 +1859,205 @@ error_socket:
 	return ret;
 }
 
+std::vector<lsc::channel_memory_usage>
+lsc::get_channels_memory_usage(consumer_socket& socket,
+			       const std::vector<std::uint64_t>& channel_keys)
+{
+	LTTNG_ASSERT(!channel_keys.empty());
+
+	lttcomm_consumer_msg header = {
+		.cmd_type = LTTNG_CONSUMER_GET_CHANNELS_MEMORY_USAGE,
+		.u = {},
+	};
+
+	header.u.get_channels_memory_usage.key_count = channel_keys.size();
+
+	health_code_update();
+
+	const lttng::pthread::lock_guard socket_lock(*socket.lock);
+
+	std::vector<std::uint8_t> request_payload;
+	request_payload.reserve(sizeof(header) + channel_keys.size() * sizeof(uint64_t));
+
+	/* Append header to the payload. */
+	request_payload.insert(request_payload.end(),
+			       reinterpret_cast<const std::uint8_t *>(&header),
+			       reinterpret_cast<const std::uint8_t *>(&header) + sizeof(header));
+
+	/* Append channel keys to the payload (64-bit each). */
+	request_payload.insert(
+		request_payload.end(),
+		reinterpret_cast<const std::uint8_t *>(channel_keys.data()),
+		reinterpret_cast<const std::uint8_t *>(channel_keys.data() + channel_keys.size()));
+
+	auto reply = consumer_request(
+		socket, LTTNG_CONSUMER_GET_CHANNELS_MEMORY_USAGE, std::move(request_payload));
+
+	health_code_update();
+
+	/*
+	 * The expected reply format is:
+	 *   [lttcomm_consumer_channel_memory_usage_reply_header] (announces the count of stream
+	 *                                                         memory usage)
+	 *   [lttcomm_stream_memory_usage]
+	 *   [lttcomm_stream_memory_usage]
+	 *   [...]
+	 *
+	 * The stream memory usage structures are delivered in the same order as the channel keys
+	 * were sent in the request. For each channel, the streams are grouped together, and their
+	 * order matches the CPU number: the first stream for a given channel corresponds to CPU #0,
+	 * the second to CPU #1, and so on.
+	 */
+	if (reply.size() < sizeof(lttcomm_consumer_channel_memory_usage_reply_header)) {
+		LTTNG_THROW_PROTOCOL_ERROR(fmt::format("Consumer reply is too short: command={}",
+						       LTTNG_CONSUMER_GET_CHANNELS_MEMORY_USAGE));
+	}
+
+	const auto stream_info_count = [reply]() {
+		const auto& mem_usage_header = *reinterpret_cast<
+			const lttcomm_consumer_channel_memory_usage_reply_header *>(reply.data());
+
+		return mem_usage_header.count;
+	}();
+
+	/* Build a view over the stream memory usage structures. */
+	const lttng::binary_view<lttcomm_stream_memory_usage> stream_usage_infos(
+		reply.data() + sizeof(lttcomm_consumer_channel_memory_usage_reply_header),
+		reply.size() - sizeof(lttcomm_consumer_channel_memory_usage_reply_header),
+		stream_info_count);
+
+	std::vector<lsc::channel_memory_usage> result;
+	result.resize(channel_keys.size());
+
+	std::size_t current_channel_index = 0;
+	auto current_channel_key = channel_keys[0];
+	for (const auto& stream_info : stream_usage_infos) {
+		if (stream_info.channel_key != current_channel_key) {
+			/* First stream of a new channel; change the current channel index. */
+			current_channel_index++;
+			current_channel_key = stream_info.channel_key;
+
+			if (current_channel_index == channel_keys.size()) {
+				LTTNG_THROW_PROTOCOL_ERROR(fmt::format(
+					"Consumer reply contains streams of too many channels: expected={}",
+					channel_keys.size()));
+			}
+		}
+
+		result[current_channel_index].streams_memory_usage.emplace_back(
+			stream_info.logical_size_bytes, stream_info.physical_size_bytes);
+	}
+
+	return result;
+}
+
+std::vector<lsc::stream_memory_reclamation_result_group> lsc::reclaim_channels_memory(
+	consumer_socket& socket,
+	const std::vector<std::uint64_t>& channel_keys,
+	const nonstd::optional<std::chrono::microseconds>& reclaim_older_than_age,
+	bool require_consumed,
+	std::uint64_t memory_reclaim_request_token)
+{
+	LTTNG_ASSERT(!channel_keys.empty());
+
+	lttcomm_consumer_msg header = {
+		.cmd_type = LTTNG_CONSUMER_RECLAIM_CHANNELS_MEMORY,
+		.u = {},
+	};
+
+	header.u.reclaim_channels_memory.key_count = channel_keys.size();
+	header.u.reclaim_channels_memory.require_consumed = require_consumed;
+	header.u.reclaim_channels_memory.memory_reclaim_request_token =
+		memory_reclaim_request_token;
+	if (reclaim_older_than_age) {
+		LTTNG_OPTIONAL_SET(&header.u.reclaim_channels_memory.age_limit_us,
+				   static_cast<uint64_t>(reclaim_older_than_age->count()));
+	}
+
+	health_code_update();
+
+	const lttng::pthread::lock_guard socket_lock(*socket.lock);
+
+	std::vector<std::uint8_t> request_payload;
+	request_payload.reserve(sizeof(header) + channel_keys.size() * sizeof(uint64_t));
+
+	/* Append header to the payload. */
+	request_payload.insert(request_payload.end(),
+			       reinterpret_cast<const std::uint8_t *>(&header),
+			       reinterpret_cast<const std::uint8_t *>(&header) + sizeof(header));
+
+	/* Append channel keys to the payload (64-bit each). */
+	request_payload.insert(
+		request_payload.end(),
+		reinterpret_cast<const std::uint8_t *>(channel_keys.data()),
+		reinterpret_cast<const std::uint8_t *>(channel_keys.data() + channel_keys.size()));
+
+	auto reply = consumer_request(
+		socket, LTTNG_CONSUMER_RECLAIM_CHANNELS_MEMORY, std::move(request_payload));
+
+	health_code_update();
+
+	/*
+	 * The expected reply format is:
+	 *   [lttcomm_consumer_channel_memory_reclamation_reply_header] (announces the count of
+	 * stream reclamation results) [lttcomm_stream_memory_reclamation_result]
+	 *   [lttcomm_stream_memory_reclamation_result]
+	 *   [...]
+	 *
+	 * The stream reclamation result structures are delivered in the same order as the channel
+	 * keys were sent in the request. For each channel, the streams are grouped together, and
+	 * their order matches the CPU number: the first stream for a given channel corresponds to
+	 * CPU #0, the second to CPU #1, and so on.
+	 */
+	if (reply.size() < sizeof(lttcomm_consumer_channel_memory_reclamation_reply_header)) {
+		LTTNG_THROW_PROTOCOL_ERROR(fmt::format("Consumer reply is too short: command={}",
+						       LTTNG_CONSUMER_RECLAIM_CHANNELS_MEMORY));
+	}
+
+	const auto stream_result_count = [reply]() {
+		const auto& reclamation_header = *reinterpret_cast<
+			const lttcomm_consumer_channel_memory_reclamation_reply_header *>(
+			reply.data());
+
+		return reclamation_header.count;
+	}();
+
+	/* Build a view over the stream memory reclamation result structures. */
+	const lttng::binary_view<lttcomm_stream_memory_reclamation_result> stream_reclamation_results(
+		reply.data() + sizeof(lttcomm_consumer_channel_memory_reclamation_reply_header),
+		reply.size() - sizeof(lttcomm_consumer_channel_memory_reclamation_reply_header),
+		stream_result_count);
+
+	std::vector<lsc::stream_memory_reclamation_result_group> result;
+	result.resize(channel_keys.size());
+
+	std::size_t current_channel_index = 0;
+	auto current_channel_key = channel_keys[0];
+	/*
+	 * stream_reclamation_results are potentially unaligned (since they are packed), access them
+	 * by value.
+	 */
+	for (const auto stream_reclamation_result : stream_reclamation_results) {
+		if (stream_reclamation_result.channel_key != current_channel_key) {
+			/* First stream of a new channel; change the current channel index. */
+			current_channel_index++;
+			current_channel_key = stream_reclamation_result.channel_key;
+
+			if (current_channel_index == channel_keys.size()) {
+				LTTNG_THROW_PROTOCOL_ERROR(fmt::format(
+					"Consumer reply contains streams of too many channels: expected={}",
+					channel_keys.size()));
+			}
+		}
+
+		result[current_channel_index].streams_reclaimed_memory.emplace_back(
+			stream_reclamation_result.subbuffers_reclaimed,
+			stream_reclamation_result.pending_subbuffers_to_reclaim);
+	}
+
+	return result;
+}
+
 int consumer_init(struct consumer_socket *socket, const lttng_uuid& sessiond_uuid)
 {
 	int ret;
@@ -2153,3 +2440,108 @@ error:
 	health_code_update();
 	return ret;
 }
+
+/*
+ * Ask the consumer to recover/complete from buffer any transaction initiated
+ * by a given application (based on its owner_id).
+ *
+ * Called with the consumer socket lock held.
+ * Returns 0 on success, or a negative value on error.
+ */
+static int do_consumer_reclaim_session_owner_id(struct consumer_data& consumer,
+						uint64_t session_id,
+						uint32_t owner_id,
+						uint32_t *pending_reclamations)
+{
+	if (consumer.cmd_sock < 0) {
+		return -1;
+	}
+
+	auto consumer_socket =
+		lttng::make_unique_wrapper<struct consumer_socket, consumer_destroy_socket>(
+			consumer_allocate_socket(&consumer.cmd_sock));
+
+	if (!consumer_socket) {
+		ERR_FMT("Failed to create consumer socket wrapper during app owner id reclamation: "
+			"owner_id={}",
+			owner_id);
+		return -1;
+	}
+
+	/*
+	 * I don't expect an application using UINT32_MAX channels, but if it
+	 * does, we should probably split this in multiple messages.
+	 */
+	int socket_ret;
+	const struct lttcomm_consumer_msg msg = {
+		.cmd_type = LTTNG_CONSUMER_RECLAIM_SESSION_OWNER_ID,
+		.u = {
+			.reclaim_session_owner_id = {
+				.session_id = session_id,
+				.owner_id = owner_id,
+			},
+		},
+	};
+
+	DBG("Sending ̀LTTNG_CONSUMER_RECLAIM_SESSION_OWNER_ID command header to consumer");
+	socket_ret = consumer_socket_send(consumer_socket.get(), &msg, sizeof(msg));
+	if (socket_ret != 0) {
+		ERR("Failed to send ̀LTTNG_CONSUMER_RECLAIM_SESSION_OWNER_ID command header to consumer");
+		return -1;
+	}
+
+	socket_ret = consumer_socket_recv(
+		consumer_socket.get(), pending_reclamations, sizeof(*pending_reclamations));
+
+	if (socket_ret != 0) {
+		ERR_FMT("Failed to received pending reclamation count while asking for owner reclamation.",
+			owner_id);
+		return -1;
+	}
+
+	return 0;
+}
+
+/*
+ * Notification the consumer-daemon about channels with keys in
+ * `key_of_channels_used_by_app` about reclamation of app's owner-id.
+ *
+ * Return the number of channels that were not notified about the
+ * reclamation. This can happen if the channels were destroyed while sending the
+ * reclamation request.
+ */
+unsigned int consumer_reclaim_session_owner_id(const struct ust_app_session& ua_sess,
+					       uint32_t owner_id)
+{
+	struct consumer_data *consumer;
+
+	switch (ua_sess.bits_per_long) {
+	case 32:
+		consumer = &the_ustconsumer32_data;
+		break;
+	case 64:
+		consumer = &the_ustconsumer64_data;
+		break;
+	default:
+		return 0;
+	}
+
+	uint32_t pending_reclamations = 0;
+
+	int ret;
+	{
+		const lttng::pthread::lock_guard consumer_lock(consumer->lock);
+		ret = do_consumer_reclaim_session_owner_id(
+			*consumer, ua_sess.tracing_id, owner_id, &pending_reclamations);
+	}
+
+	if (ret) {
+		ERR_FMT("Failed to reclaim application owner id during "
+			"application unregistration: "
+			"session_id={}, owner_id={}",
+			ua_sess.tracing_id,
+			owner_id);
+	}
+
+	return pending_reclamations;
+}
