@@ -7,6 +7,9 @@
 
 #include "domain.hpp"
 
+#include <common/exception.hpp>
+#include <common/format.hpp>
+
 lttng::sessiond::exceptions::channel_not_found_error::channel_not_found_error(
 	std::string channel_name_, const lttng::source_location& source_location_) :
 	lttng::runtime_error(
@@ -15,4 +18,4 @@ lttng::sessiond::exceptions::channel_not_found_error::channel_not_found_error(
 		source_location_),
 	channel_name{ std::move(channel_name_) }
 {
-}
\ No newline at end of file
+}
