@@ -192,6 +192,7 @@ int kernel_create_session(const ltt_session::locked_ref& session)
 
 	lks->id = session->id;
 	lks->consumer_fds_sent = 0;
+	lks->trace_format = session->trace_format;
 	session->kernel_session = lks;
 
 	DBG("Kernel session created (fd: %d)", lks->fd);
@@ -219,10 +220,17 @@ int kernel_create_session(const ltt_session::locked_ref& session)
 	}
 
 	ret = kernctl_session_set_output_format(lks->fd,
-						utils_force_experimental_ctf_2() ?
+						session->trace_format == LTTNG_TRACE_FORMAT_CTF_2 ?
 							LTTNG_KERNEL_ABI_OUTPUT_FORMAT_CTF_2 :
 							LTTNG_KERNEL_ABI_OUTPUT_FORMAT_CTF_1_8);
 	if (ret) {
+		if (ret == -ENOSYS && session->trace_format == LTTNG_TRACE_FORMAT_CTF_2) {
+			ERR("Kernel tracer does not support CTF 2 trace format for session %" PRIu64
+			    " name: %s",
+			    session->id,
+			    session->name);
+			goto error;
+		}
 		WARN_FMT("Could not set kernel output format for session {} name: {}",
 			 session->id,
 			 session->name);
