@@ -46,18 +46,20 @@
 #include <common/compat/getenv.hpp>
 #include <common/compat/socket.hpp>
 #include <common/config/session-config.hpp>
+#include <common/consumer/consumer-type.hpp>
 #include <common/daemonize.hpp>
 #include <common/defaults.hpp>
 #include <common/dynamic-buffer.hpp>
+#include <common/error.hpp>
 #include <common/exception.hpp>
 #include <common/futex.hpp>
 #include <common/ini-config/ini-config.hpp>
-#include <common/kernel-consumer/kernel-consumer.hpp>
 #include <common/lockfile.hpp>
 #include <common/logging-utils.hpp>
 #include <common/make-unique-wrapper.hpp>
 #include <common/path.hpp>
 #include <common/relayd/relayd.hpp>
+#include <common/systemd-utils.hpp>
 #include <common/utils.hpp>
 
 #include <lttng/event-internal.hpp>
@@ -138,6 +140,7 @@ static const struct option long_options[] = {
 	{ "extra-kmod-probes", required_argument, nullptr, '\0' },
 	{ EVENT_NOTIFIER_ERROR_BUFFER_SIZE_KERNEL_OPTION_STR, required_argument, nullptr, '\0' },
 	{ EVENT_NOTIFIER_ERROR_BUFFER_SIZE_USERSPACE_OPTION_STR, required_argument, nullptr, '\0' },
+	{ "default-trace-format", required_argument, nullptr, '\0' },
 	{ nullptr, 0, nullptr, 0 }
 };
 
@@ -754,6 +757,22 @@ static int set_option(int opt, const char *arg, const char *optname)
 		DBG3("Number of event notifier error buffer userspace size to non default: %i",
 		     the_config.event_notifier_buffer_size_userspace);
 		goto end;
+	} else if (string_match(optname, "default-trace-format")) {
+		if (!arg || *arg == '\0') {
+			ret = -EINVAL;
+			goto end;
+		}
+		if (strcmp(arg, "ctf-1.8") == 0) {
+			the_config.default_trace_format = LTTNG_TRACE_FORMAT_CTF_1_8;
+		} else if (strcmp(arg, "ctf-2") == 0) {
+			the_config.default_trace_format = LTTNG_TRACE_FORMAT_CTF_2;
+		} else {
+			ERR_FMT("Wrong value in --{} parameter: expecting `ctf-1.8` or `ctf-2` instead of `{}`",
+				optname,
+				arg);
+			ret = -EINVAL;
+			goto end;
+		}
 	} else if (string_match(optname, "config") || opt == 'f') {
 		/* This is handled in set_options() thus silent skip. */
 		goto end;
@@ -1987,8 +2006,15 @@ static int _main(int argc, char **argv)
 	}
 
 	/* Initialization completed. */
+	DBG_FMT("Initialization complete; ready to accept commands");
 	sessiond_signal_parents();
 
+	/*
+	 * Notify systemd that the daemon is ready, no-op on non-Linux
+	 * platforms.
+	 */
+	lttng::systemd::notify_ready();
+
 	/*
 	 * This is where we start awaiting program completion (e.g. through
 	 * signal that asks threads to teardown).
@@ -2001,6 +2027,12 @@ stop_threads:
 
 	DBG("Terminating all threads");
 
+	/*
+	 * Notify systemd that the stopping process has started, no-op on
+	 * non-Linux platforms.
+	 */
+	lttng::systemd::notify_stopping();
+
 	/*
 	 * Ensure that the client thread is no longer accepting new commands,
 	 * which could cause new sessions to be created.
