@@ -16,6 +16,7 @@
 #include <common/config/session-config.hpp>
 #include <common/defaults.hpp>
 #include <common/error.hpp>
+#include <common/optional.hpp>
 #include <common/runas.hpp>
 #include <common/urcu.hpp>
 #include <common/utils.hpp>
@@ -113,6 +114,17 @@ static int save_kernel_channel_attributes(struct config_writer *writer,
 			goto end;
 		}
 
+		if (ext->watchdog_timer_interval.is_set) {
+			ret = config_writer_write_element_unsigned_int(
+				writer,
+				config_element_watchdog_timer_interval,
+				LTTNG_OPTIONAL_GET(ext->watchdog_timer_interval));
+			if (ret) {
+				ret = LTTNG_ERR_SAVE_IO_FAIL;
+				goto end;
+			}
+		}
+
 		ret = config_writer_write_element_signed_int(
 			writer, config_element_blocking_timeout, ext->blocking_timeout);
 		if (ret) {
@@ -126,12 +138,64 @@ end:
 	return ret;
 }
 
+static int save_ust_channel_memory_reclamation_policy(const struct ltt_ust_channel& ust_chan,
+						      struct config_writer *writer)
+{
+	if (!ust_chan.automatic_memory_reclamation_maximal_age) {
+		return LTTNG_OK;
+	}
+
+	const auto age_threshold =
+		ust_chan.automatic_memory_reclamation_maximal_age.value().count();
+
+	int ret = config_writer_open_element(writer, config_element_channel_reclaim_policy);
+	if (ret) {
+		return LTTNG_ERR_SAVE_IO_FAIL;
+	}
+
+	if (age_threshold) {
+		ret = config_writer_open_element(writer,
+						 config_element_channel_reclaim_policy_periodic);
+		if (ret) {
+			return LTTNG_ERR_SAVE_IO_FAIL;
+		}
+		ret = config_writer_write_element_unsigned_int(
+			writer,
+			config_element_channel_reclaim_policy_periodic_age_threshold,
+			age_threshold);
+		if (ret) {
+			return LTTNG_ERR_SAVE_IO_FAIL;
+		}
+		ret = config_writer_close_element(writer);
+		if (ret) {
+			return LTTNG_ERR_SAVE_IO_FAIL;
+		}
+	} else {
+		ret = config_writer_open_element(writer,
+						 config_element_channel_reclaim_policy_consumed);
+		if (ret) {
+			return LTTNG_ERR_SAVE_IO_FAIL;
+		}
+		ret = config_writer_close_element(writer);
+		if (ret) {
+			return LTTNG_ERR_SAVE_IO_FAIL;
+		}
+	}
+
+	ret = config_writer_close_element(writer);
+	if (ret) {
+		return LTTNG_ERR_SAVE_IO_FAIL;
+	}
+
+	return LTTNG_OK;
+}
+
 /* Return LTTNG_OK on success else a LTTNG_ERR* code. */
 static int save_ust_channel_attributes(struct config_writer *writer,
+				       const struct ltt_ust_channel *ust_chan,
 				       struct lttng_ust_abi_channel_attr *attr)
 {
 	int ret;
-	struct ltt_ust_channel *channel = nullptr;
 
 	ret = config_writer_write_element_string(writer,
 						 config_element_overwrite_mode,
@@ -181,7 +245,7 @@ static int save_ust_channel_attributes(struct config_writer *writer,
 	}
 
 	ret = config_writer_write_element_signed_int(
-		writer, config_element_blocking_timeout, attr->u.s.blocking_timeout);
+		writer, config_element_blocking_timeout, attr->blocking_timeout);
 	if (ret) {
 		ret = LTTNG_ERR_SAVE_IO_FAIL;
 		goto end;
@@ -189,7 +253,7 @@ static int save_ust_channel_attributes(struct config_writer *writer,
 
 	const char *allocation_policy_str;
 
-	switch (attr->u.s.type) {
+	switch (attr->type) {
 	case LTTNG_UST_ABI_CHAN_PER_CPU:
 		allocation_policy_str = config_element_channel_allocation_policy_per_cpu;
 		break;
@@ -208,19 +272,56 @@ static int save_ust_channel_attributes(struct config_writer *writer,
 		goto end;
 	}
 
-	/*
-	 * Fetch the monitor timer which is located in the parent of
-	 * lttng_ust_channel_attr
-	 */
-	channel = lttng::utils::container_of(attr, &ltt_ust_channel::attr);
-	ret = config_writer_write_element_unsigned_int(
-		writer, config_element_monitor_timer_interval, channel->monitor_timer_interval);
+	const char *preallocation_policy_str;
+
+	switch (ust_chan->preallocation_policy) {
+	case lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t::
+		PREALLOCATE:
+		preallocation_policy_str = config_element_channel_preallocation_policy_preallocate;
+		break;
+	case lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t::
+		ON_DEMAND:
+		preallocation_policy_str = config_element_channel_preallocation_policy_on_demand;
+		break;
+	default:
+		ret = LTTNG_ERR_SAVE_IO_FAIL;
+		goto end;
+	}
+
+	ret = config_writer_write_element_string(
+		writer, config_element_channel_preallocation_policy, preallocation_policy_str);
 	if (ret) {
 		ret = LTTNG_ERR_SAVE_IO_FAIL;
 		goto end;
 	}
 
-	ret = LTTNG_OK;
+	/*
+	 * Fetch the monitor timer which is located in the parent of
+	 * lttng_ust_channel_attr
+	 */
+	ret = config_writer_write_element_unsigned_int(
+		writer, config_element_monitor_timer_interval, ust_chan->monitor_timer_interval);
+	if (ret) {
+		ret = LTTNG_ERR_SAVE_IO_FAIL;
+		goto end;
+	}
+
+	/*
+	 * Fetch the watchdog timer which is located in the parent of
+	 * lttng_ust_channel_attr
+	 */
+	if (ust_chan->watchdog_timer_interval.is_set) {
+		ret = config_writer_write_element_unsigned_int(
+			writer,
+			config_element_watchdog_timer_interval,
+			LTTNG_OPTIONAL_GET(ust_chan->watchdog_timer_interval));
+		if (ret) {
+			ret = LTTNG_ERR_SAVE_IO_FAIL;
+			goto end;
+		}
+	}
+
+	ret = save_ust_channel_memory_reclamation_policy(*ust_chan, writer);
 end:
 	return ret;
 }
@@ -1616,7 +1717,7 @@ static int save_ust_channel(struct config_writer *writer,
 		goto end;
 	}
 
-	ret = save_ust_channel_attributes(writer, &ust_chan->attr);
+	ret = save_ust_channel_attributes(writer, ust_chan, &ust_chan->attr);
 	if (ret != LTTNG_OK) {
 		goto end;
 	}
@@ -2652,6 +2753,17 @@ static int save_session(const ltt_session::locked_ref& session,
 		}
 	}
 
+	ret = config_writer_write_element_string(writer,
+						 config_element_trace_format,
+						 session->trace_format ==
+								 LTTNG_TRACE_FORMAT_CTF_1_8 ?
+							 config_element_trace_format_ctf_1_8 :
+							 config_element_trace_format_ctf_2);
+	if (ret) {
+		ret = LTTNG_ERR_SAVE_IO_FAIL;
+		goto end;
+	}
+
 	ret = save_domains(writer, session);
 	if (ret != LTTNG_OK) {
 		goto end;
