@@ -10,6 +10,7 @@
 #include "cmd.hpp"
 #include "kernel.hpp"
 #include "lttng-sessiond.hpp"
+#include "pending-memory-reclamation-request.hpp"
 #include "session.hpp"
 #include "timer.hpp"
 #include "trace-ust.hpp"
@@ -18,6 +19,7 @@
 
 #include <common/common.hpp>
 #include <common/ctl/format.hpp>
+#include <common/session.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
 #include <common/trace-chunk.hpp>
 #include <common/urcu.hpp>
@@ -58,9 +60,6 @@ namespace ls = lttng::sessiond;
  * using session_lock() and session_unlock().
  */
 
-/* These characters are forbidden in a session name. Used by validate_name. */
-const char *forbidden_name_chars = "/";
-
 /* Global hash table to keep the sessions, indexed by id. */
 struct lttng_ht *ltt_sessions_ht_by_id = nullptr;
 /* Global hash table to keep the sessions, indexed by name. */
@@ -135,39 +134,6 @@ end:
 }
 } /* namespace */
 
-/*
- * Validate the session name for forbidden characters.
- *
- * Return 0 on success else -1 meaning a forbidden char. has been found.
- */
-static int validate_name(const char *name)
-{
-	int ret;
-	char *tok, *tmp_name;
-
-	LTTNG_ASSERT(name);
-
-	tmp_name = strdup(name);
-	if (!tmp_name) {
-		/* ENOMEM here. */
-		ret = -1;
-		goto error;
-	}
-
-	tok = strpbrk(tmp_name, forbidden_name_chars);
-	if (tok) {
-		DBG("Session name %s contains a forbidden character", name);
-		/* Forbidden character has been found. */
-		ret = -1;
-		goto error;
-	}
-	ret = 0;
-
-error:
-	free(tmp_name);
-	return ret;
-}
-
 /*
  * Add a ltt_session structure to the global list.
  *
@@ -427,7 +393,7 @@ static bool ltt_sessions_ht_empty()
 	LTTNG_ASSERT(ltt_sessions_ht_by_name);
 
 	if (lttng_ht_get_count(ltt_sessions_ht_by_id) != 0) {
-		/* Not empty.*/
+		/* Not empty. */
 		goto end;
 	}
 
@@ -496,23 +462,23 @@ void ltt_session::unlock() const noexcept
 	ltt_session::_const_session_unlock(*this);
 }
 
-ls::domain& ltt_session::get_domain(ls::domain_class domain)
+ls::domain& ltt_session::get_domain(lttng::domain_class domain)
 {
 	switch (domain) {
-	case ls::domain_class::LOG4J:
-	case ls::domain_class::LOG4J2:
-	case ls::domain_class::JAVA_UTIL_LOGGING:
-	case ls::domain_class::PYTHON_LOGGING:
-	case ls::domain_class::USER_SPACE:
+	case lttng::domain_class::LOG4J:
+	case lttng::domain_class::LOG4J2:
+	case lttng::domain_class::JAVA_UTIL_LOGGING:
+	case lttng::domain_class::PYTHON_LOGGING:
+	case lttng::domain_class::USER_SPACE:
 		return user_space_domain;
-	case ls::domain_class::KERNEL_SPACE:
+	case lttng::domain_class::KERNEL_SPACE:
 		return kernel_space_domain;
 	}
 
 	std::abort();
 }
 
-const ls::domain& ltt_session::get_domain(ls::domain_class domain) const
+const ls::domain& ltt_session::get_domain(lttng::domain_class domain) const
 {
 	/* NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast) */
 	return const_cast<ltt_session *>(this)->get_domain(domain);
@@ -1258,7 +1224,7 @@ session_create(const char *name, uid_t uid, gid_t gid, struct ltt_session **out_
 			ret_code = LTTNG_ERR_SESSION_INVALID_CHAR;
 			goto error;
 		}
-		ret = validate_name(name);
+		ret = session_validate_name(name);
 		if (ret < 0) {
 			ret_code = LTTNG_ERR_SESSION_INVALID_CHAR;
 			goto error;
@@ -1610,15 +1576,15 @@ std::unique_lock<std::mutex> ls::lock_session_list()
 }
 
 lttng::sessiond::user_space_consumer_channel_keys
-ltt_session::user_space_consumer_channel_keys() const
+ltt_session::user_space_consumer_channel_keys(lttng::c_string_view channel_name_filter) const
 {
 	switch (ust_session->buffer_type) {
 	case LTTNG_BUFFER_PER_PID:
-		return lttng::sessiond::user_space_consumer_channel_keys(*ust_session,
-									 *ust_app_get_all());
+		return lttng::sessiond::user_space_consumer_channel_keys(
+			*ust_session, *ust_app_get_all(), channel_name_filter);
 	case LTTNG_BUFFER_PER_UID:
 		return lttng::sessiond::user_space_consumer_channel_keys(
-			*ust_session, ust_session->buffer_reg_uid_list);
+			*ust_session, ust_session->buffer_reg_uid_list, channel_name_filter);
 	default:
 		abort();
 	}
@@ -1641,15 +1607,16 @@ ls::user_space_consumer_channel_keys::iterator::operator++()
 {
 	if (_is_end) {
 		LTTNG_THROW_OUT_OF_RANGE(fmt::format(
-			"Attempted to advance channel key iterator past the end of channel keys: iteration_mode={}",
-			_creation_context._session.buffer_type));
+			"Attempted to advance channel key iterator past the end of channel keys: iteration_mode='{}'",
+			_creation_context._mode == _iteration_mode::PER_PID ? "per-pid" :
+									      "per-uid"));
 	}
 
-	switch (_creation_context._session.buffer_type) {
-	case LTTNG_BUFFER_PER_PID:
+	switch (_creation_context._mode) {
+	case _iteration_mode::PER_PID:
 		_advance_one_per_pid();
 		break;
-	case LTTNG_BUFFER_PER_UID:
+	case _iteration_mode::PER_UID:
 		_advance_one_per_uid();
 		break;
 	default:
@@ -1659,13 +1626,6 @@ ls::user_space_consumer_channel_keys::iterator::operator++()
 	return *this;
 }
 
-namespace {
-bool is_list_empty(const cds_list_head *head)
-{
-	return head == head->next;
-}
-} /* namespace */
-
 ls::user_space_consumer_channel_keys::iterator::iterator(
 	const _iterator_creation_context& creation_context, bool is_end) :
 	_creation_context(creation_context), _is_end(is_end)
@@ -1731,8 +1691,28 @@ void ls::user_space_consumer_channel_keys::iterator::_skip_to_next_app_per_pid(
 		}
 
 		position.current_registry_session = registry;
-		lttng_ht_get_first((*position.current_app_session)->channels,
-				   &_position.channel_iterator);
+		if (!_creation_context._channel_filter) {
+			/* No filter; point to the app's first channel. */
+			lttng_ht_get_first((*position.current_app_session)->channels,
+					   &_position.channel_iterator);
+		} else {
+			if (_creation_context._channel_filter->is_metadata()) {
+				/*
+				 * An invalid iterator will cause the "metadata" value to be
+				 * delivered.
+				 */
+				_position.channel_iterator.iter = {};
+			} else {
+				/* A filter was specified, point to the target channel. */
+				lttng_ht_lookup(
+					(*position.current_app_session)->channels,
+					(void *) _creation_context._channel_filter->name.data(),
+					&_position.channel_iterator);
+				LTTNG_ASSERT(
+					cds_lfht_iter_get_node(&_position.channel_iterator.iter));
+			}
+		}
+
 		break;
 	}
 }
@@ -1750,7 +1730,7 @@ void ls::user_space_consumer_channel_keys::iterator::_init_per_uid() noexcept
 	auto& position = _position._per_uid;
 
 	/* Start the iteration: get the first registry and point to its first channel. */
-	if (is_list_empty(&_creation_context._session.buffer_reg_uid_list)) {
+	if (lttng::urcu::is_list_empty(&_creation_context._session.buffer_reg_uid_list)) {
 		_is_end = true;
 		return;
 	}
@@ -1758,16 +1738,44 @@ void ls::user_space_consumer_channel_keys::iterator::_init_per_uid() noexcept
 	position.registry_list_head = &_creation_context._session.buffer_reg_uid_list;
 	position.current_registry = lttng::utils::container_of(
 		_creation_context._session.buffer_reg_uid_list.next, &buffer_reg_uid::lnode);
-	lttng_ht_get_first(position.current_registry->registry->channels,
-			   &_position.channel_iterator);
+	if (!_creation_context._channel_filter) {
+		/* No filter; point to the registry's first channel. */
+		lttng_ht_get_first(position.current_registry->registry->channels,
+				   &_position.channel_iterator);
+	} else {
+		/* A filter was specified, point to the target channel. */
+		if (_creation_context._channel_filter->is_metadata()) {
+			/*
+			 * An invalid iterator will cause the "metadata" value to be
+			 * delivered.
+			 */
+			_position.channel_iterator.iter = {};
+		} else {
+			const auto config_key = _creation_context._channel_filter->config_key();
+
+			lttng_ht_lookup(position.current_registry->registry->channels,
+					&config_key,
+					&_position.channel_iterator);
+			if (!cds_lfht_iter_get_node(&_position.channel_iterator.iter)) {
+				/* No stream set allocated for this channel yet. */
+				_is_end = true;
+			}
+		}
+	}
 }
 
 void ls::user_space_consumer_channel_keys::iterator::_advance_one_per_pid()
 {
 	auto& position = _position._per_pid;
 
-	if (!cds_lfht_iter_get_node(&_position.channel_iterator.iter)) {
-		/* Reached the last channel. Move on to the next app. */
+	if (_creation_context._channel_filter ||
+	    !cds_lfht_iter_get_node(&_position.channel_iterator.iter)) {
+		/*
+		 * If a channel filter is present in the creation context, it is guaranteed that the
+		 * current channel is the only one matching the filter, and thus, is considered the
+		 * "last" channel for this iteration. In this case, or if there are no more
+		 * channels, the iterator skips to the next application.
+		 */
 		_skip_to_next_app_per_pid(false);
 		return;
 	}
@@ -1783,8 +1791,13 @@ void ls::user_space_consumer_channel_keys::iterator::_advance_one_per_uid()
 {
 	auto& position = _position._per_uid;
 
-	if (!cds_lfht_iter_get_node(&_position.channel_iterator.iter)) {
-		/* Reached the last channel of the registry. Move on to the next registry. */
+	if (_creation_context._channel_filter ||
+	    !cds_lfht_iter_get_node(&_position.channel_iterator.iter)) {
+		/*
+		 * If a channel filter is present in the creation context, it is guaranteed that the
+		 * current channel is the only one matching the filter, and thus, is considered the
+		 * "last" channel for this iteration. Move on to the next registry.
+		 */
 		if (position.current_registry->lnode.next == position.registry_list_head) {
 			_is_end = true;
 			return;
@@ -1792,14 +1805,32 @@ void ls::user_space_consumer_channel_keys::iterator::_advance_one_per_uid()
 
 		position.current_registry = lttng::utils::container_of(
 			position.current_registry->lnode.next, &buffer_reg_uid::lnode);
-		cds_lfht_first(position.current_registry->registry->channels->ht,
-			       &_position.channel_iterator.iter);
+
+		if (!_creation_context._channel_filter) {
+			/* No filter; point to the registry's first channel. */
+			lttng_ht_get_first(position.current_registry->registry->channels,
+					   &_position.channel_iterator);
+		} else {
+			if (_creation_context._channel_filter->is_metadata()) {
+				_position.channel_iterator = {};
+			} else {
+				const auto config_key =
+					_creation_context._channel_filter->config_key();
+
+				/* A filter was specified, point to the target channel. */
+				lttng_ht_lookup(position.current_registry->registry->channels,
+						&config_key,
+						&_position.channel_iterator);
+				LTTNG_ASSERT(
+					cds_lfht_iter_get_node(&_position.channel_iterator.iter));
+			}
+		}
 
 		/* Assumes a registry can't be empty. */
 		LTTNG_ASSERT(cds_lfht_iter_get_node(&_position.channel_iterator.iter));
 	} else {
-		cds_lfht_next(position.current_registry->registry->channels->ht,
-			      &_position.channel_iterator.iter);
+		lttng_ht_get_next(position.current_registry->registry->channels,
+				  &_position.channel_iterator);
 	}
 }
 
@@ -1821,7 +1852,7 @@ bool ls::user_space_consumer_channel_keys::iterator::operator!=(const iterator&
 ls::user_space_consumer_channel_keys::iterator::key
 ls::user_space_consumer_channel_keys::iterator::_get_current_value_per_pid() const noexcept
 {
-	auto& position = _position._per_pid;
+	const auto& position = _position._per_pid;
 
 	const auto *channel_node =
 		lttng_ht_iter_get_node<lttng_ht_node_str>(&_position.channel_iterator);
@@ -1837,9 +1868,12 @@ ls::user_space_consumer_channel_keys::iterator::_get_current_value_per_pid() con
 
 		return { static_cast<consumer_bitness>(app.abi.bits_per_long),
 			 channel.key,
-			 ls::user_space_consumer_channel_keys::channel_type::DATA };
+			 ls::user_space_consumer_channel_keys::channel_type::DATA,
+			 app.pid };
 	} else {
 		LTTNG_ASSERT(position.current_registry_session);
+		LTTNG_ASSERT(!_creation_context._channel_filter ||
+			     _creation_context._channel_filter->is_metadata());
 
 		/*
 		 * Once the last data channel is delivered (iter points to the 'end' of the ht),
@@ -1847,7 +1881,8 @@ ls::user_space_consumer_channel_keys::iterator::_get_current_value_per_pid() con
 		 */
 		return { static_cast<consumer_bitness>(app.abi.bits_per_long),
 			 position.current_registry_session->_metadata_key,
-			 ls::user_space_consumer_channel_keys::channel_type::METADATA };
+			 ls::user_space_consumer_channel_keys::channel_type::METADATA,
+			 app.pid };
 	}
 }
 
@@ -1864,8 +1899,12 @@ ls::user_space_consumer_channel_keys::iterator::_get_current_value_per_uid() con
 		return { static_cast<consumer_bitness>(
 				 _position._per_uid.current_registry->bits_per_long),
 			 channel.consumer_key,
-			 ls::user_space_consumer_channel_keys::channel_type::DATA };
+			 ls::user_space_consumer_channel_keys::channel_type::DATA,
+			 _position._per_uid.current_registry->uid };
 	} else {
+		LTTNG_ASSERT(!_creation_context._channel_filter ||
+			     _creation_context._channel_filter->name == DEFAULT_METADATA_NAME);
+
 		/*
 		 * Once the last data channel is delivered (iter points to the 'end' of the ht),
 		 * deliver the metadata channel's key.
@@ -1873,7 +1912,8 @@ ls::user_space_consumer_channel_keys::iterator::_get_current_value_per_uid() con
 		return { static_cast<consumer_bitness>(
 				 _position._per_uid.current_registry->bits_per_long),
 			 _position._per_uid.current_registry->registry->reg.ust->_metadata_key,
-			 ls::user_space_consumer_channel_keys::channel_type::METADATA };
+			 ls::user_space_consumer_channel_keys::channel_type::METADATA,
+			 _position._per_uid.current_registry->uid };
 	}
 }
 
@@ -1923,3 +1963,33 @@ ls::user_space_consumer_channel_keys::iterator::_get_registry_session_per_uid()
 {
 	return _position._per_uid.current_registry->registry->reg.ust;
 }
+
+bool ltt_session::has_pending_recording_channel_memory_reclamation(
+	const lttng::c_string_view channel_name) const
+{
+	ASSERT_LOCKED(_lock);
+	return _channels_with_pending_reclaim.count(channel_name) > 0;
+}
+
+void ltt_session::add_pending_reclamation(std::string channel_name)
+{
+	ASSERT_LOCKED(_lock);
+	const auto result = _channels_with_pending_reclaim.emplace(std::move(channel_name));
+	if (!result.second) {
+		LTTNG_THROW_ERROR(fmt::format(
+			"Channel already present in pending reclaim set: session_name=`{}`, channel_name=`{}`",
+			name,
+			*result.first));
+	}
+}
+
+void ltt_session::remove_pending_reclamation(const lttng::c_string_view channel_name)
+{
+	ASSERT_LOCKED(_lock);
+	if (_channels_with_pending_reclaim.erase(channel_name) != 1) {
+		LTTNG_THROW_ERROR(fmt::format(
+			"Failed to remove channel from pending reclaim set: session_name=`{}`, channel_name=`{}`",
+			name,
+			channel_name));
+	}
+}
