@@ -18,20 +18,27 @@
 #include <common/dynamic-array.hpp>
 #include <common/exception.hpp>
 #include <common/hashtable/hashtable.hpp>
+#include <common/macros.hpp>
 #include <common/make-unique-wrapper.hpp>
 #include <common/pthread-lock.hpp>
 #include <common/reference.hpp>
+#include <common/trace-chunk.hpp>
 #include <common/urcu.hpp>
 
 #include <lttng/location.h>
 #include <lttng/lttng-error.h>
 #include <lttng/rotation.h>
+#include <lttng/session-descriptor.h>
 
 #include <condition_variable>
+#include <iterator>
 #include <limits.h>
 #include <mutex>
 #include <stdbool.h>
+#include <string>
+#include <unordered_set>
 #include <urcu/list.h>
+#include <utility>
 
 #define ASSERT_SESSION_LIST_LOCKED() LTTNG_ASSERT(session_trylock_list())
 
@@ -115,6 +122,19 @@ public:
 		DATA,
 	};
 
+	struct key {
+		/* Bitness is needed to query the appropriate consumer daemon. */
+		consumer_bitness bitness;
+		std::uint64_t key_value;
+		channel_type type;
+
+		bool operator==(const key& other)
+		{
+			return bitness == other.bitness && key_value == other.key_value &&
+				type == other.type;
+		}
+	};
+
 	iterator begin() const noexcept;
 	iterator end() const noexcept;
 
@@ -125,31 +145,136 @@ private:
 
 	};
 
-	struct _iterator_creation_context {
+	class _channel_filter_spec final {
+	public:
+		explicit _channel_filter_spec(
+			const lttng::c_string_view target_name,
+			nonstd::optional<std::uint64_t> config_key = nonstd::nullopt) noexcept :
+			name(target_name), _config_key(config_key)
+		{
+			/*
+			 * A metadata channel doesn't have a config key (the 'id' of a matching
+			 * ltt_ust_channel instance).
+			 */
+			LTTNG_ASSERT((target_name == DEFAULT_METADATA_NAME) !=
+				     config_key.has_value());
+		}
+
+		const lttng::c_string_view name;
+		bool is_metadata() const noexcept
+		{
+			return !_config_key.has_value();
+		}
+
+		std::uint64_t config_key() const
+		{
+			LTTNG_ASSERT(!is_metadata());
+			return _config_key.value();
+		}
+
+	private:
+		/* Unique ID of the channel's ltt_ust_channel instance. */
+		const nonstd::optional<std::uint64_t> _config_key;
+	};
+
+	struct _iterator_creation_context final {
+		_iterator_creation_context(_iteration_mode mode,
+					   const ltt_ust_session& session,
+					   lttng_ht *apps,
+					   nonstd::optional<_channel_filter_spec> channel_filter) :
+			_mode(mode),
+			_session(session),
+			_container{ .apps = apps },
+			_channel_filter(std::move(channel_filter))
+		{
+		}
+
+		_iterator_creation_context(_iteration_mode mode,
+					   const ltt_ust_session& session,
+					   const cds_list_head *buffer_registry,
+					   nonstd::optional<_channel_filter_spec> channel_filter) :
+			_mode(mode),
+			_session(session),
+			_container{ .buffer_registry = buffer_registry },
+			_channel_filter(channel_filter)
+		{
+		}
+
 		const _iteration_mode _mode;
 		const ltt_ust_session& _session;
-		union {
+		const union {
 			lttng_ht *apps;
 			const cds_list_head *buffer_registry;
 		} _container;
+		const nonstd::optional<_channel_filter_spec> _channel_filter;
 	};
 
 public:
-	class iterator : public std::iterator<std::input_iterator_tag, std::uint64_t> {
+	class iterator : public std::iterator<std::forward_iterator_tag, key> {
 		friend user_space_consumer_channel_keys;
 
 	public:
-		struct key {
-			/* Bitness is needed to query the appropriate consumer daemon. */
-			consumer_bitness bitness;
-			std::uint64_t key_value;
-			channel_type type;
-
-			bool operator==(const key& other)
+		class key {
+		public:
+			key(consumer_bitness bitness_,
+			    std::uint64_t consumer_key_,
+			    channel_type channel_type,
+			    uid_t uid) noexcept :
+				bitness(bitness_),
+				consumer_key(consumer_key_),
+				type(channel_type),
+				ownership_model(lttng::sessiond::recording_channel_configuration::
+							owership_model_t::PER_UID),
+				owner_id{ .uid = uid }
 			{
-				return bitness == other.bitness && key_value == other.key_value &&
-					type == other.type;
 			}
+
+			key(consumer_bitness bitness_,
+			    std::uint64_t consumer_key_,
+			    channel_type channel_type,
+			    pid_t pid) noexcept :
+				bitness(bitness_),
+				consumer_key(consumer_key_),
+				type(channel_type),
+				ownership_model(lttng::sessiond::recording_channel_configuration::
+							owership_model_t::PER_PID),
+				owner_id{ .pid = pid }
+			{
+			}
+
+			const consumer_bitness bitness;
+			const std::uint64_t consumer_key;
+			const channel_type type;
+
+			bool operator==(const key& other) const noexcept
+			{
+				return bitness == other.bitness &&
+					consumer_key == other.consumer_key && type == other.type;
+			}
+
+			uid_t owner_uid() const noexcept
+			{
+				LTTNG_ASSERT(ownership_model ==
+					     lttng::sessiond::recording_channel_configuration::
+						     owership_model_t::PER_UID);
+				return owner_id.uid;
+			}
+
+			pid_t owner_pid() const noexcept
+			{
+				LTTNG_ASSERT(ownership_model ==
+					     lttng::sessiond::recording_channel_configuration::
+						     owership_model_t::PER_PID);
+				return owner_id.pid;
+			}
+
+		private:
+			const lttng::sessiond::recording_channel_configuration::owership_model_t
+				ownership_model;
+			const union {
+				uid_t uid; /* per-UID ownership model */
+				pid_t pid; /* per-PID ownership model */
+			} owner_id;
 		};
 
 		/*
@@ -211,19 +336,54 @@ public:
 	};
 
 private:
-	user_space_consumer_channel_keys(const ltt_ust_session& ust_session, lttng_ht& apps) :
-		_creation_context{ _iteration_mode::PER_PID, ust_session, { .apps = &apps } }
+	user_space_consumer_channel_keys(const ltt_ust_session& ust_session,
+					 lttng_ht& apps,
+					 lttng::c_string_view channel_name_filter) :
+		_creation_context{ _iteration_mode::PER_PID,
+				   ust_session,
+				   &apps,
+				   _filter_from_channel_name(ust_session, channel_name_filter) }
 	{
 	}
 
 	user_space_consumer_channel_keys(const ltt_ust_session& ust_session,
-					 const cds_list_head& buffer_registry) :
+					 const cds_list_head& buffer_registry,
+					 lttng::c_string_view channel_name_filter) :
 		_creation_context{ _iteration_mode::PER_UID,
 				   ust_session,
-				   { .buffer_registry = &buffer_registry } }
+				   &buffer_registry,
+				   _filter_from_channel_name(ust_session, channel_name_filter) }
+
 	{
 	}
 
+	static nonstd::optional<_channel_filter_spec>
+	_filter_from_channel_name(const ltt_ust_session& ust_session,
+				  lttng::c_string_view channel_name)
+	{
+		if (!channel_name) {
+			/* No filter if no channel name is provided. */
+			return nonstd::nullopt;
+		}
+
+		if (channel_name == DEFAULT_METADATA_NAME) {
+			/*
+			 * Metadata channel is special, it has no config key since
+			 * it is not part of the ltt_ust_session's channels.
+			 */
+			return _channel_filter_spec(channel_name);
+		}
+
+		const lttng::urcu::read_lock_guard read_lock;
+		auto *ust_channel = trace_ust_find_channel_by_name(
+			ust_session.domain_global.channels, channel_name.data());
+		if (!ust_channel) {
+			LTTNG_THROW_CHANNEL_NOT_FOUND_BY_NAME_ERROR(channel_name);
+		}
+
+		return _channel_filter_spec(channel_name, ust_channel->id);
+	}
+
 	class _scoped_rcu_read_lock {
 	public:
 		_scoped_rcu_read_lock()
@@ -266,6 +426,12 @@ struct ltt_session {
 	friend lttng::sessiond::user_space_consumer_channel_keys::iterator;
 
 private:
+	/*
+	 * Set of channel names that have a user-initiated reclamation request
+	 * in progress. Used to reject concurrent requests on the same channel.
+	 */
+	std::unordered_set<std::string> _channels_with_pending_reclaim;
+
 	static void _locked_session_release(ltt_session *session);
 	static void _locked_const_session_release(const ltt_session *session);
 	static void _const_session_put(const ltt_session *session);
@@ -323,10 +489,16 @@ public:
 	void lock() const noexcept;
 	void unlock() const noexcept;
 
-	lttng::sessiond::domain& get_domain(lttng::sessiond::domain_class domain);
-	const lttng::sessiond::domain& get_domain(lttng::sessiond::domain_class domain) const;
+	bool has_pending_recording_channel_memory_reclamation(
+		const lttng::c_string_view channel_name) const;
+	void add_pending_reclamation(std::string channel_name);
+	void remove_pending_reclamation(const lttng::c_string_view channel_name);
 
-	lttng::sessiond::user_space_consumer_channel_keys user_space_consumer_channel_keys() const;
+	lttng::sessiond::domain& get_domain(lttng::domain_class domain);
+	const lttng::sessiond::domain& get_domain(lttng::domain_class domain) const;
+
+	lttng::sessiond::user_space_consumer_channel_keys
+	user_space_consumer_channel_keys(lttng::c_string_view channel_name_filter = {}) const;
 
 	/*
 	 * Session list lock must be acquired by the caller.
@@ -364,7 +536,7 @@ public:
 	struct cds_list_head list = {};
 	/* session unique identifier */
 	id_t id = 0;
-	/* Indicates if the session has been added to the session list and ht.*/
+	/* Indicates if the session has been added to the session list and ht. */
 	bool published = false;
 	/* Indicates if a destroy command has been applied to this session. */
 	bool destroyed = false;
@@ -419,6 +591,10 @@ public:
 	 * Path where to keep the shared memory files.
 	 */
 	char shm_path[PATH_MAX] = {};
+	/*
+	 * Trace format version of the session.
+	 */
+	enum lttng_trace_format trace_format;
 	/*
 	 * Node in ltt_sessions_ht_by_id.
 	 */
@@ -432,10 +608,10 @@ public:
 	 * the last rotation.
 	 */
 	bool rotation_pending_check_timer_enabled = false;
-	timer_t rotation_pending_check_timer = nullptr;
+	timer_t rotation_pending_check_timer = {};
 	/* Timer to periodically rotate a session. */
 	bool rotation_schedule_timer_enabled = false;
-	timer_t rotation_schedule_timer = nullptr;
+	timer_t rotation_schedule_timer = {};
 	/* Value for periodic rotations, 0 if disabled. */
 	uint64_t rotate_timer_period = 0;
 	/* Value for size-based rotations, 0 if disabled. */
@@ -476,9 +652,9 @@ public:
 	char *base_path = nullptr;
 
 	lttng::sessiond::multi_channel_domain user_space_domain =
-		lttng::sessiond::multi_channel_domain(lttng::sessiond::domain_class::USER_SPACE);
+		lttng::sessiond::multi_channel_domain(lttng::domain_class::USER_SPACE);
 	lttng::sessiond::multi_channel_domain kernel_space_domain =
-		lttng::sessiond::multi_channel_domain(lttng::sessiond::domain_class::KERNEL_SPACE);
+		lttng::sessiond::multi_channel_domain(lttng::domain_class::KERNEL_SPACE);
 };
 
 /*
@@ -612,6 +788,27 @@ public:
 } /* namespace sessiond */
 } /* namespace lttng */
 
+namespace fmt {
+template <>
+struct formatter<lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness>
+	: formatter<std::string> {
+	template <typename FormatContextType>
+	typename FormatContextType::iterator
+	format(lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness bitness,
+	       FormatContextType& ctx) const
+	{
+		switch (bitness) {
+		case lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness::ABI_32:
+			return formatter<std::string>::format("32-bit", ctx);
+		case lttng::sessiond::user_space_consumer_channel_keys::consumer_bitness::ABI_64:
+			return formatter<std::string>::format("64-bit", ctx);
+		default:
+			return formatter<std::string>::format("unknown", ctx);
+		}
+	}
+};
+} /* namespace fmt */
+
 void session_destroy(struct ltt_session *session);
 int session_add_destroy_notifier(const ltt_session::locked_ref& session,
 				 ltt_session_destroy_notifier notifier,
