@@ -286,7 +286,7 @@ struct ltt_ust_session *trace_ust_create_session(uint64_t session_id)
 	lus->metadata_attr.switch_timer_interval = DEFAULT_METADATA_SWITCH_TIMER;
 	lus->metadata_attr.read_timer_interval = DEFAULT_METADATA_READ_TIMER;
 	lus->metadata_attr.output = LTTNG_UST_ABI_MMAP;
-	lus->metadata_attr.u.s.type = LTTNG_UST_ABI_CHAN_METADATA;
+	lus->metadata_attr.type = LTTNG_UST_ABI_CHAN_METADATA;
 
 	/*
 	 * Default buffer type. This can be changed through an enable channel
@@ -349,12 +349,18 @@ struct ltt_ust_channel *trace_ust_create_channel(struct lttng_channel *chan,
 
 	LTTNG_ASSERT(chan);
 
-	luc = zmalloc<ltt_ust_channel>();
-	if (luc == nullptr) {
-		PERROR("ltt_ust_channel zmalloc");
+	try {
+		luc = new ltt_ust_channel;
+	} catch (const std::bad_alloc&) {
+		PERROR("Failed to allocate ltt_ust_channel");
 		return nullptr;
 	}
 
+	const auto extended =
+		static_cast<const struct lttng_channel_extended *>(chan->attr.extended.ptr);
+	const auto allocation_policy =
+		static_cast<enum lttng_channel_allocation_policy>(extended->allocation_policy);
+
 	luc->domain = domain;
 
 	/* Copy UST channel attributes */
@@ -364,29 +370,61 @@ struct ltt_ust_channel *trace_ust_create_channel(struct lttng_channel *chan,
 	luc->attr.switch_timer_interval = chan->attr.switch_timer_interval;
 	luc->attr.read_timer_interval = chan->attr.read_timer_interval;
 	luc->attr.output = (enum lttng_ust_abi_output) chan->attr.output;
-	luc->monitor_timer_interval =
-		((struct lttng_channel_extended *) chan->attr.extended.ptr)->monitor_timer_interval;
-	luc->attr.u.s.blocking_timeout =
-		((struct lttng_channel_extended *) chan->attr.extended.ptr)->blocking_timeout;
+	luc->monitor_timer_interval = extended->monitor_timer_interval;
+	luc->attr.blocking_timeout = extended->blocking_timeout;
 
-	const auto extended =
-		static_cast<const struct lttng_channel_extended *>(chan->attr.extended.ptr);
+	if (extended->watchdog_timer_interval.is_set) {
+		switch (domain) {
+		case LTTNG_DOMAIN_UST: /* Fallthrough */
+		case LTTNG_DOMAIN_JUL: /* Fallthrough */
+		case LTTNG_DOMAIN_LOG4J: /* Fallthrough */
+		case LTTNG_DOMAIN_PYTHON: /* Fallthrough */
+		case LTTNG_DOMAIN_LOG4J2:
+		{
+			const auto watchdog_timer_value =
+				LTTNG_OPTIONAL_GET(extended->watchdog_timer_interval);
 
-	const auto allocation_policy =
-		static_cast<enum lttng_channel_allocation_policy>(extended->allocation_policy);
+			LTTNG_OPTIONAL_SET(&luc->watchdog_timer_interval, watchdog_timer_value);
+			break;
+		}
+		default:
+			ERR_FMT("Watchdog timer only valid for UST, JUL, LOG4J, PYTHON and LOG4J2 domains: domain={}",
+				static_cast<int>(domain));
+			goto error;
+		}
+	}
+	luc->attr.blocking_timeout = extended->blocking_timeout;
+
+	if (extended->automatic_memory_reclamation_maximal_age_us.is_set) {
+		luc->automatic_memory_reclamation_maximal_age = std::chrono::microseconds(
+			LTTNG_OPTIONAL_GET(extended->automatic_memory_reclamation_maximal_age_us));
+	}
 
 	switch (allocation_policy) {
 	case LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU:
-		luc->attr.u.s.type = LTTNG_UST_ABI_CHAN_PER_CPU;
+		luc->attr.type = LTTNG_UST_ABI_CHAN_PER_CPU;
 		break;
 	case LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CHANNEL:
-		luc->attr.u.s.type = LTTNG_UST_ABI_CHAN_PER_CHANNEL;
+		luc->attr.type = LTTNG_UST_ABI_CHAN_PER_CHANNEL;
 		break;
 	default:
 		PERROR("Unknown channel stream allocation");
 		goto error;
 	}
 
+	luc->preallocation_policy = [](enum lttng_channel_preallocation_policy policy) {
+		switch (policy) {
+		case LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE:
+			return lttng::sessiond::recording_channel_configuration::
+				buffer_preallocation_policy_t::PREALLOCATE;
+		case LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND:
+			return lttng::sessiond::recording_channel_configuration::
+				buffer_preallocation_policy_t::ON_DEMAND;
+		default:
+			std::abort();
+		}
+	}(static_cast<enum lttng_channel_preallocation_policy>(extended->preallocation_policy));
+
 	/* Translate to UST output enum */
 	switch (luc->attr.output) {
 	default:
@@ -423,7 +461,7 @@ struct ltt_ust_channel *trace_ust_create_channel(struct lttng_channel *chan,
 	return luc;
 
 error:
-	free(luc);
+	delete luc;
 	return nullptr;
 }
 
@@ -1301,7 +1339,7 @@ static void _trace_ust_destroy_channel(struct ltt_ust_channel *channel)
 
 	DBG2("Trace destroy UST channel %s", channel->name);
 
-	free(channel);
+	delete channel;
 }
 
 /*
@@ -1464,4 +1502,4 @@ bool trace_ust_runtime_ctl_version_matches_build_version()
 	}
 
 	return true;
-}
\ No newline at end of file
+}
