@@ -9,18 +9,23 @@
 #ifndef _LTT_UST_APP_H
 #define _LTT_UST_APP_H
 
+#include "lttng-ust-ctl.hpp"
 #include "trace-class.hpp"
 #include "trace-ust.hpp"
 #include "ust-field-quirks.hpp"
 
 #include <common/format.hpp>
 #include <common/index-allocator.hpp>
+#include <common/optional.hpp>
 #include <common/reference.hpp>
 #include <common/scope-exit.hpp>
 #include <common/uuid.hpp>
 
+#include <vendor/optional.hpp>
+
 #include <list>
 #include <stdint.h>
+#include <vector>
 
 #define UST_APP_EVENT_LIST_SIZE 32
 
@@ -154,42 +159,47 @@ struct ust_app_stream {
 };
 
 struct ust_app_channel {
-	bool enabled;
-	int handle;
+	bool enabled = false;
+	int handle = 0;
 	/*
 	 * Unique key used to identify the channel on the consumer side.
 	 * 0 is a reserved 'invalid' value used to indicate that the consumer
 	 * does not know about this channel (i.e. an error occurred).
 	 */
-	uint64_t key;
+	uint64_t key = 0;
 	/* Id of the tracing channel set on creation. */
-	uint64_t tracing_channel_id;
+	uint64_t tracing_channel_id = 0;
 	/* Number of stream that this channel is expected to receive. */
-	unsigned int expected_stream_count;
-	char name[LTTNG_UST_ABI_SYM_NAME_LEN];
-	struct lttng_ust_abi_object_data *obj;
-	struct lttng_ust_ctl_consumer_channel_attr attr;
-	struct ust_app_stream_list streams;
+	unsigned int expected_stream_count = 0;
+	char name[LTTNG_UST_ABI_SYM_NAME_LEN] = {};
+	struct lttng_ust_abi_object_data *obj = nullptr;
+	struct lttng_ust_ctl_consumer_channel_attr attr = {};
+	struct ust_app_stream_list streams = {};
 	/* Session pointer that owns this object. */
-	struct ust_app_session *session;
+	struct ust_app_session *session = nullptr;
 	/* Hashtable of ust_app_ctx instances. */
-	struct lttng_ht *ctx;
+	struct lttng_ht *ctx = nullptr;
 	/* Hashtable of ust_app_event instances. */
-	struct lttng_ht *events;
-	uint64_t tracefile_size;
-	uint64_t tracefile_count;
-	uint64_t monitor_timer_interval;
+	struct lttng_ht *events = nullptr;
+	uint64_t tracefile_size = 0;
+	uint64_t tracefile_count = 0;
+	uint64_t monitor_timer_interval = 0;
+	LTTNG_OPTIONAL(uint64_t) watchdog_timer_interval = {};
+	lttng::sessiond::recording_channel_configuration::buffer_preallocation_policy_t
+		preallocation_policy = lttng::sessiond::recording_channel_configuration::
+			buffer_preallocation_policy_t::PREALLOCATE;
+	nonstd::optional<std::chrono::microseconds> automatic_memory_reclamation_maximal_age;
 	/*
 	 * Node indexed by channel name in the channels' hash table of a session.
 	 */
-	struct lttng_ht_node_str node;
+	struct lttng_ht_node_str node = {};
 	/*
 	 * Node indexed by UST channel object descriptor (handle). Stored in the
 	 * ust_objd hash table in the ust_app object.
 	 */
-	struct lttng_ht_node_ulong ust_objd_node;
+	struct lttng_ht_node_ulong ust_objd_node = {};
 	/* For delayed reclaim */
-	struct rcu_head rcu_head;
+	struct rcu_head rcu_head = {};
 };
 
 struct ust_app_session {
@@ -383,6 +393,7 @@ struct ust_app {
 	struct lttng_ht_node_ulong pid_n = {};
 	struct lttng_ht_node_ulong sock_n = {};
 	struct lttng_ht_node_ulong notify_sock_n = {};
+	struct lttng_ht_node_u64 owner_id_n = {};
 	/*
 	 * This is a list of ust app session that, once the app is going into
 	 * teardown mode, in the RCU call, each node in this list is removed and
@@ -495,7 +506,13 @@ void ust_app_global_update_all_event_notifier_rules();
 
 void ust_app_clean_list();
 int ust_app_ht_alloc();
-struct ust_app *ust_app_find_by_pid(pid_t pid);
+
+bool ust_app_get(ust_app& app);
+void ust_app_put(ust_app *app);
+using ust_app_reference =
+	std::unique_ptr<ust_app, lttng::memory::create_deleter_class<ust_app, ust_app_put>::deleter>;
+
+nonstd::optional<ust_app_reference> ust_app_find_by_pid(pid_t pid);
 struct ust_app_stream *ust_app_alloc_stream();
 int ust_app_recv_registration(int sock, struct ust_register_msg *msg);
 int ust_app_recv_notify(int sock);
@@ -508,7 +525,7 @@ enum lttng_error_code ust_app_snapshot_record(const struct ltt_ust_session *uses
 					      uint64_t nb_packets_per_stream);
 uint64_t ust_app_get_size_one_more_packet_per_stream(const struct ltt_ust_session *usess,
 						     uint64_t cur_nr_packets);
-struct ust_app *ust_app_find_by_sock(int sock);
+nonstd::optional<ust_app_reference> ust_app_find_by_sock(int sock);
 int ust_app_uid_get_channel_runtime_stats(uint64_t ust_session_id,
 					  struct cds_list_head *buffer_reg_uid_list,
 					  struct consumer_output *consumer,
@@ -543,11 +560,7 @@ lttng_ht *ust_app_get_all();
 bool ust_app_supports_notifiers(const struct ust_app *app);
 bool ust_app_supports_counters(const struct ust_app *app);
 
-bool ust_app_get(ust_app& app);
-void ust_app_put(ust_app *app);
-
-using ust_app_reference =
-	std::unique_ptr<ust_app, lttng::memory::create_deleter_class<ust_app, ust_app_put>::deleter>;
+void ust_app_notify_reclaimed_owner_ids(const std::vector<uint32_t>& owners);
 
 #else /* HAVE_LIBLTTNG_UST_CTL */
 
@@ -749,14 +762,28 @@ static inline bool ust_app_supports_counters(const struct ust_app *app __attribu
 	return false;
 }
 
-static inline struct ust_app *ust_app_find_by_sock(int sock __attribute__((unused)))
+static inline bool ust_app_get(ust_app& app __attribute__((unused)))
 {
-	return NULL;
+	return false;
 }
 
-static inline struct ust_app *ust_app_find_by_pid(pid_t pid __attribute__((unused)))
+static inline void ust_app_put(ust_app *app __attribute__((unused)))
 {
-	return NULL;
+}
+
+using ust_app_reference =
+	std::unique_ptr<ust_app, lttng::memory::create_deleter_class<ust_app, ust_app_put>::deleter>;
+
+static inline nonstd::optional<ust_app_reference> ust_app_find_by_sock(int sock
+								       __attribute__((unused)))
+{
+	return nonstd::nullopt;
+}
+
+static inline nonstd::optional<ust_app_reference> ust_app_find_by_pid(pid_t pid
+								      __attribute__((unused)))
+{
+	return nonstd::nullopt;
 }
 
 static inline uint64_t
@@ -827,11 +854,8 @@ static inline int ust_app_release_object(struct ust_app *app __attribute__((unus
 	return 0;
 }
 
-static inline void ust_app_get(ust_app& app __attribute__((unused)))
-{
-}
-
-static inline void ust_app_put(ust_app *app __attribute__((unused)))
+static inline void ust_app_notify_reclaimed_owner_ids(const std::vector<uint32_t>& owners
+						      __attribute__((unused)))
 {
 }
 
