@@ -9,6 +9,7 @@
 #include "../loglevel.hpp"
 #include "../uprobe.hpp"
 #include "common/argpar-utils/argpar-utils.hpp"
+#include "common/ctl/format.hpp"
 #include "common/dynamic-array.hpp"
 #include "common/mi-lttng.hpp"
 #include "common/string-utils/string-utils.hpp"
@@ -18,6 +19,7 @@
 #include <lttng/domain-internal.hpp>
 
 #include <ctype.h>
+#include <exception>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
@@ -26,6 +28,7 @@
 #include "common/filter/filter-ast.hpp"
 #include "common/filter/filter-ir.hpp"
 
+#include <lttng/condition/condition-internal.hpp>
 #include <lttng/event-rule/event-rule-internal.hpp>
 #include <lttng/lttng.h>
 
@@ -65,6 +68,22 @@ enum {
 	OPT_PATH,
 
 	OPT_CAPTURE,
+
+	OPT_SESSION_NAME,
+	OPT_THRESHOLD_SIZE,
+
+	OPT_CHANNEL_NAME,
+	OPT_DOMAIN,
+	OPT_THRESHOLD_RATIO,
+};
+
+static const struct argpar_opt_descr buffer_usage_opt_descriptions[] = {
+	{ OPT_SESSION_NAME, 's', "session", true },
+	{ OPT_CHANNEL_NAME, 'c', "channel", true },
+	{ OPT_DOMAIN, 'd', "domain", true },
+	{ OPT_THRESHOLD_SIZE, 't', "threshold-size", true },
+	{ OPT_THRESHOLD_RATIO, 'r', "threshold-ratio", true },
+	ARGPAR_OPT_DESCR_SENTINEL
 };
 
 static const struct argpar_opt_descr event_rule_opt_descrs[] = {
@@ -83,6 +102,17 @@ static const struct argpar_opt_descr event_rule_opt_descrs[] = {
 	ARGPAR_OPT_DESCR_SENTINEL
 };
 
+static const struct argpar_opt_descr session_consumed_size_opt_descriptions[] = {
+	{ OPT_SESSION_NAME, 's', "session", true },
+	{ OPT_THRESHOLD_SIZE, 't', "threshold-size", true },
+	ARGPAR_OPT_DESCR_SENTINEL,
+};
+
+static const struct argpar_opt_descr session_rotation_opt_descriptions[] = {
+	{ OPT_SESSION_NAME, 's', "session", true },
+	ARGPAR_OPT_DESCR_SENTINEL,
+};
+
 static bool has_syscall_prefix(const char *arg)
 {
 	bool matches = false;
@@ -644,8 +674,291 @@ struct parse_event_rule_res {
 	/* Array of `struct lttng_event_expr *` */
 	struct lttng_dynamic_pointer_array capture_descriptors;
 };
+struct parse_session_consumed_size_res {
+	bool success = false;
+	std::string session_name;
+	uint64_t threshold_size_bytes = 0;
+};
+struct parse_buffer_usage_res {
+	bool success = false;
+	std::string session_name;
+	std::string channel_name;
+	enum lttng_domain_type domain_type = LTTNG_DOMAIN_NONE;
+	double threshold_ratio = 0.0;
+	uint64_t threshold_bytes = 0;
+	bool is_threshold_bytes = false;
+};
+struct parse_session_rotation_res {
+	bool success = false;
+	std::string session_name;
+};
 } /* namespace */
 
+static struct parse_buffer_usage_res
+parse_buffer_usage(int *argc, const char ***argv, int argc_offset, const char *condition_cli_name)
+{
+	bool error = false, has_threshold_bytes = false, has_threshold_ratio = false;
+	parse_buffer_usage_res res;
+	std::string prefix;
+	auto argpar_iter = lttng::make_unique_wrapper<struct argpar_iter, argpar_iter_destroy>(
+		argpar_iter_create(*argc, *argv, buffer_usage_opt_descriptions));
+	auto argpar_item =
+		lttng::make_unique_wrapper<const struct argpar_item, argpar_item_destroy>(
+			(const struct argpar_item *) nullptr);
+
+	try {
+		prefix = fmt::format("While parsing condition `{}`: ", condition_cli_name);
+	} catch (const std::exception& e) {
+		ERR_FMT("Failed to format prefix string for `{}`: {}",
+			condition_cli_name,
+			e.what());
+		return res;
+	}
+
+	if (!argpar_iter) {
+		ERR_FMT("{}failed to create argpar-iter", prefix);
+		return res;
+	}
+
+	while (true) {
+		const struct argpar_item *temp = nullptr;
+		parse_next_item_status status = parse_next_item(
+			argpar_iter.get(), &temp, argc_offset, *argv, false, nullptr, nullptr);
+
+		argpar_item.reset(temp);
+		if (status == PARSE_NEXT_ITEM_STATUS_ERROR ||
+		    status == PARSE_NEXT_ITEM_STATUS_ERROR_MEMORY) {
+			error = true;
+			ERR_FMT("{}parse_next_item error {}", prefix, status);
+			break;
+		} else if (status == PARSE_NEXT_ITEM_STATUS_END) {
+			break;
+		}
+
+		LTTNG_ASSERT(status == PARSE_NEXT_ITEM_STATUS_OK);
+		if (argpar_item_type(argpar_item.get()) == ARGPAR_ITEM_TYPE_OPT) {
+			const auto *descr = argpar_item_opt_descr(argpar_item.get());
+			const auto *arg = argpar_item_opt_arg(argpar_item.get());
+
+			switch (descr->id) {
+			case OPT_SESSION_NAME:
+				try {
+					res.session_name = arg;
+				} catch (const std::exception& e) {
+					error = true;
+					ERR_FMT("{}Failed to assign session_name: {}",
+						prefix,
+						e.what());
+				}
+
+				break;
+			case OPT_CHANNEL_NAME:
+				try {
+					res.channel_name = arg;
+				} catch (const std::exception& e) {
+					error = true;
+					ERR_FMT("{}Failed to assign channel_name: {}",
+						prefix,
+						e.what());
+				}
+
+				break;
+			case OPT_DOMAIN:
+				if (res.domain_type != LTTNG_DOMAIN_NONE) {
+					error = true;
+					ERR_FMT("{}domain type already set.", prefix);
+					break;
+				}
+
+				if (lttng_domain_type_parse(arg, &res.domain_type) != LTTNG_OK) {
+					error = true;
+					res.domain_type = LTTNG_DOMAIN_NONE;
+					ERR_FMT("{}Unable to parse `{}` into LTTng domain type",
+						prefix,
+						arg);
+					break;
+				}
+
+				break;
+			case OPT_THRESHOLD_SIZE:
+				if (has_threshold_ratio || has_threshold_bytes) {
+					error = true;
+					ERR_FMT("{}only one of `-r/--threshold-ratio` and `-t/--threshold-size` may be given",
+						prefix);
+					break;
+				}
+
+				if (utils_parse_size_suffix(arg, &res.threshold_bytes) < 0) {
+					error = true;
+					ERR_FMT("{}wrong value in `-t/--threshold-size` parameter: `{}`",
+						prefix,
+						arg);
+					break;
+				}
+
+				res.is_threshold_bytes = true;
+				has_threshold_bytes = true;
+				break;
+			case OPT_THRESHOLD_RATIO:
+				if (has_threshold_ratio or has_threshold_bytes) {
+					error = true;
+					ERR_FMT("{}only one of `-r/--threshold-ratio` and `-t/--threshold-size` may be given",
+						prefix);
+					break;
+				}
+
+				try {
+					res.threshold_ratio = std::stod(arg);
+				} catch (const std::exception& e) {
+					error = true;
+					ERR_FMT("Failed to convert `{}` to double: {}",
+						arg,
+						e.what());
+					break;
+				}
+
+				has_threshold_ratio = true;
+				res.is_threshold_bytes = false;
+				break;
+			default:
+				abort();
+			}
+		} else {
+			const auto *arg = argpar_item_non_opt_arg(argpar_item.get());
+			error = true;
+			ERR_FMT("{}unexpected argument '{}'", prefix, arg);
+			break;
+		}
+	}
+
+	const auto consumed_args = argpar_iter_ingested_orig_args(argpar_iter.get());
+
+	*argc -= consumed_args;
+	*argv += consumed_args;
+	if (res.session_name.empty()) {
+		error = true;
+		ERR_FMT("{}`-s/--session` is required", prefix);
+	}
+
+	if (res.channel_name.empty()) {
+		error = true;
+		ERR_FMT("{}`-c/--channel` is required", prefix);
+	}
+
+	if (res.domain_type == LTTNG_DOMAIN_NONE) {
+		error = true;
+		ERR_FMT("{}`-d/--domain` is required", prefix);
+	}
+
+	if (!has_threshold_ratio && !has_threshold_bytes) {
+		error = true;
+		ERR_FMT("{}One of `-r/--threshold-ratio` or `-t/--threshold-size` is required",
+			prefix);
+	}
+
+	res.success = !error;
+	return res;
+}
+
+static struct lttng_condition *
+_handle_condition_buffer_usage(int *argc,
+			       const char ***argv,
+			       int argc_offset,
+			       enum lttng_condition_type condition_type,
+			       const char *condition_cli_name)
+{
+	enum lttng_condition_status status;
+	const auto result = parse_buffer_usage(argc, argv, argc_offset, condition_cli_name);
+	if (!result.success) {
+		ERR("Failed to parse buffer-usage arguments");
+		return nullptr;
+	}
+
+	auto condition = [condition_type]() {
+		struct lttng_condition *raw_condition = nullptr;
+		switch (condition_type) {
+		case LTTNG_CONDITION_TYPE_BUFFER_USAGE_LOW:
+			raw_condition = lttng_condition_buffer_usage_low_create();
+			break;
+		case LTTNG_CONDITION_TYPE_BUFFER_USAGE_HIGH:
+			raw_condition = lttng_condition_buffer_usage_high_create();
+			break;
+		default:
+			break;
+		}
+
+		return lttng::make_unique_wrapper<struct lttng_condition, lttng_condition_destroy>(
+			raw_condition);
+	}();
+	if (!condition) {
+		ERR("Failed to create lttng_condition");
+		return nullptr;
+	}
+
+	if (result.is_threshold_bytes) {
+		status = lttng_condition_buffer_usage_set_threshold(condition.get(),
+								    result.threshold_bytes);
+		if (status != LTTNG_CONDITION_STATUS_OK) {
+			ERR_FMT("Failed to set buffer-usage condition threshold bytes: {}", status);
+			return nullptr;
+		}
+	} else {
+		status = lttng_condition_buffer_usage_set_threshold_ratio(condition.get(),
+									  result.threshold_ratio);
+		if (status != LTTNG_CONDITION_STATUS_OK) {
+			ERR_FMT("Failed to set buffer-usage condition threshold ratio: {}", status);
+			return nullptr;
+		}
+	}
+
+	status = lttng_condition_buffer_usage_set_session_name(condition.get(),
+							       result.session_name.c_str());
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set buffer-usage condition session name: {}", status);
+		return nullptr;
+	}
+
+	status = lttng_condition_buffer_usage_set_channel_name(condition.get(),
+							       result.channel_name.c_str());
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set buffer-usage condition channel name: {}", status);
+		return nullptr;
+	}
+
+	status = lttng_condition_buffer_usage_set_domain_type(condition.get(), result.domain_type);
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set buffer-usage condition domain type: {}", status);
+		return nullptr;
+	}
+
+	if (!lttng_condition_validate(condition.get())) {
+		ERR("Failed to validate condition");
+		return nullptr;
+	}
+
+	return condition.release();
+}
+
+static struct lttng_condition *
+handle_condition_buffer_usage_ge(int *argc, const char ***argv, int argc_offset)
+{
+	return _handle_condition_buffer_usage(argc,
+					      argv,
+					      argc_offset,
+					      LTTNG_CONDITION_TYPE_BUFFER_USAGE_HIGH,
+					      "condition-buffer-usage-ge");
+}
+
+static struct lttng_condition *
+handle_condition_buffer_usage_le(int *argc, const char ***argv, int argc_offset)
+{
+	return _handle_condition_buffer_usage(argc,
+					      argv,
+					      argc_offset,
+					      LTTNG_CONDITION_TYPE_BUFFER_USAGE_LOW,
+					      "condition-buffer-usage-le");
+}
+
 static struct parse_event_rule_res parse_event_rule(int *argc, const char ***argv, int argc_offset)
 {
 	enum lttng_event_rule_type event_rule_type = LTTNG_EVENT_RULE_TYPE_UNKNOWN;
@@ -885,7 +1198,7 @@ static struct parse_event_rule_res parse_event_rule(int *argc, const char ***arg
 	 * Update *argc and *argv so our caller can keep parsing what follows.
 	 */
 	consumed_args = argpar_iter_ingested_orig_args(argpar_iter);
-	LTTNG_ASSERT(consumed_args >= 0);
+
 	*argc -= consumed_args;
 	*argv += consumed_args;
 
@@ -1399,6 +1712,290 @@ end:
 	return c;
 }
 
+static struct parse_session_consumed_size_res
+parse_session_consumed_size(int *argc, const char ***argv, int argc_offset)
+{
+	const char *prefix = "While parsing condition `session-consumed-size-ge`: ";
+	bool has_session_name = false, has_threshold_size = false;
+	struct parse_session_consumed_size_res res;
+	auto argpar_iter = lttng::make_unique_wrapper<struct argpar_iter, argpar_iter_destroy>(
+		argpar_iter_create(*argc, *argv, session_consumed_size_opt_descriptions));
+	auto argpar_item =
+		lttng::make_unique_wrapper<const struct argpar_item, argpar_item_destroy>(
+			(const struct argpar_item *) nullptr);
+
+	if (!argpar_iter) {
+		ERR_FMT("{}failed to create argpar_iter", prefix);
+		return res;
+	}
+
+	while (true) {
+		const struct argpar_item *temp = nullptr;
+
+		const auto status = parse_next_item(
+			argpar_iter.get(), &temp, argc_offset, *argv, false, nullptr, nullptr);
+		argpar_item.reset(temp);
+		if (status == PARSE_NEXT_ITEM_STATUS_ERROR ||
+		    status == PARSE_NEXT_ITEM_STATUS_ERROR_MEMORY) {
+			ERR_FMT("{}parse_next_item error {}", prefix, status);
+			break;
+		} else if (status == PARSE_NEXT_ITEM_STATUS_END) {
+			break;
+		}
+
+		LTTNG_ASSERT(status == PARSE_NEXT_ITEM_STATUS_OK);
+		if (argpar_item_type(argpar_item.get()) == ARGPAR_ITEM_TYPE_OPT) {
+			const struct argpar_opt_descr *descr =
+				argpar_item_opt_descr(argpar_item.get());
+			const char *arg = argpar_item_opt_arg(argpar_item.get());
+
+			switch (descr->id) {
+			case OPT_SESSION_NAME:
+				try {
+					res.session_name = arg;
+				} catch (const std::bad_alloc&) {
+					ERR_FMT("{}failed to allocate memory for session name",
+						prefix);
+					break;
+				}
+
+				has_session_name = true;
+				break;
+			case OPT_THRESHOLD_SIZE:
+				if (utils_parse_size_suffix(arg, &res.threshold_size_bytes) < 0) {
+					ERR_FMT("{}wrong value in `-t/--threshold-size` parameter: `{}`",
+						prefix,
+						arg);
+				} else {
+					has_threshold_size = true;
+				}
+				break;
+			default:
+				abort();
+			}
+		} else {
+			const char *arg = argpar_item_non_opt_arg(argpar_item.get());
+			ERR_FMT("{}unexpected argument `{}`", prefix, arg);
+			break;
+		}
+	}
+
+	const auto consumed_args = argpar_iter_ingested_orig_args(argpar_iter.get());
+
+	*argc -= consumed_args;
+	*argv += consumed_args;
+	res.success = has_threshold_size && has_session_name;
+	if (!has_threshold_size) {
+		ERR_FMT("{}Missing or invalid argument for `-t/--threshold-size`", prefix);
+	}
+
+	if (!has_session_name) {
+		ERR_FMT("{}Missing or invalid argument for `-s/--session`", prefix);
+	}
+
+	return res;
+}
+
+static lttng_condition *
+handle_condition_session_consumed_size(int *argc, const char ***argv, int argc_offset)
+{
+	auto condition = []() {
+		lttng_condition *raw_condition = lttng_condition_session_consumed_size_create();
+
+		return lttng::make_unique_wrapper<lttng_condition, lttng_condition_destroy>(
+			raw_condition);
+	}();
+
+	if (!condition || !condition.get()) {
+		ERR("Failed to create lttng_condition structure");
+		return nullptr;
+	}
+
+	auto result = parse_session_consumed_size(argc, argv, argc_offset);
+	if (!result.success) {
+		ERR("Failed to parse session-consumed-size-ge arguments");
+		return nullptr;
+	}
+
+	auto status = lttng_condition_session_consumed_size_set_session_name(
+		condition.get(), result.session_name.c_str());
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set condition's session name: {}", status);
+		return nullptr;
+	}
+
+	status = lttng_condition_session_consumed_size_set_threshold(condition.get(),
+								     result.threshold_size_bytes);
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set condition's threshold: {}", status);
+		return nullptr;
+	}
+
+	if (!lttng_condition_validate(condition.get())) {
+		ERR("Failed to validate condition");
+		return nullptr;
+	}
+
+	return condition.release();
+}
+
+static struct parse_session_rotation_res parse_session_rotation(int *argc,
+								const char ***argv,
+								int argc_offset,
+								const char *condition_cli_name)
+{
+	parse_session_rotation_res res;
+	bool error = false;
+	std::string prefix;
+	auto argpar_iter = lttng::make_unique_wrapper<struct argpar_iter, argpar_iter_destroy>(
+		argpar_iter_create(*argc, *argv, session_rotation_opt_descriptions));
+	auto argpar_item =
+		lttng::make_unique_wrapper<const struct argpar_item, argpar_item_destroy>(
+			(const struct argpar_item *) nullptr);
+
+	try {
+		prefix = fmt::format("While parsing condition `{}`: ", condition_cli_name);
+	} catch (const std::bad_alloc&) {
+		ERR_FMT("Failed to allocate memory for prefix string for `{}`", condition_cli_name);
+		return res;
+	}
+
+	if (!argpar_iter) {
+		ERR_FMT("{}failed to create argpar-iter", prefix);
+		return res;
+	}
+
+	while (true) {
+		const struct argpar_item *temp = nullptr;
+		auto status = parse_next_item(
+			argpar_iter.get(), &temp, argc_offset, *argv, false, nullptr, nullptr);
+
+		argpar_item.reset(temp);
+		if (status == PARSE_NEXT_ITEM_STATUS_ERROR ||
+		    status == PARSE_NEXT_ITEM_STATUS_ERROR_MEMORY) {
+			error = true;
+			ERR_FMT("{}parse_next_item error {}", prefix, status);
+			break;
+		} else if (status == PARSE_NEXT_ITEM_STATUS_END) {
+			break;
+		}
+
+		LTTNG_ASSERT(status == PARSE_NEXT_ITEM_STATUS_OK);
+		if (argpar_item_type(argpar_item.get()) == ARGPAR_ITEM_TYPE_OPT) {
+			const struct argpar_opt_descr *descr =
+				argpar_item_opt_descr(argpar_item.get());
+			const char *arg = argpar_item_opt_arg(argpar_item.get());
+
+			switch (descr->id) {
+			case OPT_SESSION_NAME:
+				try {
+					res.session_name = arg;
+				} catch (const std::exception& e) {
+					error = true;
+					ERR_FMT("{}Failed to assign session name: {}",
+						prefix,
+						e.what());
+				}
+
+				break;
+			default:
+				ERR_FMT("Unknown description id: {}, arg `{}`",
+					(int) descr->id,
+					arg);
+				abort();
+			}
+		} else {
+			const auto *arg = argpar_item_non_opt_arg(argpar_item.get());
+			error = true;
+			ERR_FMT("{}unexpected argument `{}`", prefix, arg);
+			break;
+		}
+	}
+
+	const auto consumed_args = argpar_iter_ingested_orig_args(argpar_iter.get());
+
+	*argc -= consumed_args;
+	*argv += consumed_args;
+
+	if (res.session_name.empty()) {
+		error = true;
+		ERR_FMT("{}`-s/--session` is required", prefix);
+	}
+
+	res.success = !error;
+	return res;
+}
+
+static struct lttng_condition *
+_handle_condition_session_rotation(int *argc,
+				   const char ***argv,
+				   int argc_offset,
+				   enum lttng_condition_type condition_type,
+				   const char *condition_cli_name)
+{
+	const auto result = parse_session_rotation(argc, argv, argc_offset, condition_cli_name);
+
+	if (!result.success) {
+		ERR_FMT("Failed to parse {} arguments", condition_cli_name);
+		return nullptr;
+	}
+
+	auto condition = [condition_type]() {
+		struct lttng_condition *raw_condition = nullptr;
+		switch (condition_type) {
+		case LTTNG_CONDITION_TYPE_SESSION_ROTATION_COMPLETED:
+			raw_condition = lttng_condition_session_rotation_completed_create();
+			break;
+		case LTTNG_CONDITION_TYPE_SESSION_ROTATION_ONGOING:
+			raw_condition = lttng_condition_session_rotation_ongoing_create();
+			break;
+		default:
+			break;
+		}
+		return lttng::make_unique_wrapper<struct lttng_condition, lttng_condition_destroy>(
+			raw_condition);
+	}();
+
+	if (!condition) {
+		ERR("Failed to create lttng_condition");
+		return nullptr;
+	}
+
+	const auto status = lttng_condition_session_rotation_set_session_name(
+		condition.get(), result.session_name.c_str());
+	if (status != LTTNG_CONDITION_STATUS_OK) {
+		ERR_FMT("Failed to set session rotation condition session name: {}", status);
+		return nullptr;
+	}
+
+	if (!lttng_condition_validate(condition.get())) {
+		ERR("Failed to validate condition");
+		return nullptr;
+	}
+
+	return condition.release();
+}
+
+static struct lttng_condition *
+handle_condition_session_rotation_starts(int *argc, const char ***argv, int argc_offset)
+{
+	return _handle_condition_session_rotation(argc,
+						  argv,
+						  argc_offset,
+						  LTTNG_CONDITION_TYPE_SESSION_ROTATION_ONGOING,
+						  "session-rotation-starts");
+}
+
+static struct lttng_condition *
+handle_condition_session_rotation_finishes(int *argc, const char ***argv, int argc_offset)
+{
+	return _handle_condition_session_rotation(argc,
+						  argv,
+						  argc_offset,
+						  LTTNG_CONDITION_TYPE_SESSION_ROTATION_COMPLETED,
+						  "session-rotation-finishes");
+}
+
 namespace {
 struct condition_descr {
 	const char *name;
@@ -1407,7 +2004,12 @@ struct condition_descr {
 } /* namespace */
 
 static const struct condition_descr condition_descrs[] = {
+	{ "channel-buffer-usage-ge", handle_condition_buffer_usage_ge },
+	{ "channel-buffer-usage-le", handle_condition_buffer_usage_le },
 	{ "event-rule-matches", handle_condition_event },
+	{ "session-consumed-size-ge", handle_condition_session_consumed_size },
+	{ "session-rotation-finishes", handle_condition_session_rotation_finishes },
+	{ "session-rotation-starts", handle_condition_session_rotation_starts },
 };
 
 static void print_valid_condition_names()
