@@ -12,6 +12,7 @@
 
 #include <common/compat/time.hpp>
 #include <common/defaults.hpp>
+#include <common/error.hpp>
 #include <common/mi-lttng.hpp>
 #include <common/path.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
@@ -19,6 +20,7 @@
 #include <common/utils.hpp>
 
 #include <lttng/lttng.h>
+#include <lttng/session-descriptor.h>
 
 #include <ctype.h>
 #include <popt.h>
@@ -36,6 +38,7 @@ static char *opt_url;
 static char *opt_ctrl_url;
 static char *opt_data_url;
 static char *opt_shm_path;
+static char *opt_trace_format;
 static int opt_no_consumer;
 static int opt_no_output;
 static int opt_snapshot;
@@ -80,6 +83,7 @@ static struct poptOption long_options[] = {
 	  nullptr,
 	  nullptr },
 	{ "shm-path", 0, POPT_ARG_STRING, &opt_shm_path, 0, nullptr, nullptr },
+	{ "trace-format", 'F', POPT_ARG_STRING, &opt_trace_format, 0, nullptr, nullptr },
 	{ nullptr, 0, 0, nullptr, 0, nullptr, nullptr }
 };
 
@@ -249,21 +253,37 @@ static struct lttng_session_descriptor *create_session_descriptor(const char *se
 	}
 	if (!descriptor) {
 		ERR("Failed to initialize session creation command.");
-	} else {
-		/*
-		 * Auto-launch the relay daemon when a live session
-		 * is created using default URLs.
-		 */
-		if (!opt_url && !opt_ctrl_url && !opt_data_url && opt_live_timer &&
-		    !check_relayd()) {
-			int ret;
-			const char *pathname = opt_relayd_path ?: INSTALL_BIN_PATH "/lttng-relayd";
+		goto end;
+	}
 
-			ret = spawn_relayd(pathname, 0);
-			if (ret < 0) {
-				lttng_session_descriptor_destroy(descriptor);
-				descriptor = nullptr;
-			}
+	if (opt_trace_format) {
+		enum lttng_trace_format format;
+
+		if (strcmp(opt_trace_format, "ctf-2") == 0) {
+			format = LTTNG_TRACE_FORMAT_CTF_2;
+		} else if (strcmp(opt_trace_format, "ctf-1.8") == 0) {
+			format = LTTNG_TRACE_FORMAT_CTF_1_8;
+		} else {
+			format = LTTNG_TRACE_FORMAT_DEFAULT;
+		}
+
+		if (lttng_session_descriptor_set_trace_format(descriptor, format) !=
+		    LTTNG_SESSION_DESCRIPTOR_STATUS_OK) {
+			ERR("Failed to set session trace format");
+			lttng_session_descriptor_destroy(descriptor);
+			descriptor = nullptr;
+			goto end;
+		}
+	}
+
+	if (!opt_url && !opt_ctrl_url && !opt_data_url && opt_live_timer && !check_relayd()) {
+		int ret;
+		const char *pathname = opt_relayd_path ?: INSTALL_BIN_PATH "/lttng-relayd";
+
+		ret = spawn_relayd(pathname, 0);
+		if (ret < 0) {
+			lttng_session_descriptor_destroy(descriptor);
+			descriptor = nullptr;
 		}
 	}
 end:
@@ -313,6 +333,17 @@ static int create_session(const char *session_name)
 		}
 	}
 
+	if (opt_trace_format) {
+		if (strcmp(opt_trace_format, "ctf-2") != 0 &&
+		    strcmp(opt_trace_format, "ctf-1.8") != 0 &&
+		    strcmp(opt_trace_format, "default") != 0) {
+			ERR_FMT("Invalid trace format `{}`; expecting `ctf-1.8`, `ctf-2`, or `default`",
+				opt_trace_format);
+			ret = CMD_ERROR;
+			goto error;
+		}
+	}
+
 	if (opt_snapshot && opt_live_timer) {
 		ERR("Snapshot and live modes are mutually exclusive.");
 		ret = CMD_ERROR;
