@@ -11,6 +11,7 @@
 
 #include <common/defaults.hpp>
 #include <common/lttng-kernel.hpp>
+#include <common/make-unique-wrapper.hpp>
 #include <common/mi-lttng.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
 #include <common/utils.hpp>
@@ -35,10 +36,29 @@ static char *opt_output;
 static int opt_buffer_type = -1;
 static enum lttng_channel_allocation_policy opt_allocation_policy =
 	DEFAULT_CHANNEL_ALLOCATION_POLICY;
+
+static enum lttng_channel_preallocation_policy opt_preallocation_policy =
+	DEFAULT_CHANNEL_PREALLOCATION_POLICY;
+
 static struct {
 	bool set;
 	uint64_t interval;
 } opt_monitor_timer;
+static struct {
+	bool set;
+	uint64_t interval;
+} opt_watchdog_timer;
+enum class auto_reclaim_memory_strategy {
+	OFF,
+	CONSUMED,
+	OLDER_THAN,
+};
+
+static struct {
+	bool set;
+	auto_reclaim_memory_strategy strategy;
+	uint64_t older_than_age;
+} opt_auto_reclaim_memory;
 static struct {
 	bool set;
 	int64_t value;
@@ -60,6 +80,7 @@ enum {
 	OPT_NUM_SUBBUF,
 	OPT_SWITCH_TIMER,
 	OPT_MONITOR_TIMER,
+	OPT_AUTO_RECLAIM_MEMORY,
 	OPT_READ_TIMER,
 	OPT_USERSPACE,
 	OPT_LIST_OPTIONS,
@@ -68,6 +89,8 @@ enum {
 	OPT_BLOCKING_TIMEOUT,
 	OPT_BUFFER_OWNERSHIP,
 	OPT_BUFFER_ALLOCATION,
+	OPT_WATCHDOG_TIMER,
+	OPT_BUFFER_PREALLOCATION,
 };
 
 static struct lttng_handle *handle;
@@ -88,6 +111,8 @@ static struct poptOption long_options[] = {
 	{ "num-subbuf", 0, POPT_ARG_INT, nullptr, OPT_NUM_SUBBUF, nullptr, nullptr },
 	{ "switch-timer", 0, POPT_ARG_INT, nullptr, OPT_SWITCH_TIMER, nullptr, nullptr },
 	{ "monitor-timer", 0, POPT_ARG_INT, nullptr, OPT_MONITOR_TIMER, nullptr, nullptr },
+	{ "watchdog-timer", 0, POPT_ARG_INT, nullptr, OPT_WATCHDOG_TIMER, nullptr, nullptr },
+	{ "auto-reclaim-memory", 0, POPT_ARG_STRING, nullptr, OPT_AUTO_RECLAIM_MEMORY, nullptr, nullptr },
 	{ "read-timer", 0, POPT_ARG_INT, nullptr, OPT_READ_TIMER, nullptr, nullptr },
 	{ "list-options", 0, POPT_ARG_NONE, nullptr, OPT_LIST_OPTIONS, nullptr, nullptr },
 	{ "output", 0, POPT_ARG_STRING, &opt_output, 0, nullptr, nullptr },
@@ -96,6 +121,7 @@ static struct poptOption long_options[] = {
 	{ "buffers-global", 0, POPT_ARG_VAL, &opt_buffer_type, LTTNG_BUFFER_GLOBAL, nullptr, nullptr },
 	{ "buffer-ownership", 0, POPT_ARG_STRING, nullptr, OPT_BUFFER_OWNERSHIP, nullptr, nullptr },
 	{ "buffer-allocation", 0, POPT_ARG_STRING, nullptr, OPT_BUFFER_ALLOCATION, nullptr, nullptr },
+	{ "buffer-preallocation", 0, POPT_ARG_STRING, nullptr, OPT_BUFFER_PREALLOCATION, nullptr, nullptr },
 	{ "tracefile-size", 'C', POPT_ARG_INT, nullptr, OPT_TRACEFILE_SIZE, nullptr, nullptr },
 	{ "tracefile-count", 'W', POPT_ARG_INT, nullptr, OPT_TRACEFILE_COUNT, nullptr, nullptr },
 	{ "blocking-timeout", 0, POPT_ARG_INT, nullptr, OPT_BLOCKING_TIMEOUT, nullptr, nullptr },
@@ -142,6 +168,12 @@ static void set_default_attr(struct lttng_domain *dom)
 	}
 }
 
+static size_t real_subbuff_count(const struct lttng_channel& channel)
+{
+	/* An additional sub-buffer is allocated to support sparse-buffers. */
+	return channel.attr.num_subbuf + 1;
+}
+
 static bool system_has_memory_for_channel_buffers(char *session_name,
 						  struct lttng_channel *channel,
 						  uint64_t *bytes_required,
@@ -163,16 +195,18 @@ static bool system_has_memory_for_channel_buffers(char *session_name,
 		return false;
 	}
 
-	if (channel->attr.num_subbuf > UINT64_MAX / channel->attr.subbuf_size) {
+	const auto total_num_subbuf = real_subbuff_count(*channel);
+
+	if (total_num_subbuf > UINT64_MAX / channel->attr.subbuf_size) {
 		/* Overflow */
 		ERR_FMT("Integer overflow calculating total buffer size per CPU on channel '{}': num_subbuf={}, subbuf_size={}",
 			channel->name,
-			channel->attr.num_subbuf,
+			total_num_subbuf,
 			channel->attr.subbuf_size)
 		return false;
 	}
 
-	total_buffer_size_needed_per_cpu = channel->attr.num_subbuf * channel->attr.subbuf_size;
+	total_buffer_size_needed_per_cpu = total_num_subbuf * channel->attr.subbuf_size;
 	try {
 		switch (opt_allocation_policy) {
 		case LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU:
@@ -218,6 +252,11 @@ static int enable_channel(char *session_name, char *channel_list)
 			ret = CMD_ERROR;
 			goto error;
 		}
+		if (opt_watchdog_timer.set) {
+			ERR("Watchdog timer option not supported for kernel domain (-k)");
+			ret = CMD_ERROR;
+			goto error;
+		}
 	}
 
 	/* Create lttng domain */
@@ -258,6 +297,12 @@ static int enable_channel(char *session_name, char *channel_list)
 			ret = CMD_ERROR;
 			goto error;
 		}
+
+		if (opt_watchdog_timer.set && (dom.buf_type != LTTNG_BUFFER_PER_UID)) {
+			ERR("Watchdog timer is only valid for channels with the `user` ownership model");
+			ret = CMD_ERROR;
+			goto error;
+		}
 	} else {
 		/* Checked by the caller. */
 		abort();
@@ -351,6 +396,15 @@ static int enable_channel(char *session_name, char *channel_list)
 				goto error;
 			}
 		}
+		if (opt_watchdog_timer.set) {
+			ret = lttng_channel_set_watchdog_timer_interval(
+				channel, opt_watchdog_timer.interval);
+			if (ret != LTTNG_CHANNEL_STATUS_OK) {
+				ERR("Failed to set the channel's watchdog timer interval");
+				error = 1;
+				goto error;
+			}
+		}
 		if (opt_blocking_timeout.set) {
 			ret = lttng_channel_set_blocking_timeout(channel,
 								 opt_blocking_timeout.value);
@@ -368,6 +422,42 @@ static int enable_channel(char *session_name, char *channel_list)
 			goto error;
 		}
 
+		ret = lttng_channel_set_preallocation_policy(channel, opt_preallocation_policy);
+		if (ret != LTTNG_CHANNEL_STATUS_OK) {
+			ERR("Failed to set the channel's buffer preallocation policy");
+			error = 1;
+			goto error;
+		}
+
+		if (opt_auto_reclaim_memory.set) {
+			uint64_t reclaim_age = 0;
+
+			switch (opt_auto_reclaim_memory.strategy) {
+			case auto_reclaim_memory_strategy::CONSUMED:
+				reclaim_age = 0;
+				ret = lttng_channel_set_automatic_memory_reclamation_policy(
+					channel, reclaim_age);
+				if (ret != LTTNG_CHANNEL_STATUS_OK) {
+					ERR("Failed to set the channel's automatic memory reclamation policy to 'consumed'");
+					error = 1;
+					goto error;
+				}
+				break;
+			case auto_reclaim_memory_strategy::OLDER_THAN:
+				reclaim_age = opt_auto_reclaim_memory.older_than_age;
+				ret = lttng_channel_set_automatic_memory_reclamation_policy(
+					channel, reclaim_age);
+				if (ret != LTTNG_CHANNEL_STATUS_OK) {
+					ERR("Failed to set the channel's automatic memory reclamation policy");
+					error = 1;
+					goto error;
+				}
+				break;
+			default:
+				break;
+			}
+		}
+
 		if (!system_has_memory_for_channel_buffers(
 			    session_name, channel, &bytes_required, &bytes_available)) {
 			ERR_FMT("Not enough system memory available for channel '{}'. At least {}MiB required, {}MiB available",
@@ -672,6 +762,74 @@ int cmd_enable_channels(int argc, const char **argv)
 			    USEC_UNIT);
 			break;
 		}
+		case OPT_WATCHDOG_TIMER:
+		{
+			uint64_t v;
+
+			errno = 0;
+			opt_arg = poptGetOptArg(pc);
+
+			if (utils_parse_time_suffix(opt_arg, &v) < 0) {
+				ERR("Wrong value for --watchdog-timer parameter: %s", opt_arg);
+				ret = CMD_ERROR;
+				goto end;
+			}
+			opt_watchdog_timer.interval = (uint64_t) v;
+			opt_watchdog_timer.set = true;
+			DBG("Channel watchdog timer interval set to %" PRIu64 " %s",
+			    opt_watchdog_timer.interval,
+			    USEC_UNIT);
+			break;
+		}
+		case OPT_AUTO_RECLAIM_MEMORY:
+		{
+			const auto value_ptr =
+				lttng::make_unique_wrapper<char, lttng::memory::free>(
+					poptGetOptArg(pc));
+			const lttng::c_string_view value(value_ptr.get());
+			static constexpr auto older_than_prefix = "older-than:";
+
+			if (value == "off") {
+				opt_auto_reclaim_memory.set = true;
+				opt_auto_reclaim_memory.strategy =
+					auto_reclaim_memory_strategy::OFF;
+				DBG("Automatic memory reclaim disabled");
+			} else if (value == "consumed") {
+				opt_auto_reclaim_memory.set = true;
+				opt_auto_reclaim_memory.strategy =
+					auto_reclaim_memory_strategy::CONSUMED;
+				DBG("Automatic memory reclaim set to when consumed");
+			} else if (strncmp(value.data(),
+					   older_than_prefix,
+					   strlen(older_than_prefix)) == 0) {
+				const auto age_str = value.data() +
+					lttng::c_string_view(older_than_prefix).len();
+				uint64_t age_us;
+
+				if (utils_parse_time_suffix(age_str, &age_us) < 0) {
+					ERR_FMT("Wrong value for `--auto-reclaim-memory=older-than:` parameter: `{}`",
+						age_str);
+					ret = CMD_ERROR;
+					goto end;
+				}
+
+				opt_auto_reclaim_memory.set = true;
+				opt_auto_reclaim_memory.strategy =
+					auto_reclaim_memory_strategy::OLDER_THAN;
+				opt_auto_reclaim_memory.older_than_age = age_us;
+				DBG_FMT("Automatic memory reclaim set to older than {} {}",
+					age_us,
+					USEC_UNIT);
+			} else {
+				ERR_FMT("Wrong value for `--auto-reclaim-memory`: `{}`: "
+					"expecting `off`, `consumed`, or `older-than:AGE`",
+					value.data());
+				ret = CMD_ERROR;
+				goto end;
+			}
+
+			break;
+		}
 		case OPT_BLOCKING_TIMEOUT:
 		{
 			uint64_t v;
@@ -764,7 +922,10 @@ int cmd_enable_channels(int argc, const char **argv)
 			goto end;
 		case OPT_BUFFER_OWNERSHIP:
 		{
-			const lttng::c_string_view ownership(poptGetOptArg(pc));
+			const auto ownership_ptr =
+				lttng::make_unique_wrapper<char, lttng::memory::free>(
+					poptGetOptArg(pc));
+			const lttng::c_string_view ownership(ownership_ptr.get());
 
 			if (ownership == "user") {
 				opt_buffer_type = LTTNG_BUFFER_PER_UID;
@@ -783,7 +944,10 @@ int cmd_enable_channels(int argc, const char **argv)
 		}
 		case OPT_BUFFER_ALLOCATION:
 		{
-			const lttng::c_string_view policy(poptGetOptArg(pc));
+			const auto policy_ptr =
+				lttng::make_unique_wrapper<char, lttng::memory::free>(
+					poptGetOptArg(pc));
+			const lttng::c_string_view policy(policy_ptr.get());
 
 			if (policy == "per-cpu") {
 				opt_allocation_policy = LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU;
@@ -798,6 +962,27 @@ int cmd_enable_channels(int argc, const char **argv)
 			}
 			break;
 		}
+		case OPT_BUFFER_PREALLOCATION:
+		{
+			const auto mode_ptr = lttng::make_unique_wrapper<char, lttng::memory::free>(
+				poptGetOptArg(pc));
+			const lttng::c_string_view mode(mode_ptr.get());
+
+			if (mode == "preallocate") {
+				opt_preallocation_policy =
+					LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE;
+			} else if (mode == "on-demand") {
+				opt_preallocation_policy =
+					LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND;
+			} else {
+				ERR_FMT("Wrong value for --buffer-preallocation: `{}`: "
+					"expecting `preallocate` or `on-demand`",
+					mode.data());
+				ret = CMD_ERROR;
+				goto end;
+			}
+			break;
+		}
 		default:
 			ret = CMD_UNDEFINED;
 			goto end;
