@@ -20,6 +20,7 @@
 #include <common/kernel-consumer/kernel-consumer.hpp>
 #include <common/kernel-ctl/kernel-ctl.hpp>
 #include <common/macros.hpp>
+#include <common/make-unique.hpp>
 #include <common/relayd/relayd.hpp>
 #include <common/urcu.hpp>
 #include <common/ust-consumer/ust-consumer.hpp>
@@ -27,7 +28,6 @@
 
 #include <fcntl.h>
 #include <inttypes.h>
-#include <new>
 #include <sys/mman.h>
 #include <unistd.h>
 
@@ -245,7 +245,8 @@ static ssize_t consumer_stream_consume_mmap(struct lttng_consumer_local_data *ct
 
 	if (the_consumer_data.type == LTTNG_CONSUMER_KERNEL &&
 	    stream->chan->type == CONSUMER_CHANNEL_TYPE_METADATA &&
-	    !utils_force_experimental_ctf_2() && !stream->first_metadata_write_done) {
+	    stream->chan->trace_format != LTTNG_TRACE_FORMAT_CTF_2 &&
+	    !stream->first_metadata_write_done) {
 		/*
 		 * The kernel tracer does not write the CTF 1.x announcement
 		 * comment in the metadata stream, so we do it here on
@@ -338,7 +339,7 @@ static ssize_t consumer_stream_consume_splice(struct lttng_consumer_local_data *
 	return written_bytes;
 }
 
-static int consumer_stream_send_index(struct lttng_consumer_stream *stream,
+static int consumer_stream_send_index(lttng_consumer_stream& stream,
 				      const struct stream_subbuffer *subbuffer,
 				      struct lttng_consumer_local_data *ctx __attribute__((unused)))
 {
@@ -349,12 +350,29 @@ static int consumer_stream_send_index(struct lttng_consumer_stream *stream,
 	 * This is called after consuming the sub-buffer; substract the
 	 * effect this sub-buffer from the offset.
 	 */
-	if (!stream->has_network_destination()) {
-		packet_offset = stream->out_fd_offset - subbuffer->info.data.padded_subbuf_size;
+	if (!stream.has_network_destination()) {
+		packet_offset = stream.out_fd_offset - subbuffer->info.data.padded_subbuf_size;
 	}
 
 	ctf_packet_index_populate(&index, packet_offset, subbuffer);
-	return consumer_stream_write_index(stream, &index);
+	return consumer_stream_write_index(stream, index);
+}
+
+static int try_current_subbuffer_reclamation(lttng_consumer_stream& stream,
+					     const stream_subbuffer *subbuffer [[maybe_unused]],
+					     struct lttng_consumer_local_data *ctx [[maybe_unused]])
+{
+	try {
+		consumer_stream_try_reclaim_current_subbuffer(stream, *subbuffer);
+	} catch (const lttng::runtime_error& e) {
+		ERR_FMT("Failed to reclaim reader sub-buffer: channel_name=`{}`, stream_key={}: {}",
+			stream.chan->name,
+			stream.key,
+			e.what());
+		return -1;
+	}
+
+	return 0;
 }
 
 /*
@@ -517,7 +535,7 @@ end:
 	return ret;
 }
 
-static int consumer_stream_sync_metadata_index(struct lttng_consumer_stream *stream,
+static int consumer_stream_sync_metadata_index(lttng_consumer_stream& stream,
 					       const struct stream_subbuffer *subbuffer,
 					       struct lttng_consumer_local_data *ctx)
 {
@@ -525,20 +543,20 @@ static int consumer_stream_sync_metadata_index(struct lttng_consumer_stream *str
 	int ret;
 
 	/* Block until all the metadata is sent. */
-	pthread_mutex_lock(&stream->metadata_timer_lock);
-	LTTNG_ASSERT(!stream->missed_metadata_flush);
-	stream->waiting_on_metadata = true;
-	pthread_mutex_unlock(&stream->metadata_timer_lock);
+	pthread_mutex_lock(&stream.metadata_timer_lock);
+	LTTNG_ASSERT(!stream.missed_metadata_flush);
+	stream.waiting_on_metadata = true;
+	pthread_mutex_unlock(&stream.metadata_timer_lock);
 
-	ret = consumer_stream_sync_metadata(ctx, stream->session_id);
+	ret = consumer_stream_sync_metadata(ctx, stream.session_id);
 
-	pthread_mutex_lock(&stream->metadata_timer_lock);
-	stream->waiting_on_metadata = false;
-	missed_metadata_flush = stream->missed_metadata_flush;
+	pthread_mutex_lock(&stream.metadata_timer_lock);
+	stream.waiting_on_metadata = false;
+	missed_metadata_flush = stream.missed_metadata_flush;
 	if (missed_metadata_flush) {
-		stream->missed_metadata_flush = false;
+		stream.missed_metadata_flush = false;
 	}
-	pthread_mutex_unlock(&stream->metadata_timer_lock);
+	pthread_mutex_unlock(&stream.metadata_timer_lock);
 	if (ret < 0) {
 		goto end;
 	}
@@ -560,7 +578,7 @@ static int consumer_stream_sync_metadata_index(struct lttng_consumer_stream *str
 	 * beacon and a following trace packet.
 	 */
 	if (missed_metadata_flush) {
-		(void) stream->read_subbuffer_ops.send_live_beacon(stream);
+		(void) stream.read_subbuffer_ops.send_live_beacon(stream);
 	}
 end:
 	return ret;
@@ -662,80 +680,79 @@ static int metadata_stream_pre_consume(struct lttng_consumer_stream *stream,
 	return 0;
 }
 
-static bool stream_is_rotating_to_null_chunk(const struct lttng_consumer_stream *stream)
+static bool stream_is_rotating_to_null_chunk(const struct lttng_consumer_stream& stream)
 {
 	bool rotating_to_null_chunk = false;
 
-	if (stream->rotate_position == -1ULL) {
+	if (stream.rotate_position == -1ULL) {
 		/* No rotation ongoing. */
 		goto end;
 	}
 
-	if (stream->trace_chunk == stream->chan->trace_chunk || !stream->chan->trace_chunk) {
+	if (stream.trace_chunk == stream.chan->trace_chunk || !stream.chan->trace_chunk) {
 		rotating_to_null_chunk = true;
 	}
 end:
 	return rotating_to_null_chunk;
 }
 
-enum consumer_stream_open_packet_status
-consumer_stream_open_packet(struct lttng_consumer_stream *stream)
+enum consumer_stream_open_packet_status consumer_stream_open_packet(lttng_consumer_stream& stream)
 {
 	int ret;
 	enum consumer_stream_open_packet_status status;
 	unsigned long produced_pos_before, produced_pos_after;
 
-	ret = lttng_consumer_sample_snapshot_positions(stream);
+	ret = lttng_consumer_sample_snapshot_positions(&stream);
 	if (ret < 0) {
 		ERR("Failed to snapshot positions before post-rotation empty packet flush: stream id = %" PRIu64
 		    ", channel name = %s, session id = %" PRIu64,
-		    stream->key,
-		    stream->chan->name,
-		    stream->chan->session_id);
+		    stream.key,
+		    stream.chan->name,
+		    stream.chan->session_id);
 		status = CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR;
 		goto end;
 	}
 
-	ret = lttng_consumer_get_produced_snapshot(stream, &produced_pos_before);
+	ret = lttng_consumer_get_produced_snapshot(&stream, &produced_pos_before);
 	if (ret < 0) {
 		ERR("Failed to read produced position before post-rotation empty packet flush: stream id = %" PRIu64
 		    ", channel name = %s, session id = %" PRIu64,
-		    stream->key,
-		    stream->chan->name,
-		    stream->chan->session_id);
+		    stream.key,
+		    stream.chan->name,
+		    stream.chan->session_id);
 		status = CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR;
 		goto end;
 	}
 
-	ret = consumer_stream_flush_buffer(stream, false);
+	ret = consumer_stream_flush_buffer(&stream, false);
 	if (ret) {
 		ERR("Failed to flush an empty packet at rotation point: stream id = %" PRIu64
 		    ", channel name = %s, session id = %" PRIu64,
-		    stream->key,
-		    stream->chan->name,
-		    stream->chan->session_id);
+		    stream.key,
+		    stream.chan->name,
+		    stream.chan->session_id);
 		status = CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR;
 		goto end;
 	}
 
-	ret = lttng_consumer_sample_snapshot_positions(stream);
+	ret = lttng_consumer_sample_snapshot_positions(&stream);
 	if (ret < 0) {
 		ERR("Failed to snapshot positions after post-rotation empty packet flush: stream id = %" PRIu64
 		    ", channel name = %s, session id = %" PRIu64,
-		    stream->key,
-		    stream->chan->name,
-		    stream->chan->session_id);
+		    stream.key,
+		    stream.chan->name,
+		    stream.chan->session_id);
 		status = CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR;
 		goto end;
 	}
 
-	ret = lttng_consumer_get_produced_snapshot(stream, &produced_pos_after);
+	ret = lttng_consumer_get_produced_snapshot(&stream, &produced_pos_after);
 	if (ret < 0) {
 		ERR("Failed to read produced position after post-rotation empty packet flush: stream id = %" PRIu64
 		    ", channel name = %s, session id = %" PRIu64,
-		    stream->key,
-		    stream->chan->name,
-		    stream->chan->session_id);
+		    stream.key,
+		    stream.chan->name,
+		    stream.chan->session_id);
 		status = CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR;
 		goto end;
 	}
@@ -748,7 +765,7 @@ consumer_stream_open_packet(struct lttng_consumer_stream *stream)
 		CONSUMER_STREAM_OPEN_PACKET_STATUS_OPENED :
 		CONSUMER_STREAM_OPEN_PACKET_STATUS_NO_SPACE;
 	if (status == CONSUMER_STREAM_OPEN_PACKET_STATUS_OPENED) {
-		stream->opened_packet_in_current_trace_chunk = true;
+		stream.opened_packet_in_current_trace_chunk = true;
 	}
 
 end:
@@ -763,7 +780,7 @@ end:
  * ring-buffer. In that case, a second attempt is performed after consuming
  * a packet since that will have freed enough space in the ring-buffer.
  */
-static int post_consume_open_new_packet(struct lttng_consumer_stream *stream,
+static int post_consume_open_new_packet(lttng_consumer_stream& stream,
 					const struct stream_subbuffer *subbuffer
 					__attribute__((unused)),
 					struct lttng_consumer_local_data *ctx
@@ -771,7 +788,7 @@ static int post_consume_open_new_packet(struct lttng_consumer_stream *stream,
 {
 	int ret = 0;
 
-	if (!stream->opened_packet_in_current_trace_chunk && stream->trace_chunk &&
+	if (!stream.opened_packet_in_current_trace_chunk && stream.trace_chunk &&
 	    !stream_is_rotating_to_null_chunk(stream)) {
 		const enum consumer_stream_open_packet_status status =
 			consumer_stream_open_packet(stream);
@@ -780,10 +797,10 @@ static int post_consume_open_new_packet(struct lttng_consumer_stream *stream,
 		case CONSUMER_STREAM_OPEN_PACKET_STATUS_OPENED:
 			DBG("Opened a packet after consuming a packet rotation: stream id = %" PRIu64
 			    ", channel name = %s, session id = %" PRIu64,
-			    stream->key,
-			    stream->chan->name,
-			    stream->chan->session_id);
-			stream->opened_packet_in_current_trace_chunk = true;
+			    stream.key,
+			    stream.chan->name,
+			    stream.chan->session_id);
+			stream.opened_packet_in_current_trace_chunk = true;
 			break;
 		case CONSUMER_STREAM_OPEN_PACKET_STATUS_NO_SPACE:
 			/*
@@ -794,10 +811,10 @@ static int post_consume_open_new_packet(struct lttng_consumer_stream *stream,
 			 */
 			DBG("No space left to open a packet after consuming a packet: stream id = %" PRIu64
 			    ", channel name = %s, session id = %" PRIu64,
-			    stream->key,
-			    stream->chan->name,
-			    stream->chan->session_id);
-			stream->opened_packet_in_current_trace_chunk = true;
+			    stream.key,
+			    stream.chan->name,
+			    stream.chan->session_id);
+			stream.opened_packet_in_current_trace_chunk = true;
 			break;
 		case CONSUMER_STREAM_OPEN_PACKET_STATUS_ERROR:
 			/* Logged by callee. */
@@ -807,28 +824,28 @@ static int post_consume_open_new_packet(struct lttng_consumer_stream *stream,
 			abort();
 		}
 
-		stream->opened_packet_in_current_trace_chunk = true;
+		stream.opened_packet_in_current_trace_chunk = true;
 	}
 
 end:
 	return ret;
 }
 
-static int post_consume_update_metadata_coherent_flag(struct lttng_consumer_stream *stream,
+static int post_consume_update_metadata_coherent_flag(lttng_consumer_stream& stream,
 						      const struct stream_subbuffer *subbuffer,
 						      struct lttng_consumer_local_data *ctx
 						      __attribute__((unused)))
 {
-	const auto original_state = stream->is_metadata_coherent;
+	const auto original_state = stream.is_metadata_coherent;
 
 	if (subbuffer->info.metadata.coherent.is_set) {
-		stream->is_metadata_coherent = subbuffer->info.metadata.coherent.value;
+		stream.is_metadata_coherent = subbuffer->info.metadata.coherent.value;
 	}
 
-	if (original_state != stream->is_metadata_coherent) {
+	if (original_state != stream.is_metadata_coherent) {
 		DBG_FMT("Metadata coherence state changed: session_id={}, stream_key={}, old_value={}, new_value={}",
-			stream->chan->session_id,
-			stream->key,
+			stream.chan->session_id,
+			stream.key,
 			original_state,
 			!original_state);
 	}
@@ -836,7 +853,7 @@ static int post_consume_update_metadata_coherent_flag(struct lttng_consumer_stre
 	return 0;
 }
 
-static int post_consume_check_metadata_rotate_ready(struct lttng_consumer_stream *stream,
+static int post_consume_check_metadata_rotate_ready(lttng_consumer_stream& stream,
 						    const struct stream_subbuffer *subbuffer
 						    [[maybe_unused]],
 						    struct lttng_consumer_local_data *ctx
@@ -850,37 +867,37 @@ static int post_consume_check_metadata_rotate_ready(struct lttng_consumer_stream
 	 */
 	bool is_metadata_coherent = false;
 
-	if (caa_likely(stream->tracer_supports_metadata_coherency_indicator)) {
-		is_metadata_coherent = stream->is_metadata_coherent;
+	if (caa_likely(stream.tracer_supports_metadata_coherency_indicator)) {
+		is_metadata_coherent = stream.is_metadata_coherent;
 	} else {
 		/*
 		 * If the tracer does not support the metadata coherency indicator, assume
 		 * metadata is coherent if none of it is left to be written. This is racy, but
 		 * it's the best we can do with these older tracers.
 		 */
-		const int snapshot_ret = lttng_consumer_sample_snapshot_positions(stream);
+		const int snapshot_ret = lttng_consumer_sample_snapshot_positions(&stream);
 		if (snapshot_ret < 0 && snapshot_ret != -ENODATA && snapshot_ret != -EAGAIN) {
 			ERR_FMT("Failed to sample snapshot positions to determine metadata coherence: session_id={}, stream_key={}",
-				stream->chan->session_id,
-				stream->key);
+				stream.chan->session_id,
+				stream.key);
 		} else if (snapshot_ret == 0) {
 			unsigned long produced_pos, consumed_pos;
 
 			const auto get_produced_ret =
-				lttng_consumer_get_produced_snapshot(stream, &produced_pos);
+				lttng_consumer_get_produced_snapshot(&stream, &produced_pos);
 			if (get_produced_ret < 0) {
 				ERR_FMT("Failed to sample produced position when determining if metadata is coherent: session_id={}, stream_key={}",
-					stream->chan->session_id,
-					stream->key);
+					stream.chan->session_id,
+					stream.key);
 				return -1;
 			}
 
 			const auto get_consumed_ret =
-				lttng_consumer_get_consumed_snapshot(stream, &consumed_pos);
+				lttng_consumer_get_consumed_snapshot(&stream, &consumed_pos);
 			if (get_consumed_ret < 0) {
 				ERR_FMT("Failed to sample consumed position when determining if metadata is coherent: session_id={}, stream_key={}",
-					stream->chan->session_id,
-					stream->key);
+					stream.chan->session_id,
+					stream.key);
 				return -1;
 			}
 		} else {
@@ -889,24 +906,24 @@ static int post_consume_check_metadata_rotate_ready(struct lttng_consumer_stream
 		}
 	}
 
-	if (is_metadata_coherent && stream->pending_metadata_rotation.has_value()) {
-		stream->rotate_ready = true;
+	if (is_metadata_coherent && stream.pending_metadata_rotation.has_value()) {
+		stream.rotate_ready = true;
 	}
 
 	return 0;
 }
 
-static int post_consume_rotate_metadata_relayd_stream(struct lttng_consumer_stream *stream,
+static int post_consume_rotate_metadata_relayd_stream(lttng_consumer_stream& stream,
 						      const struct stream_subbuffer *subbuffer
 						      [[maybe_unused]],
 						      struct lttng_consumer_local_data *ctx
 						      [[maybe_unused]])
 {
-	if (stream->rotate_ready) {
-		LTTNG_ASSERT(stream->is_metadata_coherent &&
-			     stream->pending_metadata_rotation.has_value());
+	if (stream.rotate_ready) {
+		LTTNG_ASSERT(stream.is_metadata_coherent &&
+			     stream.pending_metadata_rotation.has_value());
 		return lttng_consumer_rotate_metadata_relayd_stream(
-			*stream, stream->pending_metadata_rotation.value());
+			stream, stream.pending_metadata_rotation.value());
 	}
 
 	return 0;
@@ -925,15 +942,9 @@ struct lttng_consumer_stream *consumer_stream_create(struct lttng_consumer_chann
 						     unsigned int monitor)
 {
 	int ret;
-	struct lttng_consumer_stream *stream;
 	const lttng::urcu::read_lock_guard read_lock;
 
-	stream = new (std::nothrow) lttng_consumer_stream();
-	if (stream == nullptr) {
-		PERROR("malloc struct lttng_consumer_stream");
-		ret = -ENOMEM;
-		goto end;
-	}
+	auto stream = lttng::make_unique<lttng_consumer_stream>();
 
 	if (trace_chunk && !lttng_trace_chunk_get(trace_chunk)) {
 		ERR("Failed to acquire trace chunk reference during the creation of a stream");
@@ -1009,6 +1020,10 @@ struct lttng_consumer_stream *consumer_stream_create(struct lttng_consumer_chann
 	     stream->net_seq_idx,
 	     stream->session_id);
 
+	if (channel->subbuffer_count) {
+		stream->subbuffer_transaction_states.resize(channel->subbuffer_count.value());
+	}
+
 	lttng_dynamic_array_init(
 		&stream->read_subbuffer_ops.post_consume_cbs, sizeof(post_consume_cb), nullptr);
 
@@ -1062,6 +1077,18 @@ struct lttng_consumer_stream *consumer_stream_create(struct lttng_consumer_chann
 			goto error;
 		}
 
+		if (the_consumer_data.type != LTTNG_CONSUMER_KERNEL) {
+			const post_consume_cb reclaim_post_consume =
+				try_current_subbuffer_reclamation;
+			ret = lttng_dynamic_array_add_element(
+				&stream->read_subbuffer_ops.post_consume_cbs,
+				&reclaim_post_consume);
+			if (ret) {
+				PERROR("Failed to add `sub-buffer reclaim` callback to stream's post consumption callbacks");
+				goto error;
+			}
+		}
+
 		ret = lttng_dynamic_array_add_element(&stream->read_subbuffer_ops.post_consume_cbs,
 						      &post_consume_open_new_packet_);
 		if (ret) {
@@ -1081,13 +1108,12 @@ struct lttng_consumer_stream *consumer_stream_create(struct lttng_consumer_chann
 		stream->read_subbuffer_ops.consume_subbuffer = consumer_stream_consume_splice;
 	}
 
-	return stream;
+	return stream.release();
 
 error:
 	lttng_trace_chunk_put(stream->trace_chunk);
 	lttng_dynamic_array_reset(&stream->read_subbuffer_ops.post_consume_cbs);
-	delete stream;
-end:
+
 	if (alloc_ret) {
 		*alloc_ret = ret;
 	}
@@ -1399,24 +1425,21 @@ void consumer_stream_destroy(struct lttng_consumer_stream *stream, struct lttng_
  *
  * Return 0 on success or else a negative value.
  */
-int consumer_stream_write_index(struct lttng_consumer_stream *stream,
-				struct ctf_packet_index *element)
+int consumer_stream_write_index(lttng_consumer_stream& stream, const ctf_packet_index& element)
 {
 	int ret;
 
-	LTTNG_ASSERT(stream);
-	LTTNG_ASSERT(element);
-
 	const lttng::urcu::read_lock_guard read_lock;
-	if (stream->has_network_destination()) {
+	if (stream.has_network_destination()) {
 		struct consumer_relayd_sock_pair *relayd;
-		relayd = consumer_find_relayd(stream->net_seq_idx);
+
+		relayd = consumer_find_relayd(stream.net_seq_idx);
 		if (relayd) {
 			pthread_mutex_lock(&relayd->ctrl_sock_mutex);
-			ret = relayd_send_index(&relayd->control_sock,
+			ret = relayd_send_index(relayd->control_sock,
 						element,
-						stream->relayd_stream_id,
-						stream->next_net_seq_num - 1);
+						stream.relayd_stream_id,
+						stream.next_net_seq_num - 1);
 			if (ret < 0) {
 				/*
 				 * Communication error with lttng-relayd,
@@ -1430,12 +1453,12 @@ int consumer_stream_write_index(struct lttng_consumer_stream *stream,
 			pthread_mutex_unlock(&relayd->ctrl_sock_mutex);
 		} else {
 			ERR("Stream %" PRIu64 " relayd ID %" PRIu64 " unknown. Can't write index.",
-			    stream->key,
-			    stream->net_seq_idx);
+			    stream.key,
+			    stream.net_seq_idx);
 			ret = -1;
 		}
 	} else {
-		if (lttng_index_file_write(stream->index_file, element)) {
+		if (lttng_index_file_write(stream.index_file, &element)) {
 			ret = -1;
 		} else {
 			ret = 0;
@@ -1651,3 +1674,53 @@ int consumer_stream_flush_buffer(struct lttng_consumer_stream *stream, bool prod
 end:
 	return ret;
 }
+
+int consumer_stream_send_live_beacon(lttng_consumer_stream& stream,
+				     uint64_t timestamp,
+				     uint64_t stream_id)
+{
+	ctf_packet_index index = {};
+
+	index.stream_id = htobe64(stream_id);
+	index.timestamp_end = htobe64(timestamp);
+
+	return consumer_stream_write_index(stream, index);
+}
+
+void consumer_stream_try_reclaim_current_subbuffer(lttng_consumer_stream& stream,
+						   const stream_subbuffer& subbuffer)
+{
+	switch (the_consumer_data.type) {
+	case LTTNG_CONSUMER_KERNEL:
+		LTTNG_THROW_UNSUPPORTED_ERROR(
+			"Buffer reclamation is not available for kernel consumer streams");
+	case LTTNG_CONSUMER32_UST:
+	case LTTNG_CONSUMER64_UST:
+		lttng_ustconsumer_try_reclaim_current_subbuffer(stream, subbuffer);
+		break;
+	default:
+		ERR("Unknown consumer_data type");
+		abort();
+	}
+}
+
+lttng::consumer::memory_reclaim_result
+consumer_stream_reclaim_memory(lttng_consumer_stream& stream,
+			       const std::chrono::microseconds& age_limit,
+			       bool require_consumed,
+			       const nonstd::optional<std::uint64_t>& request_token)
+{
+	switch (the_consumer_data.type) {
+	case LTTNG_CONSUMER_KERNEL:
+		LTTNG_THROW_UNSUPPORTED_ERROR(
+			"Buffer reclamation is not available for kernel consumer streams");
+	case LTTNG_CONSUMER32_UST:
+	case LTTNG_CONSUMER64_UST:
+		return lttng_ustconsumer_reclaim_stream_memory(
+			stream, age_limit, require_consumed, request_token);
+		break;
+	default:
+		ERR("Unknown consumer_data type");
+		abort();
+	}
+}
