@@ -11,108 +11,41 @@
 #ifndef LIB_CONSUMER_H
 #define LIB_CONSUMER_H
 
+#include "consumer-type.hpp"
+
 #include <common/buffer-view.hpp>
+#include <common/consumer/consumer-channel.hpp>
 #include <common/credentials.hpp>
 #include <common/dynamic-array.hpp>
+#include <common/exception.hpp>
 #include <common/hashtable/hashtable.hpp>
 #include <common/index/ctf-index.hpp>
 #include <common/pipe.hpp>
+#include <common/scheduler.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
+#include <common/task-executor.hpp>
 #include <common/trace-chunk-registry.hpp>
 #include <common/uuid.hpp>
 #include <common/waiter.hpp>
 
 #include <lttng/lttng.h>
+#include <lttng/session-descriptor.h>
 
+#include <vendor/optional.hpp>
+
+#include <chrono>
+#include <cstdint>
 #include <limits.h>
-#include <limits>
 #include <mutex>
 #include <poll.h>
+#include <set>
 #include <stdint.h>
 #include <unistd.h>
 #include <urcu/list.h>
+#include <vector>
 
 struct lttng_consumer_local_data;
 
-/* Commands for consumer */
-enum lttng_consumer_command {
-	LTTNG_CONSUMER_ADD_CHANNEL,
-	LTTNG_CONSUMER_ADD_STREAM,
-	/* pause, delete, active depending on fd state */
-	LTTNG_CONSUMER_UPDATE_STREAM,
-	/* inform the consumer to quit when all fd has hang up */
-	LTTNG_CONSUMER_STOP, /* deprecated */
-	LTTNG_CONSUMER_ADD_RELAYD_SOCKET,
-	/* Inform the consumer to kill a specific relayd connection */
-	LTTNG_CONSUMER_DESTROY_RELAYD,
-	/* Return to the sessiond if there is data pending for a session */
-	LTTNG_CONSUMER_DATA_PENDING,
-	/* Consumer creates a channel and returns it to sessiond. */
-	LTTNG_CONSUMER_ASK_CHANNEL_CREATION,
-	LTTNG_CONSUMER_GET_CHANNEL,
-	LTTNG_CONSUMER_DESTROY_CHANNEL,
-	LTTNG_CONSUMER_PUSH_METADATA,
-	LTTNG_CONSUMER_CLOSE_METADATA,
-	LTTNG_CONSUMER_SETUP_METADATA,
-	LTTNG_CONSUMER_FLUSH_CHANNEL,
-	LTTNG_CONSUMER_SNAPSHOT_CHANNEL,
-	LTTNG_CONSUMER_SNAPSHOT_METADATA,
-	LTTNG_CONSUMER_STREAMS_SENT,
-	LTTNG_CONSUMER_DISCARDED_EVENTS,
-	LTTNG_CONSUMER_LOST_PACKETS,
-	LTTNG_CONSUMER_CLEAR_QUIESCENT_CHANNEL,
-	LTTNG_CONSUMER_SET_CHANNEL_MONITOR_PIPE,
-	LTTNG_CONSUMER_ROTATE_CHANNEL,
-	LTTNG_CONSUMER_INIT,
-	LTTNG_CONSUMER_CREATE_TRACE_CHUNK,
-	LTTNG_CONSUMER_CLOSE_TRACE_CHUNK,
-	LTTNG_CONSUMER_TRACE_CHUNK_EXISTS,
-	LTTNG_CONSUMER_CLEAR_CHANNEL,
-	LTTNG_CONSUMER_OPEN_CHANNEL_PACKETS,
-};
-
-enum lttng_consumer_type {
-	LTTNG_CONSUMER_UNKNOWN = 0,
-	LTTNG_CONSUMER_KERNEL,
-	LTTNG_CONSUMER64_UST,
-	LTTNG_CONSUMER32_UST,
-};
-
-/*
- * Due to a bug in g++ < 7.1, this specialization must be enclosed in the fmt namespace,
- * see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=56480.
- */
-namespace fmt {
-template <>
-struct formatter<lttng_consumer_type> : formatter<std::string> {
-	template <typename FormatContextType>
-	typename FormatContextType::iterator format(lttng_consumer_type consumer_type,
-						    FormatContextType& ctx) const
-	{
-		const char *name;
-
-		switch (consumer_type) {
-		case LTTNG_CONSUMER_KERNEL:
-			name = "kernel consumer";
-			break;
-		case LTTNG_CONSUMER64_UST:
-			name = "64-bit user space consumer";
-			break;
-		case LTTNG_CONSUMER32_UST:
-			name = "32-bit user space consumer";
-			break;
-		case LTTNG_CONSUMER_UNKNOWN:
-			name = "unknown consumer";
-			break;
-		default:
-			std::abort();
-		}
-
-		return format_to(ctx.out(), name);
-	}
-};
-} /* namespace fmt */
-
 enum consumer_endpoint_status {
 	CONSUMER_ENDPOINT_ACTIVE,
 	CONSUMER_ENDPOINT_INACTIVE,
@@ -123,10 +56,9 @@ enum consumer_channel_output {
 	CONSUMER_CHANNEL_SPLICE = 1,
 };
 
-enum consumer_channel_type {
-	CONSUMER_CHANNEL_TYPE_METADATA = 0,
-	CONSUMER_CHANNEL_TYPE_DATA_PER_CPU = 1,
-	CONSUMER_CHANNEL_TYPE_DATA_PER_CHANNEL = 2,
+enum consumer_channel_event_loss_mode {
+	CONSUMER_CHANNEL_EVENT_LOSS_MODE_DISCARD_EVENTS,
+	CONSUMER_CHANNEL_EVENT_LOSS_MODE_OVERWRITE_OLDEST_PACKET,
 };
 
 enum sync_metadata_status {
@@ -145,7 +77,32 @@ struct stream_list {
 /* Stub. */
 struct consumer_metadata_cache;
 
+/*
+ * The owner_id with the value `LTTNG_UST_ABI_OWNER_ID_UNSET` (0), which is
+ * never used to compare with the last seen state. Thus, it is fine to use it as
+ * the first seen value.
+ */
+struct stream_subbuffer_transaction_state {
+	uint32_t owner_id = 0; /*
+				* FIXME:ust-abi-internal This should be
+				* LTTNG_UST_ABI_OWNER_ID_UNSET
+				*/
+	unsigned long hot_commit_count = 0;
+	unsigned long cold_commit_count = 0;
+};
+
 struct lttng_consumer_channel {
+	/*
+	 * Iterate over the streams of this channel. Note that this utility provides the channel's
+	 * streams from the "streams" list  and from the global stream hash table, providing streams
+	 * that are either published or not.
+	 *
+	 * The consumer_data and channel locks must be acquired before calling this method.
+	 */
+	lttng::consumer::stream_set
+	get_streams(const nonstd::optional<lttng::consumer::stream_set::filter>& filter =
+			    nonstd::nullopt);
+
 	/* Is the channel published in the channel hash tables? */
 	bool is_published = false;
 	/*
@@ -166,6 +123,14 @@ struct lttng_consumer_channel {
 	uint64_t session_id = 0;
 	/* Current trace chunk of the session in which this channel exists. */
 	struct lttng_trace_chunk *trace_chunk = nullptr;
+	/*
+	 * Trace format of the session.
+	 *
+	 * Currently only used to determine whether to prepend the CTF 1.x
+	 * metadata announcement comment for kernel metadata streams (the kernel
+	 * tracer doesn't write it, so the consumer does).
+	 */
+	enum lttng_trace_format trace_format = LTTNG_TRACE_FORMAT_CTF_1_8;
 	/*
 	 * Session id when requesting metadata to the session daemon for
 	 * a session with per-PID buffers.
@@ -197,6 +162,9 @@ struct lttng_consumer_channel {
 	/* Channel type for stream */
 	enum consumer_channel_type type = CONSUMER_CHANNEL_TYPE_METADATA;
 
+	consumer_channel_event_loss_mode event_loss_mode =
+		CONSUMER_CHANNEL_EVENT_LOSS_MODE_DISCARD_EVENTS;
+
 	/* For UST */
 	uid_t ust_app_uid = 65534; /* Application UID. */
 	struct lttng_ust_ctl_consumer_channel *uchan = nullptr;
@@ -205,6 +173,10 @@ struct lttng_consumer_channel {
 	 * Temporary stream list used to store the streams once created and waiting
 	 * to be sent to the session daemon by receiving the
 	 * LTTNG_CONSUMER_GET_CHANNEL.
+	 *
+	 * Note that in the case of a channel that has no active output (i.e.
+	 * a "snapshot" channel), this list is used to reference the streams as they
+	 * never become globally visible (monitored).
 	 */
 	struct stream_list streams = {};
 
@@ -229,20 +201,38 @@ struct lttng_consumer_channel {
 	lttng::synchro::wait_queue metadata_consumed_wait_queue;
 
 	/* For UST metadata periodical flush */
-	int switch_timer_enabled = 0;
-	timer_t switch_timer = {};
+	lttng::scheduling::periodic_task::sptr metadata_switch_timer_task;
 	int switch_timer_error = 0;
 
 	/* For the live mode */
-	int live_timer_enabled = 0;
-	timer_t live_timer = {};
+	lttng::scheduling::periodic_task::sptr live_timer_task;
+
 	int live_timer_error = 0;
 	/* Channel is part of a live session ? */
 	bool is_live = false;
 
+	/*
+	 * Continuously reclaimed channel.
+	 *
+	 * If true, the channel's memory is continuously reclaimed as its
+	 * contents are consumed.
+	 */
+	bool continuously_reclaimed = false;
+
+	/* Maximum age of a subbuffer before it is reclaimed. */
+	nonstd::optional<std::chrono::microseconds> automatic_memory_reclamation_max_age;
+
 	/* For channel monitoring timer. */
-	int monitor_timer_enabled = 0;
-	timer_t monitor_timer = {};
+	lttng::scheduling::periodic_task::sptr monitor_timer_task;
+
+	/*
+	 * For channel buffer-stall monitoring timer (only used by user space
+	 * channels) when `subbuffer_count` is not zero.
+	 */
+	lttng::scheduling::periodic_task::sptr stall_watchdog_timer_task;
+
+	/* For periodic channel memory reclamation. */
+	lttng::scheduling::periodic_task::sptr memory_reclaim_timer_task;
 
 	/* On-disk circular buffer */
 	uint64_t tracefile_size = 0;
@@ -297,6 +287,28 @@ struct lttng_consumer_channel {
 
 	bool streams_sent_to_relayd = false;
 	uint64_t consumed_size_as_of_last_sample_sent = 0;
+
+	nonstd::optional<uint64_t> subbuffer_count;
+
+	/*
+	 * Reclaim owners set lock.
+	 *
+	 * This lock protects against concurrent read/update of owners_pending_reclamation.
+	 *
+	 * This lock is necessary instead of using the channel lock because the
+	 * channel lock is used while tearing down the channel.
+	 *
+	 * However, the reclamation sets need to be drained on tear-down. Since
+	 * the channel lock is not recursive, we need another lock to protect
+	 * the set.
+	 */
+	std::mutex owners_pending_reclamation_lock;
+
+	/*
+	 * std::set instead of std::unordered_set because std::set_* algorithms
+	 * require sorted ranges.
+	 */
+	std::set<uint32_t> owners_pending_reclamation;
 };
 
 struct stream_subbuffer {
@@ -354,7 +366,7 @@ enum get_next_subbuffer_status {
  * Perform any operation required to acknowledge
  * the wake-up of a consumer stream (e.g. consume a byte on a wake-up pipe).
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using on_wake_up_cb = int (*)(struct lttng_consumer_stream *);
 
@@ -362,14 +374,14 @@ using on_wake_up_cb = int (*)(struct lttng_consumer_stream *);
  * Perform any operation required before a consumer stream is put
  * to sleep before awaiting a data availability notification.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using on_sleep_cb = int (*)(struct lttng_consumer_stream *, struct lttng_consumer_local_data *);
 
 /*
  * Acquire the subbuffer at the current 'consumed' position.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using get_next_subbuffer_cb = enum get_next_subbuffer_status (*)(struct lttng_consumer_stream *,
 								 struct stream_subbuffer *);
@@ -378,7 +390,7 @@ using get_next_subbuffer_cb = enum get_next_subbuffer_status (*)(struct lttng_co
  * Populate the stream_subbuffer's info member. The info to populate
  * depends on the type (metadata/data) of the stream.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using extract_subbuffer_info_cb = int (*)(struct lttng_consumer_stream *,
 					  struct stream_subbuffer *);
@@ -386,14 +398,14 @@ using extract_subbuffer_info_cb = int (*)(struct lttng_consumer_stream *,
 /*
  * Invoked after a subbuffer's info has been filled.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using pre_consume_subbuffer_cb = int (*)(struct lttng_consumer_stream *, struct stream_subbuffer *);
 
 /*
  * Consume subbuffer contents.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using consume_subbuffer_cb = ssize_t (*)(struct lttng_consumer_local_data *,
 					 struct lttng_consumer_stream *,
@@ -403,25 +415,25 @@ using consume_subbuffer_cb = ssize_t (*)(struct lttng_consumer_local_data *,
  * Release the current subbuffer and advance the 'consumed' position by
  * one subbuffer.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using put_next_subbuffer_cb = int (*)(struct lttng_consumer_stream *, struct stream_subbuffer *);
 
 /*
  * Invoked after consuming a subbuffer.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
-using post_consume_cb = int (*)(struct lttng_consumer_stream *,
+using post_consume_cb = int (*)(lttng_consumer_stream&,
 				const struct stream_subbuffer *,
 				struct lttng_consumer_local_data *);
 
 /*
  * Send a live beacon if no data is available.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
-using send_live_beacon_cb = int (*)(struct lttng_consumer_stream *);
+using send_live_beacon_cb = int (*)(lttng_consumer_stream&);
 
 /*
  * Lock the stream and channel locks and any other stream-type specific
@@ -434,7 +446,7 @@ using lock_cb = void (*)(struct lttng_consumer_stream *);
  * Unlock the stream and channel locks and any other stream-type specific
  * lock before sleeping until the next availability notification.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using unlock_cb = void (*)(struct lttng_consumer_stream *);
 
@@ -449,10 +461,17 @@ using assert_locked_cb = void (*)(struct lttng_consumer_stream *);
  * Invoked when a subbuffer's metadata version does not match the last
  * known metadata version.
  *
- * Stream and channel locks are acquired during this call.
+ * Stream and channel locks are held during this call.
  */
 using reset_metadata_cb = void (*)(struct lttng_consumer_stream *);
 
+struct consumer_stream_pending_reclamation {
+	unsigned long subbuffer_count;
+	nonstd::optional<std::chrono::microseconds> max_age;
+	/* Token for async completion tracking. */
+	nonstd::optional<std::uint64_t> memory_reclaim_request_token;
+};
+
 /*
  * Internal representation of the streams, sessiond_key is used to identify
  * uniquely a stream.
@@ -499,7 +518,7 @@ struct lttng_consumer_stream {
 	off_t out_fd_offset;
 	/* Amount of bytes written to the output */
 	uint64_t output_written;
-	int shm_fd_is_copy;
+
 	/*
 	 * When a stream's pipe is hung up, a final flush is performed (see hangup_flush_done). This
 	 * indicates whether or not the data resulting from this flush is still to be consumed.
@@ -731,6 +750,10 @@ struct lttng_consumer_stream {
 		assert_locked_cb assert_locked;
 	} read_subbuffer_ops;
 	struct metadata_bucket *metadata_bucket;
+
+	std::vector<struct stream_subbuffer_transaction_state> subbuffer_transaction_states = {};
+
+	nonstd::optional<consumer_stream_pending_reclamation> pending_memory_reclamation;
 };
 
 /*
@@ -777,6 +800,11 @@ struct consumer_relayd_sock_pair {
 	struct lttng_consumer_local_data *ctx;
 };
 
+struct protected_socket {
+	pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+	int fd = -1;
+};
+
 /*
  * UST consumer local data to the program. One or more instance per
  * process.
@@ -786,9 +814,9 @@ struct lttng_consumer_local_data {
 	 * Function to call when data is available on a buffer.
 	 * Returns the number of bytes read, or negative error value.
 	 */
-	ssize_t (*on_buffer_ready)(struct lttng_consumer_stream *stream,
-				   struct lttng_consumer_local_data *ctx,
-				   bool locked_by_caller);
+	ssize_t (*on_buffer_ready)(lttng_consumer_stream *stream,
+				   lttng_consumer_local_data *ctx,
+				   bool locked_by_caller) = nullptr;
 	/*
 	 * function to call when we receive a new channel, it receives a
 	 * newly allocated channel, depending on the return code of this
@@ -800,7 +828,7 @@ struct lttng_consumer_local_data {
 	 *   == 0 (success, FD is left to library)
 	 *    < 0 (error)
 	 */
-	int (*on_recv_channel)(struct lttng_consumer_channel *channel);
+	int (*on_recv_channel)(lttng_consumer_channel *channel) = nullptr;
 	/*
 	 * function to call when we receive a new stream, it receives a
 	 * newly allocated stream, depending on the return code of this
@@ -812,7 +840,7 @@ struct lttng_consumer_local_data {
 	 *   == 0 (success, FD is left to library)
 	 *    < 0 (error)
 	 */
-	int (*on_recv_stream)(struct lttng_consumer_stream *stream);
+	int (*on_recv_stream)(lttng_consumer_stream *stream) = nullptr;
 	/*
 	 * function to call when a stream is getting updated by the session
 	 * daemon, this function receives the sessiond key and the new
@@ -825,24 +853,24 @@ struct lttng_consumer_local_data {
 	 *   == 0 (success, FD is left to library)
 	 *    < 0 (error)
 	 */
-	int (*on_update_stream)(uint64_t sessiond_key, uint32_t state);
-	enum lttng_consumer_type type;
+	int (*on_update_stream)(uint64_t sessiond_key, uint32_t state) = nullptr;
+	lttng_consumer_type type = LTTNG_CONSUMER_UNKNOWN;
 	/* socket to communicate errors with sessiond */
-	int consumer_error_socket;
-	/* socket to ask metadata to sessiond. */
-	int consumer_metadata_socket;
+	protected_socket consumer_error_socket;
+
 	/*
-	 * Protect consumer_metadata_socket.
+	 * Socket to ask metadata to the sessiond.
 	 *
-	 * This is nested OUTSIDE the metadata cache lock.
+	 * The lock is nested OUTSIDE the metadata cache lock.
 	 */
-	pthread_mutex_t metadata_socket_lock;
+	protected_socket metadata_socket;
+
 	/* socket to exchange commands with sessiond */
-	char *consumer_command_sock_path;
+	char *consumer_command_sock_path = nullptr;
 	/* communication with splice */
-	int consumer_channel_pipe[2];
+	int consumer_channel_pipe[2] = { -1, -1 };
 	/* Data stream poll thread pipe. To transfer data stream to the thread */
-	struct lttng_pipe *consumer_data_pipe;
+	lttng_pipe *consumer_data_pipe = nullptr;
 
 	/*
 	 * Data thread use that pipe to catch wakeup from read subbuffer that
@@ -852,20 +880,23 @@ struct lttng_consumer_local_data {
 	 *
 	 * Both pipes (read/write) are owned and used inside the data thread.
 	 */
-	struct lttng_pipe *consumer_wakeup_pipe;
+	lttng_pipe *consumer_wakeup_pipe = nullptr;
 	/* Indicate if the wakeup thread has been notified. */
-	unsigned int has_wakeup:1;
+	bool has_wakeup = false;
 
 	/* to let the signal handler wake up the fd receiver thread */
-	int consumer_should_quit[2];
+	int consumer_should_quit[2] = { -1, -1 };
 	/* Metadata poll thread pipe. Transfer metadata stream to it */
-	struct lttng_pipe *consumer_metadata_pipe;
+	lttng_pipe *consumer_metadata_pipe = nullptr;
 	/*
 	 * Pipe used by the channel monitoring timers to provide state samples
 	 * to the session daemon (write-only).
 	 */
-	int channel_monitor_pipe;
-	LTTNG_OPTIONAL(lttng_uuid) sessiond_uuid;
+	int channel_monitor_pipe = -1;
+	nonstd::optional<lttng_uuid> sessiond_uuid;
+
+	lttng::scheduling::scheduler timer_task_scheduler;
+	lttng::scheduling::task_executor timer_task_executor{ timer_task_scheduler };
 };
 
 /*
@@ -925,6 +956,32 @@ struct lttng_consumer_global_data {
 	struct lttng_trace_chunk_registry *chunk_registry = nullptr;
 };
 
+#define LTTNG_THROW_CHANNEL_NOT_FOUND_BY_KEY_ERROR(channel_key)                  \
+	throw lttng::consumerd::exceptions::channel_not_found_error(channel_key, \
+								    LTTNG_SOURCE_LOCATION())
+
+namespace lttng {
+namespace consumerd {
+namespace exceptions {
+/*
+ * @class channel_not_found_error
+ * @brief Represents a channel-not-found error and provides the key of the channel looked-up
+ * for use by error-reporting code.
+ */
+class channel_not_found_error : public lttng::runtime_error {
+public:
+	explicit channel_not_found_error(std::uint64_t channel_key,
+					 const lttng::source_location& source_location);
+
+	std::uint64_t channel_key;
+};
+} /* namespace exceptions */
+
+} /* namespace consumerd */
+} /* namespace lttng */
+
+#include <common/consumer/pending-memory-reclamation-tracker.hpp>
+
 /*
  * Set to nonzero when the consumer is exiting. Updated by signal
  * handler and thread exit, read by threads.
@@ -982,7 +1039,7 @@ void lttng_consumer_set_command_sock_path(struct lttng_consumer_local_data *ctx,
  * Returns the return code of sendmsg : the number of bytes transmitted or -1
  * on error.
  */
-int lttng_consumer_send_error(struct lttng_consumer_local_data *ctx,
+int lttng_consumer_send_error(protected_socket& consumer_error_socket,
 			      enum lttcomm_return_code error_code);
 
 /*
@@ -1019,21 +1076,26 @@ struct lttng_consumer_stream *consumer_allocate_stream(struct lttng_consumer_cha
 						       int *alloc_ret,
 						       enum consumer_channel_type type,
 						       unsigned int monitor);
-struct lttng_consumer_channel *consumer_allocate_channel(uint64_t key,
-							 uint64_t session_id,
-							 const uint64_t *chunk_id,
-							 const char *pathname,
-							 const char *name,
-							 uint64_t relayd_id,
-							 enum lttng_event_output output,
-							 uint64_t tracefile_size,
-							 uint64_t tracefile_count,
-							 uint64_t session_id_per_pid,
-							 unsigned int monitor,
-							 unsigned int live_timer_interval,
-							 bool is_in_live_session,
-							 const char *root_shm_path,
-							 const char *shm_path);
+struct lttng_consumer_channel *consumer_allocate_channel(
+	uint64_t key,
+	uint64_t session_id,
+	const uint64_t *chunk_id,
+	const char *pathname,
+	const char *name,
+	uint64_t relayd_id,
+	enum lttng_event_output output,
+	uint64_t tracefile_size,
+	uint64_t tracefile_count,
+	uint64_t subbuffer_count,
+	uint64_t session_id_per_pid,
+	unsigned int monitor,
+	unsigned int live_timer_interval,
+	bool is_in_live_session,
+	bool continuously_reclaimed,
+	nonstd::optional<std::chrono::microseconds> automatic_memory_reclamation_max_age,
+	const char *root_shm_path,
+	const char *shm_path,
+	enum lttng_trace_format trace_format);
 void consumer_del_stream(struct lttng_consumer_stream *stream, struct lttng_ht *ht);
 void consumer_del_metadata_stream(struct lttng_consumer_stream *stream, struct lttng_ht *ht);
 int consumer_add_channel(struct lttng_consumer_channel *channel,
@@ -1142,8 +1204,26 @@ enum lttcomm_return_code lttng_consumer_init_command(struct lttng_consumer_local
 int lttng_consumer_clear_channel(struct lttng_consumer_channel *channel);
 enum lttcomm_return_code
 lttng_consumer_open_channel_packets(struct lttng_consumer_channel *channel);
+
+namespace lttng {
+namespace consumer {
+struct stream_memory_usage {
+	struct {
+		std::uint64_t logical;
+		std::uint64_t physical;
+	} size_bytes;
+};
+
+struct channel_memory_usage {
+	std::vector<stream_memory_usage> streams_memory_usage;
+};
+
+enum lttcomm_return_code get_channels_memory_usage(const std::vector<std::uint64_t>& channel_keys,
+						   std::vector<channel_memory_usage>& usage_stats);
+} /* namespace consumer */
+} /* namespace lttng */
+
 int consumer_metadata_wakeup_pipe(const struct lttng_consumer_channel *channel);
 void lttng_consumer_sigbus_handle(void *addr);
-void sample_and_send_channel_buffer_stats(struct lttng_consumer_channel *channel);
 
 #endif /* LIB_CONSUMER_H */
