@@ -205,7 +205,8 @@
 #define DEFAULT_PYTHON_EVENT_COMPONENT "lttng_python"
 #define DEFAULT_PYTHON_EVENT_NAME      DEFAULT_PYTHON_EVENT_COMPONENT ":*"
 
-#define DEFAULT_CHANNEL_ALLOCATION_POLICY LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU
+#define DEFAULT_CHANNEL_ALLOCATION_POLICY    LTTNG_CHANNEL_ALLOCATION_POLICY_PER_CPU
+#define DEFAULT_CHANNEL_PREALLOCATION_POLICY LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE
 
 #define DEFAULT_CHANNEL_OVERWRITE	(-1)
 #define DEFAULT_CHANNEL_TRACEFILE_SIZE	CONFIG_DEFAULT_CHANNEL_TRACEFILE_SIZE
@@ -252,12 +253,13 @@
 #define DEFAULT_UST_PID_CHANNEL_OUTPUT _DEFAULT_CHANNEL_OUTPUT
 #define DEFAULT_UST_UID_CHANNEL_OUTPUT _DEFAULT_CHANNEL_OUTPUT
 /* Timers in usec. */
-#define DEFAULT_UST_PID_CHANNEL_SWITCH_TIMER  CONFIG_DEFAULT_UST_PID_CHANNEL_SWITCH_TIMER
-#define DEFAULT_UST_UID_CHANNEL_SWITCH_TIMER  CONFIG_DEFAULT_UST_UID_CHANNEL_SWITCH_TIMER
-#define DEFAULT_UST_PID_CHANNEL_LIVE_TIMER    CONFIG_DEFAULT_UST_PID_CHANNEL_LIVE_TIMER
-#define DEFAULT_UST_UID_CHANNEL_LIVE_TIMER    CONFIG_DEFAULT_UST_UID_CHANNEL_LIVE_TIMER
-#define DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER CONFIG_DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER
-#define DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER CONFIG_DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER
+#define DEFAULT_UST_PID_CHANNEL_SWITCH_TIMER   CONFIG_DEFAULT_UST_PID_CHANNEL_SWITCH_TIMER
+#define DEFAULT_UST_UID_CHANNEL_SWITCH_TIMER   CONFIG_DEFAULT_UST_UID_CHANNEL_SWITCH_TIMER
+#define DEFAULT_UST_PID_CHANNEL_LIVE_TIMER     CONFIG_DEFAULT_UST_PID_CHANNEL_LIVE_TIMER
+#define DEFAULT_UST_UID_CHANNEL_LIVE_TIMER     CONFIG_DEFAULT_UST_UID_CHANNEL_LIVE_TIMER
+#define DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER  CONFIG_DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER
+#define DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER  CONFIG_DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER
+#define DEFAULT_UST_UID_CHANNEL_WATCHDOG_TIMER CONFIG_DEFAULT_UST_UID_CHANNEL_WATCHDOG_TIMER
 
 #define DEFAULT_UST_PID_CHANNEL_READ_TIMER CONFIG_DEFAULT_UST_PID_CHANNEL_READ_TIMER
 #define DEFAULT_UST_UID_CHANNEL_READ_TIMER CONFIG_DEFAULT_UST_UID_CHANNEL_READ_TIMER
@@ -388,6 +390,22 @@
 #define DEFAULT_LINUX_POSSIBLE_CPU_PATH	       "/sys/devices/system/cpu/possible"
 #define DEFAULT_LINUX_POSSIBLE_CPU_MASK_LENGTH 4096
 
+#define DEFAULT_MINIMAL_MEMORY_RECLAIM_TIMER_PERIOD_MS 250
+
+/*
+ * Filename used to test the support for `MADV_REMOVE` using `madvise(2)`.
+ *
+ * The value is used with either `openat(2)` -- when testing a shared-memory
+ * mounted file-system -- or `shm_open(3)`.
+ *
+ * According to `shm_open(3)`:
+ *   For portable use, a shared memory object should be identified by a name of
+ *   the form /somename; that is, a null-terminated string of up to NAME_MAX
+ *   (i.e., 255) characters consisting of an initial slash, followed by one or
+ *   more characters, none of which are slashes.
+ */
+#define DEFAULT_MADV_REMOVE_TEST_FILENAME "/lttng-test-fs-supports-madv-remove"
+
 /*
  * Returns the default subbuf size.
  *
