@@ -74,7 +74,7 @@ int lttng_kconsumer_take_snapshot(struct lttng_consumer_stream *stream)
  *
  * Returns 0 on success, < 0 on error.
  */
-int lttng_kconsumer_sample_snapshot_positions(struct lttng_consumer_stream *stream)
+int lttng_kconsumer_sample_snapshot_positions(struct lttng_consumer_stream *stream) noexcept
 {
 	LTTNG_ASSERT(stream);
 
@@ -86,7 +86,8 @@ int lttng_kconsumer_sample_snapshot_positions(struct lttng_consumer_stream *stre
  *
  * Returns 0 on success, < 0 on error
  */
-int lttng_kconsumer_get_produced_snapshot(struct lttng_consumer_stream *stream, unsigned long *pos)
+int lttng_kconsumer_get_produced_snapshot(struct lttng_consumer_stream *stream,
+					  unsigned long *pos) noexcept
 {
 	int ret;
 	const int infd = stream->wait_fd;
@@ -104,7 +105,8 @@ int lttng_kconsumer_get_produced_snapshot(struct lttng_consumer_stream *stream,
  *
  * Returns 0 on success, < 0 on error
  */
-int lttng_kconsumer_get_consumed_snapshot(struct lttng_consumer_stream *stream, unsigned long *pos)
+int lttng_kconsumer_get_consumed_snapshot(struct lttng_consumer_stream *stream,
+					  unsigned long *pos) noexcept
 {
 	int ret;
 	const int infd = stream->wait_fd;
@@ -165,9 +167,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 		return -1;
 	}
 
-	for (auto stream : lttng::urcu::list_iteration_adapter<lttng_consumer_stream,
-							       &lttng_consumer_stream::send_node>(
-		     channel->streams.head)) {
+	for (auto& stream : channel->get_streams()) {
 		unsigned long consumed_pos, produced_pos, max_subbuf_size;
 		lttng_kernel_abi_ring_buffer_packet_flush_or_populate_packet_args packet_args = {};
 
@@ -176,7 +176,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 		/*
 		 * Lock stream because we are about to change its state.
 		 */
-		const lttng::pthread::lock_guard stream_lock(stream->lock);
+		const lttng::pthread::lock_guard stream_lock(stream.lock);
 
 		LTTNG_ASSERT(channel->trace_chunk);
 		if (!lttng_trace_chunk_get(channel->trace_chunk)) {
@@ -188,36 +188,36 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 			return -1;
 		}
 
-		LTTNG_ASSERT(!stream->trace_chunk);
-		stream->trace_chunk = channel->trace_chunk;
+		LTTNG_ASSERT(!stream.trace_chunk);
+		stream.trace_chunk = channel->trace_chunk;
 
 		/*
 		 * Assign the received relayd ID so we can use it for streaming. The streams
 		 * are not visible to anyone so this is OK to change it.
 		 */
-		stream->net_seq_idx = relayd_id;
+		stream.net_seq_idx = relayd_id;
 		channel->relayd_id = relayd_id;
 
 		/* Close stream output when were are done. */
 		const auto close_stream_output = lttng::make_scope_exit(
-			[stream]() noexcept { consumer_stream_close_output(stream); });
+			[&stream]() noexcept { consumer_stream_close_output(&stream); });
 
 		if (relayd_id != (uint64_t) -1ULL) {
-			ret = consumer_send_relayd_stream(stream, path);
+			ret = consumer_send_relayd_stream(&stream, path);
 			if (ret < 0) {
 				ERR("sending stream to relayd");
 				return ret;
 			}
 		} else {
-			ret = consumer_stream_create_output_files(stream, false);
+			ret = consumer_stream_create_output_files(&stream, false);
 			if (ret < 0) {
 				return ret;
 			}
 
-			DBG("Kernel consumer snapshot stream (%" PRIu64 ")", stream->key);
+			DBG("Kernel consumer snapshot stream (%" PRIu64 ")", stream.key);
 		}
 
-		ret = kernctl_get_max_subbuf_size(stream->wait_fd, &max_subbuf_size);
+		ret = kernctl_get_max_subbuf_size(stream.wait_fd, &max_subbuf_size);
 		if (ret < 0) {
 			ERR("Failed to get max subbuf_size: %d", ret);
 			return ret;
@@ -233,7 +233,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 		packet_args.packet =
 			static_cast<uint64_t>(reinterpret_cast<uintptr_t>(packet_buffer.data()));
 
-		ret = kernctl_buffer_flush_or_populate_packet(stream->wait_fd, &packet_args);
+		ret = kernctl_buffer_flush_or_populate_packet(stream.wait_fd, &packet_args);
 		if (ret < 0) {
 			if (ret != -ENOTTY) {
 				/* kernctl_buffer_flush_or_poopulate_packet is supported, but failed
@@ -248,7 +248,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 				warn_flush_or_populate_packet = true;
 			}
 
-			ret = kernctl_buffer_flush_empty(stream->wait_fd);
+			ret = kernctl_buffer_flush_empty(stream.wait_fd);
 			if (ret < 0) {
 				if (!warn_flush) {
 					DBG("Failed to perform kernctl_buffer_flush_empty: %d",
@@ -263,7 +263,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 				 * fall-back when "flush_empty" is not
 				 * implemented by lttng-modules.
 				 */
-				ret = kernctl_buffer_flush(stream->wait_fd);
+				ret = kernctl_buffer_flush(stream.wait_fd);
 				if (ret < 0) {
 					ERR("Failed to flush kernel stream");
 					return ret;
@@ -271,26 +271,26 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 			}
 		}
 
-		ret = lttng_kconsumer_take_snapshot(stream);
+		ret = lttng_kconsumer_take_snapshot(&stream);
 		if (ret < 0) {
 			ERR("Taking kernel snapshot");
 			return ret;
 		}
 
-		ret = lttng_kconsumer_get_produced_snapshot(stream, &produced_pos);
+		ret = lttng_kconsumer_get_produced_snapshot(&stream, &produced_pos);
 		if (ret < 0) {
 			ERR("Produced kernel snapshot position");
 			return ret;
 		}
 
-		ret = lttng_kconsumer_get_consumed_snapshot(stream, &consumed_pos);
+		ret = lttng_kconsumer_get_consumed_snapshot(&stream, &consumed_pos);
 		if (ret < 0) {
 			ERR("Consumerd kernel snapshot position");
 			return ret;
 		}
 
 		consumed_pos = consumer_get_consume_start_pos(
-			consumed_pos, produced_pos, nb_packets_per_stream, stream->max_sb_size);
+			consumed_pos, produced_pos, nb_packets_per_stream, stream.max_sb_size);
 
 		while ((long) (consumed_pos - produced_pos) < 0) {
 			ssize_t read_len;
@@ -301,7 +301,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 			health_code_update();
 			DBG("Kernel consumer taking snapshot at pos %lu", consumed_pos);
 
-			ret = kernctl_get_subbuf(stream->wait_fd, &consumed_pos);
+			ret = kernctl_get_subbuf(stream.wait_fd, &consumed_pos);
 			if (ret < 0) {
 				if (ret != -EAGAIN) {
 					PERROR("kernctl_get_subbuf snapshot");
@@ -309,39 +309,39 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 				}
 
 				DBG("Kernel consumer get subbuf failed. Skipping it.");
-				consumed_pos += stream->max_sb_size;
-				stream->chan->lost_packets++;
+				consumed_pos += stream.max_sb_size;
+				stream.chan->lost_packets++;
 				continue;
 			}
 
 			/* Put the subbuffer once we are done. */
-			const auto put_subbuf = lttng::make_scope_exit([stream]() noexcept {
-				const auto put_ret = kernctl_put_subbuf(stream->wait_fd);
+			const auto put_subbuf = lttng::make_scope_exit([&stream]() noexcept {
+				const auto put_ret = kernctl_put_subbuf(stream.wait_fd);
 				if (put_ret < 0) {
 					ERR("Snapshot kernctl_put_subbuf");
 				}
 			});
 
-			ret = kernctl_get_subbuf_size(stream->wait_fd, &len);
+			ret = kernctl_get_subbuf_size(stream.wait_fd, &len);
 			if (ret < 0) {
 				ERR("Snapshot kernctl_get_subbuf_size");
 				return ret;
 			}
 
-			ret = kernctl_get_padded_subbuf_size(stream->wait_fd, &padded_len);
+			ret = kernctl_get_padded_subbuf_size(stream.wait_fd, &padded_len);
 			if (ret < 0) {
 				ERR("Snapshot kernctl_get_padded_subbuf_size");
 				return ret;
 			}
 
-			ret = get_current_subbuf_addr(stream, &subbuf_addr);
+			ret = get_current_subbuf_addr(&stream, &subbuf_addr);
 			if (ret) {
 				return ret;
 			}
 
 			subbuf_view = lttng_buffer_view_init(subbuf_addr, 0, padded_len);
 			read_len = lttng_consumer_on_read_subbuffer_mmap(
-				stream, &subbuf_view, padded_len - len);
+				&stream, &subbuf_view, padded_len - len);
 			/*
 			 * We write the padded len in local tracefiles but the data len
 			 * when using a relay. Display the error but continue processing
@@ -361,7 +361,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 				}
 			}
 
-			consumed_pos += stream->max_sb_size;
+			consumed_pos += stream.max_sb_size;
 		}
 
 		if (packet_args.packet_populated) {
@@ -370,7 +370,7 @@ static int lttng_kconsumer_snapshot_channel(struct lttng_consumer_channel *chann
 			const auto subbuf_view = lttng_buffer_view_init(
 				(char *) packet_buffer.data(), 0, packet_args.packet_length_padded);
 			const auto read_len = lttng_consumer_on_read_subbuffer_mmap(
-				stream,
+				&stream,
 				&subbuf_view,
 				packet_args.packet_length_padded - packet_args.packet_length);
 
@@ -506,7 +506,8 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		ret_recv = lttcomm_recv_unix_sock(sock, &msg, sizeof(msg));
 		if (ret_recv != sizeof(msg)) {
 			if (ret_recv > 0) {
-				lttng_consumer_send_error(ctx, LTTCOMM_CONSUMERD_ERROR_RECV_CMD);
+				lttng_consumer_send_error(ctx->consumer_error_socket,
+							  LTTCOMM_CONSUMERD_ERROR_RECV_CMD);
 				ret_recv = -1;
 			}
 			return ret_recv;
@@ -562,24 +563,29 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		health_code_update();
 
 		DBG("consumer_add_channel %" PRIu64, msg.u.channel.channel_key);
-		new_channel = consumer_allocate_channel(msg.u.channel.channel_key,
-							msg.u.channel.session_id,
-							msg.u.channel.chunk_id.is_set ? &chunk_id :
-											nullptr,
-							msg.u.channel.pathname,
-							msg.u.channel.name,
-							msg.u.channel.relayd_id,
-							msg.u.channel.output,
-							msg.u.channel.tracefile_size,
-							msg.u.channel.tracefile_count,
-							0,
-							msg.u.channel.monitor,
-							msg.u.channel.live_timer_interval,
-							msg.u.channel.is_live,
-							nullptr,
-							nullptr);
+		new_channel = consumer_allocate_channel(
+			msg.u.channel.channel_key,
+			msg.u.channel.session_id,
+			msg.u.channel.chunk_id.is_set ? &chunk_id : nullptr,
+			msg.u.channel.pathname,
+			msg.u.channel.name,
+			msg.u.channel.relayd_id,
+			msg.u.channel.output,
+			msg.u.channel.tracefile_size,
+			msg.u.channel.tracefile_count,
+			0,
+			0,
+			msg.u.channel.monitor,
+			msg.u.channel.live_timer_interval,
+			msg.u.channel.is_live,
+			false,
+			nonstd::nullopt,
+			nullptr,
+			nullptr,
+			static_cast<enum lttng_trace_format>(msg.u.channel.trace_format));
 		if (new_channel == nullptr) {
-			lttng_consumer_send_error(ctx, LTTCOMM_CONSUMERD_OUTFD_ERROR);
+			lttng_consumer_send_error(ctx->consumer_error_socket,
+						  LTTCOMM_CONSUMERD_OUTFD_ERROR);
 			goto end_nosignal;
 		}
 		new_channel->nb_init_stream_left = msg.u.channel.nb_init_streams;
@@ -626,9 +632,13 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 			int monitor_start_ret;
 
 			DBG("Consumer starting monitor timer");
-			consumer_timer_live_start(new_channel, msg.u.channel.live_timer_interval);
-			monitor_start_ret = consumer_timer_monitor_start(
-				new_channel, msg.u.channel.monitor_timer_interval);
+			consumer_timer_live_start(new_channel,
+						  msg.u.channel.live_timer_interval,
+						  ctx->timer_task_scheduler);
+			monitor_start_ret =
+				consumer_timer_monitor_start(new_channel,
+							     msg.u.channel.monitor_timer_interval,
+							     ctx->timer_task_scheduler);
 			if (monitor_start_ret < 0) {
 				ERR("Starting channel monitoring timer failed");
 				goto end_nosignal;
@@ -658,6 +668,8 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		int ret_send_status, ret_poll, ret_get_max_subbuf_size;
 		ssize_t ret_pipe_write, ret_recv;
 
+		const lttng::pthread::lock_guard consumer_data_lock(the_consumer_data.lock);
+
 		/*
 		 * Get stream's channel reference. Needed when adding the stream to the
 		 * global hash table.
@@ -672,6 +684,9 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 			ret_code = LTTCOMM_CONSUMERD_CHAN_NOT_FOUND;
 		}
 
+		const lttng::pthread::lock_guard channel_lock(channel->lock);
+		const lttng::pthread::lock_guard timer_lock(channel->timer_lock);
+
 		health_code_update();
 
 		/* First send a status message before receiving the fds. */
@@ -701,7 +716,8 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		/* Get stream file descriptor from socket */
 		ret_recv = lttcomm_recv_fds_unix_sock(sock, &fd, 1);
 		if (ret_recv != sizeof(fd)) {
-			lttng_consumer_send_error(ctx, LTTCOMM_CONSUMERD_ERROR_RECV_FD);
+			lttng_consumer_send_error(ctx->consumer_error_socket,
+						  LTTCOMM_CONSUMERD_ERROR_RECV_FD);
 			ret_func = ret_recv;
 			goto end;
 		}
@@ -721,27 +737,30 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 
 		health_code_update();
 
-		pthread_mutex_lock(&channel->lock);
-		new_stream = consumer_stream_create(channel,
-						    channel->key,
-						    fd,
-						    channel->name,
-						    channel->relayd_id,
-						    channel->session_id,
-						    channel->trace_chunk,
-						    msg.u.stream.cpu,
-						    &alloc_ret,
-						    channel->type,
-						    channel->monitor);
+		try {
+			new_stream = consumer_stream_create(channel,
+							    channel->key,
+							    fd,
+							    channel->name,
+							    channel->relayd_id,
+							    channel->session_id,
+							    channel->trace_chunk,
+							    msg.u.stream.cpu,
+							    &alloc_ret,
+							    channel->type,
+							    channel->monitor);
+		} catch (const std::bad_alloc&) {
+			LTTNG_ASSERT(!new_stream);
+		}
 		if (new_stream == nullptr) {
 			switch (alloc_ret) {
-			case -ENOMEM:
 			case -EINVAL:
 			default:
-				lttng_consumer_send_error(ctx, LTTCOMM_CONSUMERD_OUTFD_ERROR);
+				lttng_consumer_send_error(ctx->consumer_error_socket,
+							  LTTCOMM_CONSUMERD_OUTFD_ERROR);
 				break;
 			}
-			pthread_mutex_unlock(&channel->lock);
+
 			goto error_add_stream_nosignal;
 		}
 
@@ -749,7 +768,6 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		ret_get_max_subbuf_size =
 			kernctl_get_max_subbuf_size(new_stream->wait_fd, &new_stream->max_sb_size);
 		if (ret_get_max_subbuf_size < 0) {
-			pthread_mutex_unlock(&channel->lock);
 			ERR("Failed to get kernel maximal subbuffer size");
 			goto error_add_stream_nosignal;
 		}
@@ -783,7 +801,6 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 			const int ret_recv_stream = ctx->on_recv_stream(new_stream);
 			if (ret_recv_stream < 0) {
 				pthread_mutex_unlock(&new_stream->lock);
-				pthread_mutex_unlock(&channel->lock);
 				consumer_stream_free(new_stream);
 				goto error_add_stream_nosignal;
 			}
@@ -802,7 +819,6 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 			    new_stream->net_seq_idx);
 			cds_list_add(&new_stream->send_node, &channel->streams.head);
 			pthread_mutex_unlock(&new_stream->lock);
-			pthread_mutex_unlock(&channel->lock);
 			goto end_add_stream;
 		}
 
@@ -814,7 +830,6 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 				consumer_send_relayd_stream(new_stream, new_stream->chan->pathname);
 			if (ret_send_relayd_stream < 0) {
 				pthread_mutex_unlock(&new_stream->lock);
-				pthread_mutex_unlock(&channel->lock);
 				consumer_stream_free(new_stream);
 				goto error_add_stream_nosignal;
 			}
@@ -831,13 +846,10 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 					consumer_send_relayd_streams_sent(new_stream->net_seq_idx);
 				if (ret_send_relayd_streams_sent < 0) {
 					pthread_mutex_unlock(&new_stream->lock);
-					pthread_mutex_unlock(&channel->lock);
 					goto error_add_stream_nosignal;
 				}
 			}
 		}
-		pthread_mutex_unlock(&new_stream->lock);
-		pthread_mutex_unlock(&channel->lock);
 
 		/* Get the right pipe where the stream will be sent. */
 		if (new_stream->metadata_flag) {
@@ -848,6 +860,8 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 			stream_pipe = ctx->consumer_data_pipe;
 		}
 
+		pthread_mutex_unlock(&new_stream->lock);
+
 		/* Visible to other threads */
 		new_stream->globally_visible = 1;
 
@@ -1011,6 +1025,7 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 	}
 	case LTTNG_CONSUMER_SNAPSHOT_CHANNEL:
 	{
+		const lttng::pthread::lock_guard consumer_data_lock(the_consumer_data.lock);
 		struct lttng_consumer_channel *channel;
 		const uint64_t key = msg.u.snapshot_channel.key;
 		int ret_send_status;
@@ -1222,6 +1237,8 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 
 		DBG("Consumer rotate channel %" PRIu64, key);
 
+		const lttng::pthread::lock_guard consumer_data_lock(the_consumer_data.lock);
+
 		channel = consumer_find_channel(key);
 		if (!channel) {
 			ERR("Channel %" PRIu64 " not found", key);
@@ -1266,14 +1283,15 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		const uint64_t key = msg.u.clear_channel.key;
 		int ret_send_status;
 
+		const lttng::pthread::lock_guard global_lock(the_consumer_data.lock);
+
 		channel = consumer_find_channel(key);
 		if (!channel) {
 			DBG("Channel %" PRIu64 " not found", key);
 			ret_code = LTTCOMM_CONSUMERD_CHAN_NOT_FOUND;
 		} else {
-			int ret_clear_channel;
-
-			ret_clear_channel = lttng_consumer_clear_channel(channel);
+			const lttng::pthread::lock_guard channel_lock(channel->lock);
+			const auto ret_clear_channel = lttng_consumer_clear_channel(channel);
 			if (ret_clear_channel) {
 				ERR("Clear channel failed");
 				ret_code = (lttcomm_return_code) ret_clear_channel;
@@ -1408,6 +1426,7 @@ int lttng_kconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 	}
 	case LTTNG_CONSUMER_OPEN_CHANNEL_PACKETS:
 	{
+		const lttng::pthread::lock_guard consumer_data_lock(the_consumer_data.lock);
 		const uint64_t key = msg.u.open_channel_packets.key;
 		struct lttng_consumer_channel *channel = consumer_find_channel(key);
 
@@ -1781,6 +1800,44 @@ static int signal_metadata(struct lttng_consumer_stream *stream,
 	return pthread_cond_broadcast(&stream->metadata_rdv) ? -errno : 0;
 }
 
+static int stream_send_live_beacon(lttng_consumer_stream& stream)
+{
+	uint64_t ts, stream_id;
+	int ret;
+
+	ret = kernctl_get_current_timestamp(stream.wait_fd, &ts);
+	if (ret < 0) {
+		ERR("Failed to get the current timestamp");
+		goto end;
+	}
+	ret = kernctl_buffer_flush(stream.wait_fd);
+	if (ret < 0) {
+		ERR("Failed to flush kernel stream");
+		goto end;
+	}
+	ret = kernctl_snapshot(stream.wait_fd);
+	if (ret < 0) {
+		if (ret != -EAGAIN && ret != -ENODATA) {
+			PERROR("live timer kernel snapshot");
+			ret = -1;
+			goto end;
+		}
+		ret = kernctl_get_stream_id(stream.wait_fd, &stream_id);
+		if (ret < 0) {
+			PERROR("kernctl_get_stream_id");
+			goto end;
+		}
+		DBG("Stream %" PRIu64 " empty, sending beacon", stream.key);
+		ret = consumer_stream_send_live_beacon(stream, ts, stream_id);
+		if (ret < 0) {
+			goto end;
+		}
+	}
+	ret = 0;
+end:
+	return ret;
+}
+
 static int lttng_kconsumer_set_stream_ops(struct lttng_consumer_stream *stream)
 {
 	int ret = 0;
@@ -1827,7 +1884,7 @@ static int lttng_kconsumer_set_stream_ops(struct lttng_consumer_stream *stream)
 	} else {
 		stream->read_subbuffer_ops.extract_subbuffer_info = extract_data_subbuffer_info;
 		if (stream->chan->is_live) {
-			stream->read_subbuffer_ops.send_live_beacon = consumer_flush_kernel_index;
+			stream->read_subbuffer_ops.send_live_beacon = stream_send_live_beacon;
 		}
 	}
 
