@@ -15,10 +15,11 @@
 #include <common/tracker.hpp>
 
 #include <lttng/channel.h>
+#include <lttng/session.h>
 #include <lttng/snapshot-internal.hpp>
 
 #define MI_SCHEMA_MAJOR_VERSION 4
-#define MI_SCHEMA_MINOR_VERSION 1
+#define MI_SCHEMA_MINOR_VERSION 2
 
 /* Machine interface namespace URI */
 const char *const mi_lttng_xmlns = "xmlns";
@@ -363,6 +364,21 @@ const char *const mi_lttng_element_error_query_results = "error_query_results";
 /* String related to add-context command */
 const char *const mi_lttng_element_context_symbol = "symbol";
 
+/* Memory usage elements */
+const char *const mi_lttng_element_data_stream_info_sets = "data_stream_info_sets";
+const char *const mi_lttng_element_data_stream_info_set_list = "data_stream_info_list";
+const char *const mi_lttng_element_data_stream_info_set = "data_stream_info_set";
+const char *const mi_lttng_element_data_stream_info_list = "data_stream_info_list";
+const char *const mi_lttng_element_data_stream_info = "data_stream_info";
+const char *const mi_lttng_element_total_memory_usage_bytes = "total_memory_usage_bytes";
+const char *const mi_lttng_element_memory_usage_bytes = "memory_usage_bytes";
+const char *const mi_lttng_element_app_bitness = "app_bitness";
+const char *const mi_lttng_element_uid = "uid";
+const char *const mi_lttng_element_uid_id = "id";
+const char *const mi_lttng_element_pid = "pid";
+const char *const mi_lttng_element_cpu_id = "cpu_id";
+const char *const mi_lttng_element_sets = "sets";
+
 /* This is a merge of jul loglevel and regular loglevel
  * Those should never overlap by definition
  * (see struct lttng_event loglevel)
@@ -983,6 +999,28 @@ int mi_lttng_session(struct mi_writer *writer, const struct lttng_session *sessi
 		goto end;
 	}
 
+	/* Trace format */
+	{
+		enum lttng_trace_format trace_format;
+
+		if (lttng_get_session_trace_format(session, &trace_format) !=
+		    LTTNG_GET_SESSION_TRACE_FORMAT_STATUS_OK) {
+			ERR("Failed to get session trace format");
+			ret = -1;
+			goto end;
+		}
+
+		ret = mi_lttng_writer_write_element_string(
+			writer,
+			config_element_trace_format,
+			trace_format == LTTNG_TRACE_FORMAT_CTF_1_8 ?
+				config_element_trace_format_ctf_1_8 :
+				config_element_trace_format_ctf_2);
+		if (ret) {
+			goto end;
+		}
+	}
+
 	/* Enabled ? */
 	ret = mi_lttng_writer_write_element_bool(writer, config_element_enabled, session->enabled);
 	if (ret) {
@@ -1117,14 +1155,78 @@ static const char *allocation_policy_to_string(enum lttng_channel_allocation_pol
 	}
 }
 
+static const char *preallocation_policy_to_string(enum lttng_channel_preallocation_policy policy)
+{
+	switch (policy) {
+	case LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE:
+		return config_element_channel_preallocation_policy_preallocate;
+	case LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND:
+		return config_element_channel_preallocation_policy_on_demand;
+	default:
+		return nullptr;
+	}
+}
+
+static int
+mi_emit_channel_reclamation_policy(const nonstd::optional<uint64_t>& memory_reclamation_policy,
+				   struct mi_writer *writer)
+{
+	if (!memory_reclamation_policy) {
+		return 0;
+	}
+
+	int ret = mi_lttng_writer_open_element(writer, config_element_channel_reclaim_policy);
+
+	if (ret) {
+		return ret;
+	}
+
+	const auto maximal_age_us = memory_reclamation_policy.value();
+
+	if (maximal_age_us) {
+		ret = mi_lttng_writer_open_element(writer,
+						   config_element_channel_reclaim_policy_periodic);
+		if (ret) {
+			return ret;
+		}
+		ret = mi_lttng_writer_write_element_unsigned_int(
+			writer,
+			config_element_channel_reclaim_policy_periodic_age_threshold,
+			maximal_age_us);
+		if (ret) {
+			return ret;
+		}
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			return ret;
+		}
+	} else {
+		ret = mi_lttng_writer_open_element(writer,
+						   config_element_channel_reclaim_policy_consumed);
+		if (ret) {
+			return ret;
+		}
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			return ret;
+		}
+	}
+
+	return mi_lttng_writer_close_element(writer);
+}
+
 int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *attr)
 {
 	int ret = 0;
 	struct lttng_channel *chan = caa_container_of(attr, struct lttng_channel, attr);
-	uint64_t discarded_events, lost_packets, monitor_timer_interval;
+	uint64_t discarded_events, lost_packets, monitor_timer_interval, watchdog_timer_interval;
 	int64_t blocking_timeout;
 	enum lttng_channel_allocation_policy allocation_policy;
-	const char *allocation_policy_str;
+	enum lttng_channel_preallocation_policy preallocation_policy;
+	const char *allocation_policy_str, *preallocation_policy_str;
+	enum lttng_channel_status watchdog_timer_status;
+	bool print_watchdog_timer = false;
+	nonstd::optional<uint64_t> memory_reclamation_policy;
 
 	LTTNG_ASSERT(attr);
 
@@ -1143,6 +1245,19 @@ int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *a
 		goto end;
 	}
 
+	watchdog_timer_status =
+		lttng_channel_get_watchdog_timer_interval(chan, &watchdog_timer_interval);
+	switch (watchdog_timer_status) {
+	case LTTNG_CHANNEL_STATUS_INVALID:
+		ret = -LTTNG_ERR_INVALID;
+		goto end;
+	case LTTNG_CHANNEL_STATUS_UNSET:
+		break;
+	case LTTNG_CHANNEL_STATUS_OK:
+		print_watchdog_timer = true;
+		break;
+	}
+
 	ret = lttng_channel_get_blocking_timeout(chan, &blocking_timeout);
 	if (ret) {
 		goto end;
@@ -1159,6 +1274,30 @@ int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *a
 		goto end;
 	}
 
+	ret = lttng_channel_get_preallocation_policy(chan, &preallocation_policy);
+	if (ret != LTTNG_CHANNEL_STATUS_OK) {
+		goto end;
+	}
+
+	preallocation_policy_str = preallocation_policy_to_string(preallocation_policy);
+	if (!preallocation_policy_str) {
+		ret = LTTNG_ERR_SAVE_IO_FAIL;
+		goto end;
+	}
+
+	uint64_t maximal_age_us;
+	ret = lttng_channel_get_automatic_memory_reclamation_policy(chan, &maximal_age_us);
+	switch (ret) {
+	case LTTNG_CHANNEL_STATUS_OK:
+		memory_reclamation_policy = maximal_age_us;
+		break;
+	case LTTNG_CHANNEL_STATUS_UNSET:
+		break;
+	default:
+		ret = LTTNG_ERR_SAVE_IO_FAIL;
+		goto end;
+	}
+
 	/* Opening Attributes */
 	ret = mi_lttng_writer_open_element(writer, config_element_attributes);
 	if (ret) {
@@ -1209,6 +1348,15 @@ int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *a
 		goto end;
 	}
 
+	if (print_watchdog_timer) {
+		/* Watchdog timer interval in usec */
+		ret = mi_lttng_writer_write_element_unsigned_int(
+			writer, config_element_watchdog_timer_interval, watchdog_timer_interval);
+		if (ret) {
+			goto end;
+		}
+	}
+
 	/* Retry timeout in usec */
 	ret = mi_lttng_writer_write_element_signed_int(
 		writer, config_element_blocking_timeout, blocking_timeout);
@@ -1223,6 +1371,19 @@ int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *a
 		goto end;
 	}
 
+	/* Preallocation policy */
+	ret = mi_lttng_writer_write_element_string(
+		writer, config_element_channel_preallocation_policy, preallocation_policy_str);
+	if (ret) {
+		goto end;
+	}
+
+	/* Memory reclamation. */
+	ret = mi_emit_channel_reclamation_policy(memory_reclamation_policy, writer);
+	if (ret) {
+		goto end;
+	}
+
 	/* Event output */
 	ret = mi_lttng_writer_write_element_string(writer,
 						   config_element_output_type,
@@ -1277,6 +1438,237 @@ end:
 	return ret;
 }
 
+namespace {
+
+class xml_element_guard {
+public:
+	explicit xml_element_guard(struct mi_writer *writer) : _writer(writer)
+	{
+	}
+
+	xml_element_guard(const xml_element_guard&) = delete;
+	xml_element_guard(xml_element_guard&&) = delete;
+	xml_element_guard& operator=(const xml_element_guard&) = delete;
+	xml_element_guard& operator=(xml_element_guard&&) = delete;
+
+	~xml_element_guard()
+	{
+		if (_opened) {
+			mi_lttng_writer_close_element(_writer);
+		}
+	}
+
+	int open(const char *element_name)
+	{
+		if (_opened) {
+			return -1;
+		}
+
+		const auto ret = mi_lttng_writer_open_element(_writer, element_name);
+		if (ret == 0) {
+			_opened = true;
+		}
+
+		return ret;
+	}
+
+	int close()
+	{
+		if (!_opened) {
+			return 0;
+		}
+
+		const auto ret = mi_lttng_writer_close_element(_writer);
+		_opened = false;
+		return ret;
+	}
+
+private:
+	struct mi_writer *_writer;
+	bool _opened = false;
+};
+
+int write_uid_element(struct mi_writer *writer, const struct lttng_data_stream_info_set *set)
+{
+	uid_t uid;
+	auto status = lttng_data_stream_info_set_get_uid(set, &uid);
+
+	if (status == LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+		xml_element_guard uid_guard(writer);
+
+		if (uid_guard.open(mi_lttng_element_uid)) {
+			return -1;
+		}
+
+		return mi_lttng_writer_write_element_unsigned_int(
+			writer, mi_lttng_element_uid_id, uid);
+	} else if (status != LTTNG_DATA_STREAM_INFO_STATUS_NONE) {
+		return -1;
+	}
+	return 0;
+}
+
+int write_pid_element(struct mi_writer *writer, const struct lttng_data_stream_info_set *set)
+{
+	pid_t pid;
+	auto status = lttng_data_stream_info_set_get_pid(set, &pid);
+
+	if (status == LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+		return mi_lttng_writer_write_element_unsigned_int(
+			writer, mi_lttng_element_pid, pid);
+	} else if (status != LTTNG_DATA_STREAM_INFO_STATUS_NONE) {
+		return -1;
+	}
+	return 0;
+}
+
+int write_app_bitness_element(struct mi_writer *writer,
+			      const struct lttng_data_stream_info_set *set)
+{
+	enum lttng_app_bitness bitness;
+	auto status = lttng_data_stream_info_set_get_app_bitness(set, &bitness);
+
+	if (status == LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+		const auto bitness_value = (bitness == LTTNG_APP_BITNESS_32) ? 32U : 64U;
+
+		return mi_lttng_writer_write_element_unsigned_int(
+			writer, mi_lttng_element_app_bitness, bitness_value);
+	} else if (status != LTTNG_DATA_STREAM_INFO_STATUS_NONE) {
+		return -1;
+	}
+
+	return 0;
+}
+
+std::int64_t write_stream_info(struct mi_writer *writer,
+			       const struct lttng_data_stream_info *stream_info)
+{
+	xml_element_guard stream_guard(writer);
+	if (stream_guard.open(mi_lttng_element_data_stream_info) != 0) {
+		return -1;
+	}
+
+	/* Write CPU ID if available. */
+	unsigned int cpu_id;
+	auto status = lttng_data_stream_info_get_cpu_id(stream_info, &cpu_id);
+	if (status == LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+		if (mi_lttng_writer_write_element_unsigned_int(
+			    writer, mi_lttng_element_cpu_id, cpu_id) != 0) {
+			return -1;
+		}
+	} else if (status != LTTNG_DATA_STREAM_INFO_STATUS_NONE) {
+		return -1;
+	}
+
+	/* Write memory usage. */
+	uint64_t memory_usage;
+	status = lttng_data_stream_info_get_memory_usage(stream_info, &memory_usage);
+	if (status == LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+		if (mi_lttng_writer_write_element_unsigned_int(
+			    writer, mi_lttng_element_memory_usage_bytes, memory_usage) != 0) {
+			return -1;
+		}
+
+		return memory_usage;
+	} else {
+		/* Memory usage should always be available. */
+		return -1;
+	}
+}
+
+} /* namespace */
+
+int mi_lttng_data_stream_info_sets(struct mi_writer *writer,
+				   const struct lttng_data_stream_info_sets *ds_info_sets,
+				   unsigned int ds_info_sets_count)
+{
+	LTTNG_ASSERT(writer);
+	LTTNG_ASSERT(ds_info_sets);
+
+	xml_element_guard sets_element(writer);
+	if (sets_element.open(mi_lttng_element_data_stream_info_sets)) {
+		return -1;
+	}
+
+	xml_element_guard sets_list(writer);
+	if (sets_list.open(mi_lttng_element_sets)) {
+		return -1;
+	}
+
+	std::uint64_t total_memory = 0;
+	for (unsigned int set_idx = 0; set_idx < ds_info_sets_count; set_idx++) {
+		const struct lttng_data_stream_info_set *set;
+
+		auto status = lttng_data_stream_info_sets_get_at_index(ds_info_sets, set_idx, &set);
+		if (status != LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+			return -1;
+		}
+
+		xml_element_guard set_element(writer);
+		if (set_element.open(mi_lttng_element_data_stream_info_set) != 0) {
+			return -1;
+		}
+
+		if (write_uid_element(writer, set) != 0 || write_pid_element(writer, set) != 0 ||
+		    write_app_bitness_element(writer, set) != 0) {
+			return -1;
+		}
+
+		unsigned int set_count;
+		status = lttng_data_stream_info_set_get_count(set, &set_count);
+		if (status != LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+			return -1;
+		}
+
+		xml_element_guard stream_list(writer);
+		if (stream_list.open(mi_lttng_element_data_stream_info_list) != 0) {
+			return -1;
+		}
+
+		std::uint64_t set_memory_total = 0;
+		for (unsigned int stream_idx = 0; stream_idx < set_count; stream_idx++) {
+			const struct lttng_data_stream_info *stream_info;
+			status = lttng_data_stream_info_set_get_at_index(
+				set, stream_idx, &stream_info);
+			if (status != LTTNG_DATA_STREAM_INFO_STATUS_OK) {
+				return -1;
+			}
+
+			const auto stream_memory = write_stream_info(writer, stream_info);
+			if (stream_memory < 0) {
+				return -1;
+			}
+
+			set_memory_total += stream_memory;
+			total_memory += stream_memory;
+		}
+
+		if (stream_list.close() != 0) {
+			return -1;
+		}
+
+		if (mi_lttng_writer_write_element_unsigned_int(
+			    writer, mi_lttng_element_total_memory_usage_bytes, set_memory_total) !=
+		    0) {
+			return -1;
+		}
+
+		/* set_element will automatically close when it goes out of scope */
+	}
+
+	if (sets_list.close() != 0) {
+		return -1;
+	}
+
+	if (mi_lttng_writer_write_element_unsigned_int(
+		    writer, mi_lttng_element_total_memory_usage_bytes, total_memory) != 0) {
+		return -1;
+	}
+
+	/* sets_element will automatically close when it goes out of scope */
+	return 0;
+}
+
 int mi_lttng_event_common_attributes(struct mi_writer *writer, struct lttng_event *event)
 {
 	int ret;
@@ -1748,7 +2140,7 @@ int mi_lttng_pid(struct mi_writer *writer, pid_t pid, const char *name, int is_o
 	int ret;
 
 	/* Open pid process */
-	ret = mi_lttng_writer_open_element(writer, config_element_pid);
+	ret = mi_lttng_writer_open_element(writer, mi_lttng_element_pid);
 	if (ret) {
 		goto end;
 	}
