@@ -11,10 +11,12 @@
 
 #include <common/config/session-config.hpp>
 #include <common/error.hpp>
+#include <common/exception.hpp>
 #include <common/macros.hpp>
 
 #include <lttng/lttng.h>
 
+#include <memory>
 #include <stdint.h>
 
 /* Don't want to reference snapshot-internal.h here */
@@ -365,6 +367,21 @@ extern const char *const mi_lttng_element_error_query_results;
 /* String related to add-context command */
 extern const char *const mi_lttng_element_context_symbol;
 
+/* Memory usage elements */
+extern const char *const mi_lttng_element_data_stream_info_sets;
+extern const char *const mi_lttng_element_data_stream_info_set_list;
+extern const char *const mi_lttng_element_data_stream_info_set;
+extern const char *const mi_lttng_element_data_stream_info_list;
+extern const char *const mi_lttng_element_data_stream_info;
+extern const char *const mi_lttng_element_total_memory_usage_bytes;
+extern const char *const mi_lttng_element_memory_usage_bytes;
+extern const char *const mi_lttng_element_app_bitness;
+extern const char *const mi_lttng_element_uid;
+extern const char *const mi_lttng_element_uid_id;
+extern const char *const mi_lttng_element_pid;
+extern const char *const mi_lttng_element_cpu_id;
+extern const char *const mi_lttng_element_sets;
+
 /* Utility string function  */
 const char *mi_lttng_loglevel_string(int value, enum lttng_domain_type domain);
 const char *mi_lttng_logleveltype_string(enum lttng_loglevel_type value);
@@ -626,6 +643,20 @@ int mi_lttng_channel(struct mi_writer *writer, struct lttng_channel *channel, in
  */
 int mi_lttng_channel_attr(struct mi_writer *writer, struct lttng_channel_attr *attr);
 
+/*
+ * Machine interface for channel memory usage data stream info sets.
+ *
+ * writer An instance of a mi writer.
+ * ds_info_sets The data stream info sets containing memory usage information.
+ * ds_info_sets_count The number of data stream info sets.
+ *
+ * Returns zero if the element's value could be written.
+ * Negative values indicate an error.
+ */
+int mi_lttng_data_stream_info_sets(struct mi_writer *writer,
+				   const struct lttng_data_stream_info_sets *ds_info_sets,
+				   unsigned int ds_info_sets_count);
+
 /*
  * Machine interface for event common attributes.
  *
@@ -1078,4 +1109,18 @@ int mi_lttng_rotate(struct mi_writer *writer,
 		    enum lttng_rotation_state rotation_state,
 		    const struct lttng_trace_archive_location *location);
 
+namespace details {
+static inline void _mi_lttng_writer_deleter_func(mi_writer *writer)
+{
+	if (writer && mi_lttng_writer_destroy(writer)) {
+		LTTNG_THROW_ERROR("Failed to destroy mi_writer instance");
+	}
+}
+} /* namespace details */
+
+using mi_writer_uptr = std::unique_ptr<
+	mi_writer,
+	lttng::memory::create_deleter_class<mi_writer,
+					    details::_mi_lttng_writer_deleter_func>::deleter>;
+
 #endif /* _MI_LTTNG_H */
