@@ -14,6 +14,7 @@
 #include <common/defaults.hpp>
 #include <common/index/ctf-index.hpp>
 #include <common/sessiond-comm/relayd.hpp>
+#include <common/string-utils/c-string-view.hpp>
 #include <common/string-utils/format.hpp>
 #include <common/trace-chunk.hpp>
 
@@ -22,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <vector>
 
 static bool relayd_supports_chunks(const struct lttcomm_relayd_sock *sock)
 {
@@ -46,7 +48,7 @@ static bool relayd_supports_get_configuration(const struct lttcomm_relayd_sock *
 /*
  * Send command. Fill up the header and append the data.
  */
-static int send_command(struct lttcomm_relayd_sock *rsock,
+static int send_command(lttcomm_relayd_sock& rsock,
 			enum lttcomm_relayd_command cmd,
 			const void *data,
 			size_t size,
@@ -57,7 +59,7 @@ static int send_command(struct lttcomm_relayd_sock *rsock,
 	char *buf;
 	uint64_t buf_size = sizeof(header);
 
-	if (rsock->sock.fd < 0) {
+	if (rsock.sock.fd < 0) {
 		return -ECONNRESET;
 	}
 
@@ -89,12 +91,12 @@ static int send_command(struct lttcomm_relayd_sock *rsock,
 	DBG3("Relayd sending command %s of size %" PRIu64,
 	     lttcomm_relayd_command_str(cmd),
 	     buf_size);
-	ret = rsock->sock.ops->sendmsg(&rsock->sock, buf, buf_size, flags);
+	ret = rsock.sock.ops->sendmsg(&rsock.sock, buf, buf_size, flags);
 	if (ret < 0) {
 		PERROR("Failed to send command %s of size %" PRIu64,
 		       lttcomm_relayd_command_str(cmd),
 		       buf_size);
-		ret = rsock->sock.ops->sendmsg(&rsock->sock, buf, buf_size, flags);
+		ret = rsock.sock.ops->sendmsg(&rsock.sock, buf, buf_size, flags);
 		ret = -errno;
 		goto error;
 	}
@@ -108,24 +110,24 @@ alloc_error:
  * Receive reply data on socket. This MUST be call after send_command or else
  * could result in unexpected behavior(s).
  */
-static int recv_reply(struct lttcomm_relayd_sock *rsock, void *data, size_t size)
+static int recv_reply(lttcomm_relayd_sock& rsock, void *data, size_t size)
 {
 	int ret;
 
-	if (rsock->sock.fd < 0) {
+	if (rsock.sock.fd < 0) {
 		return -ECONNRESET;
 	}
 
 	DBG3("Relayd waiting for reply of size %zu", size);
 
-	ret = rsock->sock.ops->recvmsg(&rsock->sock, data, size, 0);
+	ret = rsock.sock.ops->recvmsg(&rsock.sock, data, size, 0);
 	if (ret <= 0 || ret != size) {
 		if (ret == 0) {
 			/* Orderly shutdown. */
-			DBG("Socket %d has performed an orderly shutdown", rsock->sock.fd);
+			DBG("Socket %d has performed an orderly shutdown", rsock.sock.fd);
 		} else {
 			DBG("Receiving reply failed on sock %d for size %zu with ret %d",
-			    rsock->sock.fd,
+			    rsock.sock.fd,
 			    size,
 			    ret);
 		}
@@ -144,6 +146,85 @@ error:
  * Length for both payloads is stored in the msg struct. A new dynamic size
  * payload size is introduced.
  */
+/*
+ * Common helper to populate base fields for 2.11+ create session messages.
+ * Returns 0 on success, negative value on error.
+ */
+static int relayd_create_session_2_11_base_populate(
+	struct lttcomm_relayd_create_session_2_11_base *base_header,
+	char *names_buffer,
+	const char *session_name,
+	const char *hostname,
+	const char *base_path,
+	int session_live_timer,
+	unsigned int snapshot,
+	uint64_t sessiond_session_id,
+	const lttng_uuid& sessiond_uuid,
+	const uint64_t *current_chunk_id,
+	time_t creation_time,
+	bool session_name_contains_creation_time)
+{
+	int ret;
+	size_t session_name_len;
+	size_t hostname_len;
+	size_t base_path_len;
+	char *dst;
+
+	if (!base_path) {
+		base_path = "";
+	}
+
+	/* The three names are sent with a '\0' delimiter between them. */
+	session_name_len = strlen(session_name) + 1;
+	hostname_len = strlen(hostname) + 1;
+	base_path_len = strlen(base_path) + 1;
+
+	/* Populate length fields in base header. */
+	LTTNG_ASSERT(session_name_len <= UINT32_MAX);
+	base_header->session_name_len = htobe32(session_name_len);
+
+	LTTNG_ASSERT(hostname_len <= UINT32_MAX);
+	base_header->hostname_len = htobe32(hostname_len);
+
+	LTTNG_ASSERT(base_path_len <= UINT32_MAX);
+	base_header->base_path_len = htobe32(base_path_len);
+
+	/* Copy names into the buffer. */
+	dst = names_buffer;
+	if (lttng_strncpy(dst, session_name, session_name_len)) {
+		ret = -1;
+		goto error;
+	}
+	dst += session_name_len;
+	if (lttng_strncpy(dst, hostname, hostname_len)) {
+		ret = -1;
+		goto error;
+	}
+	dst += hostname_len;
+	if (lttng_strncpy(dst, base_path, base_path_len)) {
+		ret = -1;
+		goto error;
+	}
+
+	/* Populate base fields. */
+	base_header->live_timer = htobe32(session_live_timer);
+	base_header->snapshot = !!snapshot;
+
+	std::copy(sessiond_uuid.begin(), sessiond_uuid.end(), base_header->sessiond_uuid);
+	base_header->session_id = htobe64(sessiond_session_id);
+	base_header->session_name_contains_creation_time = session_name_contains_creation_time;
+	if (current_chunk_id) {
+		LTTNG_OPTIONAL_SET(&base_header->current_chunk_id, htobe64(*current_chunk_id));
+	}
+
+	base_header->creation_time = htobe64((uint64_t) creation_time);
+
+	ret = 0;
+
+error:
+	return ret;
+}
+
 static int relayd_create_session_2_11(struct lttcomm_relayd_sock *rsock,
 				      const char *session_name,
 				      const char *hostname,
@@ -164,17 +245,18 @@ static int relayd_create_session_2_11(struct lttcomm_relayd_sock *rsock,
 	size_t hostname_len;
 	size_t base_path_len;
 	size_t msg_length;
-	char *dst;
 
 	if (!base_path) {
 		base_path = "";
 	}
-	/* The three names are sent with a '\0' delimiter between them. */
+
+	/* Calculate message size. */
 	session_name_len = strlen(session_name) + 1;
 	hostname_len = strlen(hostname) + 1;
 	base_path_len = strlen(base_path) + 1;
-
 	msg_length = sizeof(*msg) + session_name_len + hostname_len + base_path_len;
+
+	/* Allocate message. */
 	msg = zmalloc<lttcomm_relayd_create_session_2_11>(msg_length);
 	if (!msg) {
 		PERROR("zmalloc create_session_2_11 command message");
@@ -182,50 +264,30 @@ static int relayd_create_session_2_11(struct lttcomm_relayd_sock *rsock,
 		goto error;
 	}
 
-	LTTNG_ASSERT(session_name_len <= UINT32_MAX);
-	msg->session_name_len = htobe32(session_name_len);
-
-	LTTNG_ASSERT(hostname_len <= UINT32_MAX);
-	msg->hostname_len = htobe32(hostname_len);
-
-	LTTNG_ASSERT(base_path_len <= UINT32_MAX);
-	msg->base_path_len = htobe32(base_path_len);
-
-	dst = msg->names;
-	if (lttng_strncpy(dst, session_name, session_name_len)) {
-		ret = -1;
+	/* Populate base fields and names using common helper. */
+	ret = relayd_create_session_2_11_base_populate(&msg->base,
+						       msg->names,
+						       session_name,
+						       hostname,
+						       base_path,
+						       session_live_timer,
+						       snapshot,
+						       sessiond_session_id,
+						       sessiond_uuid,
+						       current_chunk_id,
+						       creation_time,
+						       session_name_contains_creation_time);
+	if (ret < 0) {
 		goto error;
 	}
-	dst += session_name_len;
-	if (lttng_strncpy(dst, hostname, hostname_len)) {
-		ret = -1;
-		goto error;
-	}
-	dst += hostname_len;
-	if (lttng_strncpy(dst, base_path, base_path_len)) {
-		ret = -1;
-		goto error;
-	}
-
-	msg->live_timer = htobe32(session_live_timer);
-	msg->snapshot = !!snapshot;
-
-	std::copy(sessiond_uuid.begin(), sessiond_uuid.end(), msg->sessiond_uuid);
-	msg->session_id = htobe64(sessiond_session_id);
-	msg->session_name_contains_creation_time = session_name_contains_creation_time;
-	if (current_chunk_id) {
-		LTTNG_OPTIONAL_SET(&msg->current_chunk_id, htobe64(*current_chunk_id));
-	}
-
-	msg->creation_time = htobe64((uint64_t) creation_time);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_CREATE_SESSION, msg, msg_length, 0);
+	ret = send_command(*rsock, RELAYD_CREATE_SESSION, msg, msg_length, 0);
 	if (ret < 0) {
 		goto error;
 	}
 	/* Receive response */
-	ret = recv_reply(rsock, reply, sizeof(*reply));
+	ret = recv_reply(*rsock, reply, sizeof(*reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -240,7 +302,7 @@ static int relayd_create_session_2_11(struct lttcomm_relayd_sock *rsock,
 		ret = -1;
 		goto error;
 	}
-	ret = recv_reply(rsock, output_path, reply->output_path_length);
+	ret = recv_reply(*rsock, output_path, reply->output_path_length);
 	if (ret < 0) {
 		goto error;
 	}
@@ -248,6 +310,87 @@ error:
 	free(msg);
 	return ret;
 }
+
+static int relayd_create_session_2_15(struct lttcomm_relayd_sock *rsock,
+				      lttng::c_string_view session_name,
+				      lttng::c_string_view hostname,
+				      lttng::c_string_view base_path,
+				      int session_live_timer,
+				      unsigned int snapshot,
+				      uint64_t sessiond_session_id,
+				      const lttng_uuid& sessiond_uuid,
+				      const uint64_t *current_chunk_id,
+				      time_t creation_time,
+				      bool session_name_contains_creation_time,
+				      enum lttng_trace_format trace_format,
+				      struct lttcomm_relayd_create_session_reply_2_11 *reply,
+				      char *output_path)
+{
+	int ret;
+
+	LTTNG_ASSERT(session_name.data());
+	LTTNG_ASSERT(hostname.data());
+
+	/* Calculate message size. */
+	const auto msg_length = sizeof(lttcomm_relayd_create_session_2_15) + session_name.len() +
+		1 + hostname.len() + 1 + (base_path.data() ? base_path.len() : 0) + 1;
+
+	/* Allocate message. */
+	std::vector<char> buffer(msg_length);
+	const auto msg = reinterpret_cast<lttcomm_relayd_create_session_2_15 *>(buffer.data());
+
+	/* Populate base fields and names using common helper. */
+	ret = relayd_create_session_2_11_base_populate(&msg->base,
+						       msg->names,
+						       session_name.data(),
+						       hostname.data(),
+						       base_path.data(),
+						       session_live_timer,
+						       snapshot,
+						       sessiond_session_id,
+						       sessiond_uuid,
+						       current_chunk_id,
+						       creation_time,
+						       session_name_contains_creation_time);
+	if (ret < 0) {
+		return ret;
+	}
+
+	/* Populate 2.15-specific field. */
+	msg->trace_format = htobe32(static_cast<uint32_t>(trace_format));
+
+	/* Send command */
+	ret = send_command(*rsock, RELAYD_CREATE_SESSION, msg, msg_length, 0);
+	if (ret < 0) {
+		return ret;
+	}
+
+	/* Receive response */
+	ret = recv_reply(*rsock, reply, sizeof(*reply));
+	if (ret < 0) {
+		return ret;
+	}
+
+	reply->generic.session_id = be64toh(reply->generic.session_id);
+	reply->generic.ret_code = be32toh(reply->generic.ret_code);
+	reply->output_path_length = be32toh(reply->output_path_length);
+	if (reply->output_path_length >= LTTNG_PATH_MAX) {
+		const auto output_path_length = reply->output_path_length;
+
+		ERR_FMT("Invalid session output path length in reply: length={}, max={}",
+			output_path_length,
+			LTTNG_PATH_MAX);
+		return -1;
+	}
+
+	ret = recv_reply(*rsock, output_path, reply->output_path_length);
+	if (ret < 0) {
+		return ret;
+	}
+
+	return ret;
+}
+
 /*
  * From 2.4 to 2.10, RELAYD_CREATE_SESSION takes additional parameters to
  * support the live reading capability.
@@ -274,13 +417,13 @@ static int relayd_create_session_2_4(struct lttcomm_relayd_sock *rsock,
 	msg.snapshot = htobe32(snapshot);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_CREATE_SESSION, &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_CREATE_SESSION, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, reply, sizeof(*reply));
+	ret = recv_reply(*rsock, reply, sizeof(*reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -299,13 +442,13 @@ static int relayd_create_session_2_1(struct lttcomm_relayd_sock *rsock,
 	int ret;
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_CREATE_SESSION, nullptr, 0, 0);
+	ret = send_command(*rsock, RELAYD_CREATE_SESSION, nullptr, 0, 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, reply, sizeof(*reply));
+	ret = recv_reply(*rsock, reply, sizeof(*reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -334,6 +477,7 @@ int relayd_create_session(struct lttcomm_relayd_sock *rsock,
 			  const uint64_t *current_chunk_id,
 			  time_t creation_time,
 			  bool session_name_contains_creation_time,
+			  enum lttng_trace_format trace_format,
 			  char *output_path)
 {
 	int ret;
@@ -351,8 +495,8 @@ int relayd_create_session(struct lttcomm_relayd_sock *rsock,
 		/* From 2.4 to 2.10 */
 		ret = relayd_create_session_2_4(
 			rsock, session_name, hostname, session_live_timer, snapshot, &reply.generic);
-	} else {
-		/* From 2.11 to ... */
+	} else if (rsock->minor >= 11 && rsock->minor < 15) {
+		/* From 2.11 to 2.14 */
 		ret = relayd_create_session_2_11(rsock,
 						 session_name,
 						 hostname,
@@ -366,6 +510,22 @@ int relayd_create_session(struct lttcomm_relayd_sock *rsock,
 						 session_name_contains_creation_time,
 						 &reply,
 						 output_path);
+	} else {
+		/* From 2.15 to ... */
+		ret = relayd_create_session_2_15(rsock,
+						 session_name,
+						 hostname,
+						 base_path,
+						 session_live_timer,
+						 snapshot,
+						 sessiond_session_id,
+						 sessiond_uuid,
+						 current_chunk_id,
+						 creation_time,
+						 session_name_contains_creation_time,
+						 trace_format,
+						 &reply,
+						 output_path);
 	}
 
 	if (ret < 0) {
@@ -407,7 +567,7 @@ static int relayd_add_stream_2_1(struct lttcomm_relayd_sock *rsock,
 	}
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_ADD_STREAM, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_ADD_STREAM, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		ret = -1;
 		goto error;
@@ -440,7 +600,7 @@ static int relayd_add_stream_2_2(struct lttcomm_relayd_sock *rsock,
 	msg.tracefile_count = htobe64(tracefile_count);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_ADD_STREAM, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_ADD_STREAM, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
@@ -494,7 +654,7 @@ static int relayd_add_stream_2_11(struct lttcomm_relayd_sock *rsock,
 	msg->trace_chunk_id = htobe64(trace_archive_id);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_ADD_STREAM, (void *) msg, msg_length, 0);
+	ret = send_command(*rsock, RELAYD_ADD_STREAM, (void *) msg, msg_length, 0);
 	if (ret < 0) {
 		goto error;
 	}
@@ -582,7 +742,7 @@ int relayd_add_stream(struct lttcomm_relayd_sock *rsock,
 	}
 
 	/* Waiting for reply */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -629,13 +789,13 @@ int relayd_streams_sent(struct lttcomm_relayd_sock *rsock)
 	}
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_STREAMS_SENT, nullptr, 0, 0);
+	ret = send_command(*rsock, RELAYD_STREAMS_SENT, nullptr, 0, 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Waiting for reply */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -684,13 +844,13 @@ int relayd_version_check(struct lttcomm_relayd_sock *rsock)
 	msg.minor = htobe32(rsock->minor);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_VERSION, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_VERSION, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &msg, sizeof(msg));
+	ret = recv_reply(*rsock, (void *) &msg, sizeof(msg));
 	if (ret < 0) {
 		goto error;
 	}
@@ -748,7 +908,7 @@ int relayd_send_metadata(struct lttcomm_relayd_sock *rsock, size_t len)
 	DBG("Relayd sending metadata of size %zu", len);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_SEND_METADATA, nullptr, len, 0);
+	ret = send_command(*rsock, RELAYD_SEND_METADATA, nullptr, len, 0);
 	if (ret < 0) {
 		goto error;
 	}
@@ -889,13 +1049,13 @@ int relayd_send_close_stream(struct lttcomm_relayd_sock *rsock,
 	msg.last_net_seq_num = htobe64(last_net_seq_num);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_CLOSE_STREAM, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_CLOSE_STREAM, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -940,13 +1100,13 @@ int relayd_data_pending(struct lttcomm_relayd_sock *rsock,
 	msg.last_net_seq_num = htobe64(last_net_seq_num);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_DATA_PENDING, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_DATA_PENDING, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -985,13 +1145,13 @@ int relayd_quiescent_control(struct lttcomm_relayd_sock *rsock, uint64_t metadat
 	msg.stream_id = htobe64(metadata_stream_id);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_QUIESCENT_CONTROL, &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_QUIESCENT_CONTROL, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -1030,13 +1190,13 @@ int relayd_begin_data_pending(struct lttcomm_relayd_sock *rsock, uint64_t id)
 	msg.session_id = htobe64(id);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_BEGIN_DATA_PENDING, &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_BEGIN_DATA_PENDING, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -1079,13 +1239,13 @@ int relayd_end_data_pending(struct lttcomm_relayd_sock *rsock,
 	msg.session_id = htobe64(id);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_END_DATA_PENDING, &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_END_DATA_PENDING, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -1109,8 +1269,8 @@ error:
 /*
  * Send index to the relayd.
  */
-int relayd_send_index(struct lttcomm_relayd_sock *rsock,
-		      struct ctf_packet_index *index,
+int relayd_send_index(lttcomm_relayd_sock& rsock,
+		      const ctf_packet_index& index,
 		      uint64_t relay_stream_id,
 		      uint64_t net_seq_num)
 {
@@ -1118,10 +1278,7 @@ int relayd_send_index(struct lttcomm_relayd_sock *rsock,
 	struct lttcomm_relayd_index msg;
 	struct lttcomm_relayd_generic_reply reply;
 
-	/* Code flow error. Safety net. */
-	LTTNG_ASSERT(rsock);
-
-	if (rsock->minor < 4) {
+	if (rsock.minor < 4) {
 		DBG("Not sending indexes before protocol 2.4");
 		ret = 0;
 		goto error;
@@ -1134,26 +1291,25 @@ int relayd_send_index(struct lttcomm_relayd_sock *rsock,
 	msg.net_seq_num = htobe64(net_seq_num);
 
 	/* The index is already in big endian. */
-	msg.packet_size = index->packet_size;
-	msg.content_size = index->content_size;
-	msg.timestamp_begin = index->timestamp_begin;
-	msg.timestamp_end = index->timestamp_end;
-	msg.events_discarded = index->events_discarded;
-	msg.stream_id = index->stream_id;
+	msg.packet_size = index.packet_size;
+	msg.content_size = index.content_size;
+	msg.timestamp_begin = index.timestamp_begin;
+	msg.timestamp_end = index.timestamp_end;
+	msg.events_discarded = index.events_discarded;
+	msg.stream_id = index.stream_id;
 
-	if (rsock->minor >= 8) {
-		msg.stream_instance_id = index->stream_instance_id;
-		msg.packet_seq_num = index->packet_seq_num;
+	if (rsock.minor >= 8) {
+		msg.stream_instance_id = index.stream_instance_id;
+		msg.packet_seq_num = index.packet_seq_num;
 	}
 
 	/* Send command */
-	ret = send_command(
-		rsock,
-		RELAYD_SEND_INDEX,
-		&msg,
-		lttcomm_relayd_index_len(lttng_to_index_major(rsock->major, rsock->minor),
-					 lttng_to_index_minor(rsock->major, rsock->minor)),
-		0);
+	ret = send_command(rsock,
+			   RELAYD_SEND_INDEX,
+			   &msg,
+			   lttcomm_relayd_index_len(lttng_to_index_major(rsock.major, rsock.minor),
+						    lttng_to_index_minor(rsock.major, rsock.minor)),
+			   0);
 	if (ret < 0) {
 		goto error;
 	}
@@ -1205,13 +1361,13 @@ int relayd_reset_metadata(struct lttcomm_relayd_sock *rsock, uint64_t stream_id,
 	msg.version = htobe64(version);
 
 	/* Send command */
-	ret = send_command(rsock, RELAYD_RESET_METADATA, (void *) &msg, sizeof(msg), 0);
+	ret = send_command(*rsock, RELAYD_RESET_METADATA, (void *) &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		goto error;
 	}
 
 	/* Receive response */
-	ret = recv_reply(rsock, (void *) &reply, sizeof(reply));
+	ret = recv_reply(*rsock, (void *) &reply, sizeof(reply));
 	if (ret < 0) {
 		goto error;
 	}
@@ -1302,14 +1458,14 @@ int relayd_rotate_streams(struct lttcomm_relayd_sock *sock,
 	}
 
 	/* Send command. */
-	ret = send_command(sock, RELAYD_ROTATE_STREAMS, payload.data, payload.size, 0);
+	ret = send_command(*sock, RELAYD_ROTATE_STREAMS, payload.data, payload.size, 0);
 	if (ret < 0) {
 		ERR("Failed to send \"rotate stream\" command");
 		goto error;
 	}
 
 	/* Receive response. */
-	ret = recv_reply(sock, &reply, sizeof(reply));
+	ret = recv_reply(*sock, &reply, sizeof(reply));
 	if (ret < 0) {
 		ERR("Failed to receive \"rotate streams\" command reply");
 		goto error;
@@ -1384,13 +1540,13 @@ int relayd_create_trace_chunk(struct lttcomm_relayd_sock *sock, struct lttng_tra
 		}
 	}
 
-	ret = send_command(sock, RELAYD_CREATE_TRACE_CHUNK, payload.data, payload.size, 0);
+	ret = send_command(*sock, RELAYD_CREATE_TRACE_CHUNK, payload.data, payload.size, 0);
 	if (ret < 0) {
 		ERR("Failed to send trace chunk creation command to relay daemon");
 		goto end;
 	}
 
-	ret = recv_reply(sock, &reply, sizeof(reply));
+	ret = recv_reply(*sock, &reply, sizeof(reply));
 	if (ret < 0) {
 		ERR("Failed to receive relay daemon trace chunk creation command reply");
 		goto end;
@@ -1463,13 +1619,13 @@ int relayd_close_trace_chunk(struct lttcomm_relayd_sock *sock,
 		},
 	};
 
-	ret = send_command(sock, RELAYD_CLOSE_TRACE_CHUNK, &msg, sizeof(msg), 0);
+	ret = send_command(*sock, RELAYD_CLOSE_TRACE_CHUNK, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		ERR("Failed to send trace chunk close command to relay daemon");
 		goto end;
 	}
 
-	ret = recv_reply(sock, &reply, sizeof(reply));
+	ret = recv_reply(*sock, &reply, sizeof(reply));
 	if (ret < 0) {
 		ERR("Failed to receive relay daemon trace chunk close command reply");
 		goto end;
@@ -1482,7 +1638,7 @@ int relayd_close_trace_chunk(struct lttcomm_relayd_sock *sock,
 		goto end;
 	}
 
-	ret = recv_reply(sock, path, reply.path_length);
+	ret = recv_reply(*sock, path, reply.path_length);
 	if (ret < 0) {
 		ERR("Failed to receive relay daemon trace chunk close command reply");
 		goto end;
@@ -1524,13 +1680,13 @@ int relayd_trace_chunk_exists(struct lttcomm_relayd_sock *sock,
 		.chunk_id = htobe64(chunk_id),
 	};
 
-	ret = send_command(sock, RELAYD_TRACE_CHUNK_EXISTS, &msg, sizeof(msg), 0);
+	ret = send_command(*sock, RELAYD_TRACE_CHUNK_EXISTS, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		ERR("Failed to send trace chunk exists command to relay daemon");
 		goto end;
 	}
 
-	ret = recv_reply(sock, &reply, sizeof(reply));
+	ret = recv_reply(*sock, &reply, sizeof(reply));
 	if (ret < 0) {
 		ERR("Failed to receive relay daemon trace chunk close command reply");
 		goto end;
@@ -1572,13 +1728,13 @@ int relayd_get_configuration(struct lttcomm_relayd_sock *sock,
 		goto end;
 	}
 
-	ret = send_command(sock, RELAYD_GET_CONFIGURATION, &msg, sizeof(msg), 0);
+	ret = send_command(*sock, RELAYD_GET_CONFIGURATION, &msg, sizeof(msg), 0);
 	if (ret < 0) {
 		ERR("Failed to send get configuration command to relay daemon");
 		goto end;
 	}
 
-	ret = recv_reply(sock, &reply, sizeof(reply));
+	ret = recv_reply(*sock, &reply, sizeof(reply));
 	if (ret < 0) {
 		ERR("Failed to receive relay daemon get configuration command reply");
 		goto end;
