@@ -167,7 +167,7 @@ ssize_t lttng_trigger_create_from_payload(struct lttng_payload_view *src_view,
 		goto end;
 	}
 
-	LTTNG_OPTIONAL_SET(&creds.uid, trigger_comm->uid);
+	LTTNG_OPTIONAL_SET(&creds.uid, static_cast<uid_t>(trigger_comm->uid));
 
 	offset += sizeof(*trigger_comm);
 
@@ -714,7 +714,8 @@ void lttng_trigger_set_credentials(struct lttng_trigger *trigger,
 {
 	/* Triggers do not use the group id to authenticate the user. */
 	LTTNG_ASSERT(creds);
-	LTTNG_OPTIONAL_SET(&trigger->creds.uid, LTTNG_OPTIONAL_GET(creds->uid));
+	const auto uid_value = LTTNG_OPTIONAL_GET(creds->uid);
+	LTTNG_OPTIONAL_SET(&trigger->creds.uid, uid_value);
 	LTTNG_OPTIONAL_UNSET(&trigger->creds.gid);
 }
 
