@@ -21,6 +21,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <vector>
 
 /*
  * Connect to unix socket using the path name.
@@ -35,7 +36,8 @@ int lttcomm_connect_unix_sock(const char *pathname)
 		    pathname,
 		    strlen(pathname) + 1,
 		    sizeof(s_un.sun_path));
-		ret = -ENAMETOOLONG;
+		errno = ENAMETOOLONG;
+		ret = -errno;
 		goto error;
 	}
 
@@ -268,12 +270,14 @@ end:
  *
  * Return the size of sent data.
  */
-ssize_t lttcomm_send_unix_sock(int sock, const void *buf, size_t len)
+ssize_t lttcomm_send_unix_sock(int sock, const void *buf, size_t len, bool quiet_on_closed)
 {
 	struct msghdr msg;
 	struct iovec iov[1];
 	ssize_t ret;
 
+	quiet_on_closed |= lttng_opt_quiet;
+
 	LTTNG_ASSERT(sock);
 	LTTNG_ASSERT(buf);
 	LTTNG_ASSERT(len > 0);
@@ -296,7 +300,7 @@ ssize_t lttcomm_send_unix_sock(int sock, const void *buf, size_t len)
 				 * deactivated.
 				 * We consider EPIPE as expected.
 				 */
-				if (errno != EPIPE || !lttng_opt_quiet) {
+				if (errno != EPIPE || !quiet_on_closed) {
 					PERROR("sendmsg");
 				}
 				goto end;
@@ -411,7 +415,6 @@ ssize_t lttcomm_send_fds_unix_sock(int sock, const int *fds, size_t nb_fd)
 	struct iovec iov[1];
 	ssize_t ret = -1;
 	const unsigned int sizeof_fds = nb_fd * sizeof(int);
-	char tmp[CMSG_SPACE(sizeof_fds)];
 	char dummy = 0;
 
 	LTTNG_ASSERT(sock);
@@ -419,12 +422,18 @@ ssize_t lttcomm_send_fds_unix_sock(int sock, const int *fds, size_t nb_fd)
 	LTTNG_ASSERT(nb_fd > 0);
 
 	memset(&msg, 0, sizeof(msg));
-	memset(tmp, 0, sizeof(tmp));
 
 	if (nb_fd > LTTCOMM_MAX_SEND_FDS)
 		return -EINVAL;
 
-	msg.msg_control = (caddr_t) tmp;
+	std::vector<char> ctrl_buf;
+	try {
+		ctrl_buf.resize(CMSG_SPACE(sizeof_fds));
+	} catch (const std::bad_alloc&) {
+		return -ENOMEM;
+	}
+
+	msg.msg_control = (caddr_t) ctrl_buf.data();
 	msg.msg_controllen = CMSG_LEN(sizeof_fds);
 
 	cmptr = CMSG_FIRSTHDR(&msg);
@@ -535,7 +544,6 @@ ssize_t lttcomm_send_fds_unix_sock_non_block(int sock, const int *fds, size_t nb
 	struct iovec iov[1];
 	ssize_t ret = -1;
 	const unsigned int sizeof_fds = nb_fd * sizeof(int);
-	char tmp[CMSG_SPACE(sizeof_fds)];
 	char dummy = 0;
 
 	LTTNG_ASSERT(sock);
@@ -543,12 +551,18 @@ ssize_t lttcomm_send_fds_unix_sock_non_block(int sock, const int *fds, size_t nb
 	LTTNG_ASSERT(nb_fd > 0);
 
 	memset(&msg, 0, sizeof(msg));
-	memset(tmp, 0, sizeof(tmp));
 
 	if (nb_fd > LTTCOMM_MAX_SEND_FDS)
 		return -EINVAL;
 
-	msg.msg_control = (caddr_t) tmp;
+	std::vector<char> ctrl_buf;
+	try {
+		ctrl_buf.resize(CMSG_SPACE(sizeof_fds));
+	} catch (const std::bad_alloc&) {
+		return -ENOMEM;
+	}
+
+	msg.msg_control = (caddr_t) ctrl_buf.data();
 	msg.msg_controllen = CMSG_LEN(sizeof_fds);
 
 	cmptr = CMSG_FIRSTHDR(&msg);
@@ -623,13 +637,19 @@ ssize_t lttcomm_recv_fds_unix_sock(int sock, int *fds, size_t nb_fd)
 	const size_t sizeof_fds = nb_fd * sizeof(int);
 
 #ifdef __linux__
-/* Account for the struct ucred cmsg in the buffer size */
-#define LTTNG_SOCK_RECV_FDS_BUF_SIZE CMSG_SPACE(sizeof_fds) + CMSG_SPACE(sizeof(struct ucred))
+	/* Account for the struct ucred cmsg in the buffer size */
+	const size_t recv_buf_size = CMSG_SPACE(sizeof_fds) + CMSG_SPACE(sizeof(struct ucred));
 #else
-#define LTTNG_SOCK_RECV_FDS_BUF_SIZE CMSG_SPACE(sizeof_fds)
+	const size_t recv_buf_size = CMSG_SPACE(sizeof_fds);
 #endif /* __linux__ */
 
-	char recv_buf[LTTNG_SOCK_RECV_FDS_BUF_SIZE];
+	std::vector<char> recv_buf;
+	try {
+		recv_buf.resize(recv_buf_size);
+	} catch (const std::bad_alloc&) {
+		return -ENOMEM;
+	}
+
 	struct msghdr msg;
 	char dummy;
 
@@ -645,13 +665,13 @@ ssize_t lttcomm_recv_fds_unix_sock(int sock, int *fds, size_t nb_fd)
 	msg.msg_iov = iov;
 	msg.msg_iovlen = 1;
 
-	cmsg = (struct cmsghdr *) recv_buf;
+	cmsg = (struct cmsghdr *) recv_buf.data();
 	cmsg->cmsg_len = CMSG_LEN(sizeof_fds);
 	cmsg->cmsg_level = SOL_SOCKET;
 	cmsg->cmsg_type = SCM_RIGHTS;
 
 	msg.msg_control = cmsg;
-	msg.msg_controllen = CMSG_LEN(sizeof(recv_buf));
+	msg.msg_controllen = CMSG_LEN(recv_buf_size);
 	msg.msg_flags = 0;
 
 retry:
@@ -853,13 +873,19 @@ ssize_t lttcomm_recv_fds_unix_sock_non_block(int sock, int *fds, size_t nb_fd)
 	LTTNG_ASSERT(nb_fd > 0);
 
 #ifdef __linux__
-/* Account for the struct ucred cmsg in the buffer size */
-#define LTTNG_SOCK_RECV_FDS_BUF_SIZE CMSG_SPACE(sizeof_fds) + CMSG_SPACE(sizeof(struct ucred))
+	/* Account for the struct ucred cmsg in the buffer size */
+	const size_t recv_buf_size = CMSG_SPACE(sizeof_fds) + CMSG_SPACE(sizeof(struct ucred));
 #else
-#define LTTNG_SOCK_RECV_FDS_BUF_SIZE CMSG_SPACE(sizeof_fds)
+	const size_t recv_buf_size = CMSG_SPACE(sizeof_fds);
 #endif /* __linux__ */
 
-	char recv_buf[LTTNG_SOCK_RECV_FDS_BUF_SIZE];
+	std::vector<char> recv_buf;
+	try {
+		recv_buf.resize(recv_buf_size);
+	} catch (const std::bad_alloc&) {
+		return -ENOMEM;
+	}
+
 	struct msghdr msg;
 	char dummy;
 
@@ -871,13 +897,14 @@ ssize_t lttcomm_recv_fds_unix_sock_non_block(int sock, int *fds, size_t nb_fd)
 	msg.msg_iov = iov;
 	msg.msg_iovlen = 1;
 
-	cmsg = (struct cmsghdr *) recv_buf;
+	cmsg = (struct cmsghdr *) recv_buf.data();
+	LTTNG_ASSERT(cmsg);
 	cmsg->cmsg_len = CMSG_LEN(sizeof_fds);
 	cmsg->cmsg_level = SOL_SOCKET;
 	cmsg->cmsg_type = SCM_RIGHTS;
 
 	msg.msg_control = cmsg;
-	msg.msg_controllen = CMSG_LEN(sizeof(recv_buf));
+	msg.msg_controllen = CMSG_LEN(recv_buf_size);
 	msg.msg_flags = 0;
 
 retry:
