@@ -30,6 +30,7 @@
 #include <lttng/constant.h>
 
 #include <algorithm>
+#include <cstdlib>
 #include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
@@ -1451,15 +1452,12 @@ unsigned int get_max_possible_cpu_id() LTTNG_MAY_THROW
 
 	const char *cpu_id_parse_start = &possible_cpu_mask.data()[i];
 	char *end_ptr;
-	unsigned long cpu_id =
-		strtoul(cpu_id_parse_start,
-			&end_ptr,
-			10);
+	unsigned long cpu_id = strtoul(cpu_id_parse_start, &end_ptr, 10);
 	if (cpu_id == 0 && end_ptr == cpu_id_parse_start) {
 		/* No digits found. */
-		LTTNG_THROW_ERROR(lttng::format(
-			"Failed to parse possible CPU mask: mask_contents=`{}`",
-			lttng::c_string_view(possible_cpu_mask.data())));
+		LTTNG_THROW_ERROR(
+			lttng::format("Failed to parse possible CPU mask: mask_contents=`{}`",
+				      lttng::c_string_view(possible_cpu_mask.data())));
 	}
 
 	if ((i != possible_cpu_mask_len) && (cpu_id < UINT_MAX)) {
@@ -1533,16 +1531,6 @@ end:
 	return ret_code;
 }
 
-bool utils_force_experimental_ctf_2()
-{
-	if (const auto val =
-		    lttng::c_string_view(lttng_secure_getenv("LTTNG_EXPERIMENTAL_FORCE_CTF_2"))) {
-		return val == "1";
-	}
-
-	return false;
-}
-
 static void lttng_closedir(DIR *d)
 {
 	if (closedir(d)) {
@@ -1617,3 +1605,86 @@ std::vector<int> list_open_fds()
 
 	return results;
 }
+
+std::pair<double, const char *> utils_value_unit_from_size(std::uint64_t bytes)
+{
+	static constexpr const char *suffixes[] = { "B", "KiB", "MiB", "GiB", "TiB", "PiB" };
+	size_t i = 0;
+	double count = static_cast<double>(bytes);
+	while (count >= 1024 && i < sizeof(suffixes) / sizeof(suffixes[0]) - 1) {
+		count /= 1024;
+		++i;
+	}
+	return { count, suffixes[i] };
+}
+
+std::string utils_string_from_size(std::uint64_t bytes)
+{
+	const auto value_unit = utils_value_unit_from_size(bytes);
+
+	return fmt::format("{:.2f} {}", value_unit.first, value_unit.second);
+}
+
+std::pair<double, const char *> utils_value_unit_from_period(const std::uint64_t period_us)
+{
+	struct unit_info final {
+		std::uint64_t divisor;
+		const char *suffix;
+	};
+
+	static const unit_info units[] = {
+		{ USEC_PER_HOURS, HR_UNIT },
+		{ USEC_PER_MINUTE, MIN_UNIT },
+		{ USEC_PER_SEC, SEC_UNIT },
+		{ USEC_PER_MSEC, MSEC_UNIT },
+		{ 1, USEC_UNIT },
+	};
+
+	for (const auto& unit : units) {
+		if (period_us >= unit.divisor) {
+			return { static_cast<double>(period_us) / unit.divisor, unit.suffix };
+		}
+	}
+
+	return { static_cast<double>(period_us), USEC_UNIT };
+}
+
+std::string utils_string_from_period(const std::uint64_t period_us)
+{
+	if (period_us < 1000) {
+		return fmt::format("{} {}", period_us, USEC_UNIT);
+	}
+
+	const auto value_unit = utils_value_unit_from_period(period_us);
+
+	return fmt::format("{:.2f} {}", value_unit.first, value_unit.second);
+}
+
+std::string utils_format_integer_grouped(const std::int64_t value)
+{
+	/* Don't group if value is small enough */
+	if (value >= -9999 && value <= 9999) {
+		return fmt::format("{}", value);
+	}
+
+	/* Convert to string and reserve space */
+	const auto digits = fmt::format("{}", static_cast<std::uint64_t>(std::abs(value)));
+	std::string result;
+
+	result.reserve(digits.size() * 2);
+
+	if (value < 0) {
+		/* Negative: start with `-` */
+		result += '-';
+	}
+
+	for (size_t i = 0; i < digits.size(); ++i) {
+		if (i > 0 && (digits.size() - i) % 3 == 0) {
+			result += ',';
+		}
+
+		result += digits[i];
+	}
+
+	return result;
+}
