@@ -33,6 +33,7 @@
 #include <common/payload.hpp>
 #include <common/scope-exit.hpp>
 #include <common/sessiond-comm/sessiond-comm.hpp>
+#include <common/stream-info.hpp>
 #include <common/tracker.hpp>
 #include <common/unix.hpp>
 #include <common/uri.hpp>
@@ -48,6 +49,7 @@
 #include <lttng/lttng.h>
 #include <lttng/session-descriptor-internal.hpp>
 #include <lttng/session-internal.hpp>
+#include <lttng/stream-info.h>
 #include <lttng/trigger/trigger-internal.hpp>
 #include <lttng/userspace-probe-internal.hpp>
 
@@ -90,7 +92,7 @@ LTTNG_EXPORT int lttng_opt_mi;
  *
  * If domain is unknown, default domain will be the kernel.
  */
-void lttng_ctl_copy_lttng_domain(struct lttng_domain *dst, struct lttng_domain *src)
+void lttng_ctl_copy_lttng_domain(struct lttng_domain *dst, const struct lttng_domain *src)
 {
 	if (src && dst) {
 		switch (src->type) {
@@ -680,7 +682,8 @@ end:
  *
  * The returned pointer will be NULL in case of malloc() error.
  */
-struct lttng_handle *lttng_create_handle(const char *session_name, struct lttng_domain *domain)
+struct lttng_handle *lttng_create_handle(const char *session_name,
+					 const struct lttng_domain *domain)
 {
 	int ret;
 	struct lttng_handle *handle = nullptr;
@@ -1173,7 +1176,7 @@ int lttng_enable_event_with_exclusions(struct lttng_handle *handle,
 		}
 	}
 
-	lttng::event_rule_uptr event_rule;
+	lttng::ctl::event_rule_uptr event_rule;
 	try {
 		event_rule = lttng::ctl::create_event_rule_from_lttng_event(
 			*ev,
@@ -1347,7 +1350,7 @@ int lttng_disable_event_ext(struct lttng_handle *handle,
 	}
 
 	const std::vector<lttng::c_string_view> exclusions_vec;
-	lttng::event_rule_uptr event_rule;
+	lttng::ctl::event_rule_uptr event_rule;
 	if (ev->type != LTTNG_EVENT_ALL) {
 		try {
 			event_rule = lttng::ctl::create_event_rule_from_lttng_event(
@@ -1665,7 +1668,7 @@ int lttng_list_tracepoints(struct lttng_handle *handle, struct lttng_event **eve
 	size_t cmd_header_len;
 	unsigned int nb_events = 0;
 
-	if (handle == nullptr) {
+	if (handle == nullptr || events == nullptr) {
 		ret = -LTTNG_ERR_INVALID;
 		goto end;
 	}
@@ -1737,7 +1740,7 @@ int lttng_list_tracepoint_fields(struct lttng_handle *handle, struct lttng_event
 
 	lttng_payload_init(&reply);
 
-	if (handle == nullptr) {
+	if (handle == nullptr || fields == nullptr) {
 		ret = -LTTNG_ERR_INVALID;
 		goto end;
 	}
@@ -2264,6 +2267,20 @@ lttng_get_session_shm_path_override(const lttng_session *session, const char **s
 					LTTNG_GET_SESSION_SHM_PATH_STATUS_OK;
 }
 
+enum lttng_get_session_trace_format_status
+lttng_get_session_trace_format(const struct lttng_session *session, enum lttng_trace_format *format)
+{
+	struct lttng_session_extended *extended;
+
+	if (!session || !format || !session->extended.ptr) {
+		return LTTNG_GET_SESSION_TRACE_FORMAT_STATUS_INVALID;
+	}
+
+	extended = (lttng_session_extended *) session->extended.ptr;
+	*format = (enum lttng_trace_format) extended->trace_format;
+	return LTTNG_GET_SESSION_TRACE_FORMAT_STATUS_OK;
+}
+
 int lttng_set_session_shm_path(const char *session_name, const char *shm_path)
 {
 	int ret;
@@ -2335,7 +2352,7 @@ error:
  * Returns the number of lttng_channel entries in channels;
  * on error, returns a negative value.
  */
-int lttng_list_channels(struct lttng_handle *handle, struct lttng_channel **channels)
+int lttng_list_channels(const struct lttng_handle *handle, struct lttng_channel **channels)
 {
 	int ret, total_payload_received;
 	struct lttcomm_session_msg lsm;
@@ -2416,7 +2433,7 @@ end:
  * Returns the number of lttng_event entries in events;
  * on error, returns a negative value.
  */
-int lttng_list_events(struct lttng_handle *handle,
+int lttng_list_events(const struct lttng_handle *handle,
 		      const char *channel_name,
 		      struct lttng_event **events)
 {
@@ -2595,7 +2612,7 @@ void lttng_channel_set_default_attr(struct lttng_domain *domain, struct lttng_ch
 	attr->extended.ptr = extended;
 }
 
-int lttng_channel_get_discarded_event_count(struct lttng_channel *channel,
+int lttng_channel_get_discarded_event_count(const struct lttng_channel *channel,
 					    uint64_t *discarded_events)
 {
 	int ret = 0;
@@ -2621,7 +2638,7 @@ end:
 	return ret;
 }
 
-int lttng_channel_get_lost_packet_count(struct lttng_channel *channel, uint64_t *lost_packets)
+int lttng_channel_get_lost_packet_count(const struct lttng_channel *channel, uint64_t *lost_packets)
 {
 	int ret = 0;
 	struct lttng_channel_extended *chan_ext;
@@ -2683,6 +2700,45 @@ end:
 	return ret;
 }
 
+enum lttng_channel_status
+lttng_channel_get_watchdog_timer_interval(const struct lttng_channel *chan,
+					  uint64_t *watchdog_timer_interval)
+{
+	if (!chan || !watchdog_timer_interval) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	if (!chan->attr.extended.ptr) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	const auto extended =
+		reinterpret_cast<const struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	if (!extended->watchdog_timer_interval.is_set) {
+		return LTTNG_CHANNEL_STATUS_UNSET;
+	}
+
+	*watchdog_timer_interval = LTTNG_OPTIONAL_GET(extended->watchdog_timer_interval);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
+enum lttng_channel_status
+lttng_channel_set_watchdog_timer_interval(struct lttng_channel *chan,
+					  uint64_t watchdog_timer_interval)
+{
+	if (!chan || !chan->attr.extended.ptr) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	auto extended = reinterpret_cast<struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	LTTNG_OPTIONAL_SET(&extended->watchdog_timer_interval, watchdog_timer_interval);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
 int lttng_channel_get_blocking_timeout(const struct lttng_channel *chan, int64_t *blocking_timeout)
 {
 	int ret = 0;
@@ -2783,6 +2839,98 @@ lttng_channel_set_allocation_policy(struct lttng_channel *chan,
 	return LTTNG_OK;
 }
 
+enum lttng_channel_status
+lttng_channel_get_preallocation_policy(const struct lttng_channel *chan,
+				       enum lttng_channel_preallocation_policy *policy)
+{
+	if (!chan || !policy) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	const auto extended =
+		static_cast<const struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	if (!extended) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	*policy = static_cast<enum lttng_channel_preallocation_policy>(
+		extended->preallocation_policy);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
+enum lttng_channel_status
+lttng_channel_set_preallocation_policy(struct lttng_channel *chan,
+				       enum lttng_channel_preallocation_policy policy)
+{
+	if (!chan) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	const auto extended = static_cast<struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	if (!extended) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	switch (policy) {
+	case LTTNG_CHANNEL_PREALLOCATION_POLICY_PREALLOCATE:
+		/* Fallthrough */
+	case LTTNG_CHANNEL_PREALLOCATION_POLICY_ON_DEMAND:
+		break;
+	default:
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	extended->preallocation_policy = static_cast<uint8_t>(policy);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
+enum lttng_channel_status
+lttng_channel_get_automatic_memory_reclamation_policy(const struct lttng_channel *chan,
+						      uint64_t *maximal_age_us)
+{
+	if (!chan || !maximal_age_us) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	const auto extended =
+		static_cast<const struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	if (!extended) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	if (!extended->automatic_memory_reclamation_maximal_age_us.is_set) {
+		return LTTNG_CHANNEL_STATUS_UNSET;
+	}
+
+	*maximal_age_us = LTTNG_OPTIONAL_GET(extended->automatic_memory_reclamation_maximal_age_us);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
+enum lttng_channel_status
+lttng_channel_set_automatic_memory_reclamation_policy(struct lttng_channel *chan,
+						      uint64_t maximal_age_us)
+{
+	if (!chan) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	const auto extended = static_cast<struct lttng_channel_extended *>(chan->attr.extended.ptr);
+
+	if (!extended) {
+		return LTTNG_CHANNEL_STATUS_INVALID;
+	}
+
+	LTTNG_OPTIONAL_SET(&extended->automatic_memory_reclamation_maximal_age_us, maximal_age_us);
+
+	return LTTNG_CHANNEL_STATUS_OK;
+}
+
 /*
  * Check if session daemon is alive.
  *
@@ -3434,6 +3582,85 @@ end:
 	return ret_code;
 }
 
+enum lttng_channel_get_data_stream_info_sets_status
+lttng_channel_get_data_stream_info_sets(const char *session_name,
+					const char *channel_name,
+					enum lttng_domain_type domain,
+					const struct lttng_data_stream_info_sets **sets)
+{
+	int ret;
+	enum lttng_channel_get_data_stream_info_sets_status ret_status =
+		LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_OK;
+	struct lttcomm_session_msg lsm = {};
+	struct lttng_payload reply;
+	struct lttng_payload_view lsm_view =
+		lttng_payload_view_init_from_buffer((const char *) &lsm, 0, sizeof(lsm));
+
+	lttng_payload_init(&reply);
+	const auto reset_payload =
+		lttng::make_scope_exit([&reply]() noexcept { lttng_payload_reset(&reply); });
+
+	if (!session_name || !channel_name || !sets) {
+		ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_INVALID_PARAMETER;
+		goto end;
+	}
+
+	if (domain != LTTNG_DOMAIN_UST) {
+		ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_UNSUPPORTED_DOMAIN;
+		goto end;
+	}
+
+	/* Setup the message */
+	lsm.cmd_type = LTTCOMM_SESSIOND_COMMAND_GET_CHANNEL_DATA_STREAM_INFO_SETS;
+
+	/* Set session name */
+	ret = lttng_strncpy(lsm.session.name, session_name, sizeof(lsm.session.name));
+	if (ret) {
+		ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_INVALID_PARAMETER;
+		goto end;
+	}
+
+	/* Set domain */
+	lsm.domain.type = domain;
+
+	/* Set channel name */
+	ret = lttng_strncpy(lsm.u.get_channel_data_stream_info_sets.channel_name,
+			    channel_name,
+			    sizeof(lsm.u.get_channel_data_stream_info_sets.channel_name));
+	if (ret) {
+		ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_INVALID_PARAMETER;
+		goto end;
+	}
+
+	lsm.fd_count = 0;
+
+	ret = lttng_ctl_ask_sessiond_payload(&lsm_view, &reply);
+	if (ret < 0) {
+		ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_ERROR;
+		goto end;
+	}
+
+	{
+		lttng_payload_view reply_view =
+			lttng_payload_view_from_payload(&reply, 0, reply.buffer.size);
+
+		try {
+			auto sets_create_result =
+				lttng_data_stream_info_sets::create_from_payload(reply_view);
+			*sets = sets_create_result.second.release();
+		} catch (const std::exception& e) {
+			/* Catch any exception thrown during creation. */
+			ERR("Failed to create data stream info sets: %s", e.what());
+			ret_status = LTTNG_CHANNEL_GET_DATA_STREAM_INFO_SETS_STATUS_ERROR;
+			goto end;
+		}
+	}
+
+end:
+	lttng_payload_reset(&reply);
+	return ret_status;
+}
+
 /*
  * lib constructor.
  */
