@@ -8,7 +8,7 @@ UST_EVENT_NAME="tp:tptest"
 KERNEL_EVENT_NAME="sched_switch"
 CHANNEL_NAME="testchan"
 HEALTH_CHECK_BIN="health_check"
-NUM_TESTS=90
+NUM_TESTS=82
 SLEEP_TIME=30
 
 source $TESTDIR/utils/utils.sh
@@ -97,7 +97,7 @@ function test_health
 		# When starting with the error test points the "start" can fail
 		# or hang waiting for the PID file which got cleaned up quickly.
 		# Use the background mode with no tap to avoid the hang and failure.
-		start_lttng_relayd_opt 0 "-b" "-o $TRACE_PATH"
+		start_lttng_relayd_opt 0 "-b" -o "$TRACE_PATH"
 	else
 		RELAYD_ARGS=
 	fi
@@ -164,7 +164,6 @@ THREAD=(
 
 	"LTTNG_CONSUMERD_THREAD_CHANNEL"
 	"LTTNG_CONSUMERD_THREAD_METADATA"
-	"LTTNG_CONSUMERD_THREAD_METADATA_TIMER"
 
 	"LTTNG_RELAYD_THREAD_DISPATCHER"
 	"LTTNG_RELAYD_THREAD_WORKER"
@@ -184,7 +183,6 @@ ERROR_STRING=(
 
 	"Thread \"Consumer daemon channel\" is not responding"
 	"Thread \"Consumer daemon metadata\" is not responding"
-	"Thread \"Consumer daemon metadata timer\" is not responding"
 
 	"Thread \"Relay daemon dispatcher\" is not responding in component \"relayd\"."
 	"Thread \"Relay daemon worker\" is not responding in component \"relayd\"."
@@ -218,7 +216,6 @@ NEEDS_ROOT=(
 	0
 	1
 
-	0
 	0
 	0
 
@@ -238,7 +235,6 @@ TEST_CONSUMERD=(
 	0
 	0
 
-	1
 	1
 	1
 
@@ -258,7 +254,6 @@ TEST_RELAYD=(
 	0
 	0
 
-	0
 	0
 	0
 
