@@ -16,11 +16,20 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(my_provider,
-		 my_first_tracepoint,
-		 TP_ARGS(int, my_integer_arg, char *, my_string_arg),
-		 TP_FIELDS(ctf_string(my_string_field, my_string_arg)
-				   ctf_integer(int, my_integer_field, my_integer_arg)))
+/* clang-format off */
+
+TRACEPOINT_EVENT(my_provider, my_first_tracepoint,
+	TP_ARGS(
+		int, my_integer_arg,
+		char *, my_string_arg
+	),
+	TP_FIELDS(
+		ctf_string(my_string_field, my_string_arg)
+		ctf_integer(int, my_integer_field, my_integer_arg)
+	)
+)
+
+/* clang-format on */
 
 #endif /* _TP_H */
 
