@@ -15,17 +15,32 @@
 
 #include <sys/types.h>
 
-TRACEPOINT_EVENT(ust_tests_daemon,
-		 before_daemon,
-		 TP_ARGS(pid_t, pid),
-		 TP_FIELDS(ctf_integer(pid_t, pid, pid)))
+/* clang-format off */
 
-TRACEPOINT_EVENT(ust_tests_daemon,
-		 after_daemon_child,
-		 TP_ARGS(pid_t, pid),
-		 TP_FIELDS(ctf_integer(pid_t, pid, pid)))
+TRACEPOINT_EVENT(ust_tests_daemon, before_daemon,
+	TP_ARGS(
+		pid_t, pid
+	),
+	TP_FIELDS(
+		ctf_integer(pid_t, pid, pid)
+	)
+)
 
-TRACEPOINT_EVENT(ust_tests_daemon, after_daemon_parent, TP_ARGS(), TP_FIELDS())
+TRACEPOINT_EVENT(ust_tests_daemon, after_daemon_child,
+	TP_ARGS(
+		pid_t, pid
+	),
+	TP_FIELDS(
+		ctf_integer(pid_t, pid, pid)
+	)
+)
+
+TRACEPOINT_EVENT(ust_tests_daemon, after_daemon_parent,
+	TP_ARGS(),
+	TP_FIELDS()
+)
+
+/* clang-format on */
 
 #endif /* _TRACEPOINT_UST_TESTS_DAEMON_H */
 
