@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * SPDX-FileCopyrightText: 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * SPDX-License-Identifier: MIT
  *
@@ -13,8 +13,12 @@
 
 #include <lttng/tracepoint.h>
 
+/* clang-format off */
+
 TRACEPOINT_EVENT(ust_tests_demo, starting,
-	TP_ARGS(int, value),
+	TP_ARGS(
+		int, value
+	),
 	TP_FIELDS(
 		ctf_integer(int, value, value)
 	)
@@ -22,13 +26,17 @@ TRACEPOINT_EVENT(ust_tests_demo, starting,
 TRACEPOINT_LOGLEVEL(ust_tests_demo, starting, TRACE_CRIT)
 
 TRACEPOINT_EVENT(ust_tests_demo, done,
-	TP_ARGS(int, value),
+	TP_ARGS(
+		int, value
+	),
 	TP_FIELDS(
 		ctf_integer(int, value, value)
 	)
 )
 TRACEPOINT_LOGLEVEL(ust_tests_demo, done, TRACE_CRIT)
 
+/* clang-format on */
+
 #endif /* _TRACEPOINT_UST_TESTS_DEMO_H */
 
 #undef TRACEPOINT_INCLUDE
