@@ -13,10 +13,20 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(tp,
-		 slow,
-		 TP_ARGS(unsigned int, c, const char *, thread_name),
-		 TP_FIELDS(ctf_integer(unsigned int, counter, c) ctf_string(th_name, thread_name)))
+/* clang-format off */
+
+TRACEPOINT_EVENT(tp, slow,
+	TP_ARGS(
+		unsigned int, c,
+		const char *, thread_name
+	),
+	TP_FIELDS(
+		ctf_integer(unsigned int, counter, c)
+		ctf_string(th_name, thread_name)
+	)
+)
+
+/* clang-format on */
 
 #endif /* _TRACEPOINT_TP_H */
 
