@@ -16,7 +16,14 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(libbar, bar, TP_ARGS(void), TP_FIELDS())
+/* clang-format off */
+
+TRACEPOINT_EVENT(libbar, bar,
+	TP_ARGS(void),
+	TP_FIELDS()
+)
+
+/* clang-format on */
 
 #endif /* _LIBBAR_TP_H */
 
