@@ -16,7 +16,14 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(libzzz, zzz, TP_ARGS(void), TP_FIELDS())
+/* clang-format off */
+
+TRACEPOINT_EVENT(libzzz, zzz,
+	TP_ARGS(void),
+	TP_FIELDS()
+)
+
+/* clang-format on */
 
 #endif /* _LIBZZZ_TP_H */
 
