@@ -10,6 +10,7 @@ from types import FrameType
 from typing import Callable, Iterator, Optional, Tuple, List, Generator
 import sys
 import pathlib
+import platform
 import pwd
 import random
 import signal
@@ -30,6 +31,16 @@ import contextlib
 import bt2
 
 
+def shutil_ignore_sockets(directory, directory_contents):
+    ignore = list()
+    for entry in directory_contents:
+        full_path = pathlib.Path(os.path.join(directory, entry))
+        if full_path.is_socket():
+            ignore.append(entry)
+
+    return ignore
+
+
 class TemporaryDirectory:
     def __init__(self, prefix):
         # type: (str) -> None
@@ -41,7 +52,22 @@ class TemporaryDirectory:
             for func, args, kwargs in self._cleanup_callbacks:
                 func(*args, **kwargs)
         finally:
-            if os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV", "0") != "1":
+            preserve_env = os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV", "0") != "0" or (
+                os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV_ON_FAILURE", "0") != "0"
+                and sys.exception() is not None
+            )
+            if preserve_env:
+                destination = os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV_DIR", None)
+                if destination is not None and pathlib.Path(destination).is_dir():
+                    shutil.copytree(
+                        self._directory_path,
+                        os.path.join(
+                            destination, os.path.basename(self._directory_path)
+                        ),
+                        ignore=shutil_ignore_sockets,
+                    )
+                    shutil.rmtree(self._directory_path, ignore_errors=True)
+            else:
                 shutil.rmtree(self._directory_path, ignore_errors=True)
 
     def add_cleanup_callback(self, func, *args, **kwargs):
@@ -155,7 +181,8 @@ class _LiveViewer:
     def messages(self):
         return [x for x in self._events if type(x) is bt2._EventMessageConst]
 
-    def _drain(self, retry=False):
+    def _drain(self, retry=False, timeout=0):
+        start = time.time()
         while True:
             try:
                 for msg in self._live_iterator:
@@ -165,6 +192,8 @@ class _LiveViewer:
                 break
             except bt2.TryAgain as e:
                 if retry:
+                    if timeout > 0 and (time.time() - start) > timeout:
+                        break
                     time.sleep(0.01)
                     continue
                 else:
@@ -224,11 +253,12 @@ class _LiveViewer:
     def wait_until_connected(self, timeout=0):
         return self._wait_until(True, timeout)
 
-    def wait(self):
+    def wait(self, timeout=0, close_iterator=True):
         if self._live_iterator:
-            self._drain(retry=True)
-            del self._live_iterator
-            self._live_iterator = None
+            self._drain(retry=True, timeout=timeout)
+            if close_iterator:
+                del self._live_iterator
+                self._live_iterator = None
 
     def __del__(self):
         pass
@@ -252,6 +282,10 @@ class _WaitTraceTestApplication:
         run_as=None,  # type: Optional[str]
         text_size=None,  # type: Optional[int]
         fill_text=False,  # type: bool
+        wait_before_last_event=False,  # type: bool
+        wait_before_last_event_file_path=None,  # type: Optional[pathlib.Path]
+        extra_env_vars=dict(),
+        emit_event_with_empty_field_name=False,
     ):
         self._process = None
         self._environment = environment  # type: Environment
@@ -272,6 +306,15 @@ class _WaitTraceTestApplication:
             )
         )
 
+        # File that the application will create when all the last event has been emitted
+        self._app_before_last_event_file_path = pathlib.Path(
+            tempfile.mktemp(
+                prefix="app_",
+                suffix="_before_last_event",
+                dir=dir,
+            )
+        )
+
         # File that the application will create when all events have been emitted.
         self._app_tracing_done_file_path = pathlib.Path(
             tempfile.mktemp(
@@ -289,16 +332,26 @@ class _WaitTraceTestApplication:
                     dir=dir,
                 )
             )
+
         self._wait_before_exit_file_path = wait_before_exit_file_path
         self._has_returned = False
         self._tracing_started = False
+        if wait_before_last_event and wait_before_last_event_file_path is None:
+            wait_before_last_event_file_path = pathlib.Path(
+                tempfile.mktemp(prefix="app_", suffix="_last_event", dir=dir)
+            )
 
+        self._wait_before_last_event_file_path = wait_before_last_event_file_path
         test_app_env = os.environ.copy()
         if environment.lttng_home_location is not None:
             test_app_env["LTTNG_HOME"] = str(environment.lttng_home_location)
+        if environment.lttng_rundir is not None:
+            test_app_env["LTTNG_UST_APP_PATH"] = str(environment.lttng_rundir)
+
         # Make sure the app is blocked until it is properly registered to
         # the session daemon.
         test_app_env["LTTNG_UST_REGISTER_TIMEOUT"] = "-1"
+        test_app_env.update(extra_env_vars)
 
         # File that the application will create to indicate it has completed its initialization.
         app_ready_file_path = tempfile.mktemp(
@@ -315,13 +368,29 @@ class _WaitTraceTestApplication:
         test_app_args.extend(
             ["--sync-before-first-event", str(self._app_start_tracing_file_path)]
         )
+        test_app_args.extend(
+            [
+                "--sync-before-last-event-touch",
+                str(self._app_before_last_event_file_path),
+            ]
+        )
         test_app_args.extend(
             ["--sync-before-exit-touch", str(self._app_tracing_done_file_path)]
         )
+
+        if wait_before_last_event:
+            test_app_args.extend(
+                [
+                    "--sync-before-last-event",
+                    str(self._wait_before_last_event_file_path),
+                ]
+            )
+
         if wait_before_exit:
             test_app_args.extend(
                 ["--sync-before-exit", str(self._wait_before_exit_file_path)]
             )
+
         if wait_time_between_events_us != 0:
             test_app_args.extend(["--wait", str(wait_time_between_events_us)])
 
@@ -331,6 +400,9 @@ class _WaitTraceTestApplication:
         if fill_text:
             test_app_args.extend(["--fill-text"])
 
+        if emit_event_with_empty_field_name:
+            test_app_args.extend(["--emit-event-with-empty-field-name"])
+
         if run_as is not None:
             # When running as root and reducing the permissions to run as another
             # user, the test binary needs to be readable and executable by the
@@ -380,7 +452,7 @@ class _WaitTraceTestApplication:
                     shutil.copy(lib, lib_dir)
 
             test_app_env["LD_LIBRARY_PATH"] = "{}:{}".format(
-                test_app_env["LD_LIBRARY_PATH"],
+                test_app_env.get("LD_LIBRARY_PATH", ""),
                 str(lib_dir),
             )
 
@@ -403,9 +475,21 @@ class _WaitTraceTestApplication:
             test_app_args,
             env=test_app_env,
             stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
         )  # type: subprocess.Popen
 
+        # Create and start a thread to consume and log the process output
+        if self._environment._logging_function:
+            self._output_consumer = ProcessOutputConsumer(
+                self._process,
+                "test-app-{}".format(os.path.basename(str(test_app_args[0]))),
+                self._environment._logging_function,
+            )
+            self._output_consumer.daemon = True
+            self._output_consumer.start()
+        else:
+            self._output_consumer = None
+
         # Wait for the application to create the file indicating it has fully
         # initialized. Make sure the app hasn't crashed in order to not wait
         # forever.
@@ -418,26 +502,63 @@ class _WaitTraceTestApplication:
                 break
 
             if self._process.poll() is not None:
-                # Application has unexepectedly returned.
+                # Application has exited. Re-check for the file since it could
+                # have been created between the existence check and the poll.
+                if os.path.exists(self._compat_pathlike(sync_file_path)):
+                    break
+
                 raise RuntimeError(
-                    "Test application has unexepectedly returned while waiting for synchronization file to be created: sync_file=`{sync_file}`, return_code=`{return_code}`, output=`{output}`".format(
+                    "Test application has unexpectedly returned while waiting for synchronization file to be created: sync_file=`{sync_file}`, return_code=`{return_code}`, output=`{output}`".format(
                         sync_file=sync_file_path,
                         return_code=self._process.returncode,
-                        output=self._process.stderr.read().decode("utf-8"),
+                        output=self._process.stdout.read().decode("utf-8"),
                     )
                 )
 
             time.sleep(0.001)
 
+    def taskset(self, cpu):
+        """
+        Use taskset to set the CPU affinity for the traced application.
+        """
+        if not self.vpid:
+            raise RuntimeError("No process ID")
+
+        if self._has_returned:
+            raise RuntimeError("Application is already marked as returned")
+
+        p = subprocess.Popen(["taskset", "-p", "-c", str(cpu), str(self.vpid)])
+        if p.wait() or p.returncode != 0:
+            raise RuntimeError("taskset failed, return code: {}".format(p.returncode))
+
+    def taskset_anycpu(self, retry=True):
+        while True:
+            try:
+                self.taskset(list(online_cpus())[0])
+                break
+            except RuntimeError as e:
+                if retry:
+                    pass
+
+    def touch_last_event_file(self):
+        if self._wait_before_last_event_file_path is None:
+            raise RuntimeError("wait_before_laswt_event_file_path not set")
+
+        open(self._compat_pathlike(self._wait_before_last_event_file_path), mode="x")
+
     def touch_exit_file(self):
         open(self._compat_pathlike(self._wait_before_exit_file_path), mode="x")
 
+    @property
+    def start_tracing_path(self):
+        return self._compat_pathlike(self._app_start_tracing_file_path)
+
     def trace(self):
         # type: () -> None
         if self._process.poll() is not None:
-            # Application has unexepectedly returned.
+            # Application has unexpectedly returned.
             raise RuntimeError(
-                "Test application has unexepectedly before tracing with return code `{return_code}`".format(
+                "Test application has unexpectedly returned before tracing with return code `{return_code}`".format(
                     return_code=self._process.returncode
                 )
             )
@@ -445,6 +566,13 @@ class _WaitTraceTestApplication:
         self._environment._log("[{}] Tracing started".format(self.vpid))
         self._tracing_started = True
 
+    def wait_for_before_last_event(self):
+        # type: () -> None
+        if not self._tracing_started:
+            raise RuntimeError("Tracing hasn't been started")
+        self._wait_for_file_to_be_created(self._app_before_last_event_file_path)
+        self._environment._log("[{}] Before last event done".format(self.vpid))
+
     def wait_for_tracing_done(self):
         # type: () -> None
         if not self._tracing_started:
@@ -456,14 +584,17 @@ class _WaitTraceTestApplication:
         # type: () -> None
         if self._process.wait() != 0:
             raise RuntimeError(
-                "Test application [{pid}] has exit with return code `{return_code}`, output=`{output}`".format(
+                "Test application [{pid}] has exit with return code `{return_code}`".format(
                     pid=self.vpid,
                     return_code=self._process.returncode,
-                    output=self._process.stderr.read().decode("utf-8"),
                 )
             )
         self._has_returned = True
 
+    @property
+    def status(self):
+        return self._process.poll()
+
     @property
     def vpid(self):
         # type: () -> int
@@ -576,7 +707,7 @@ class _TraceTestApplication:
     scenarios, it is preferable to use a WaitTraceTestApplication.
     """
 
-    def __init__(self, binary_path, environment):
+    def __init__(self, binary_path, environment, extra_env_vars=dict(), **kwargs):
         # type: (pathlib.Path, Environment)
         self._process = None
         self._environment = environment  # type: Environment
@@ -584,15 +715,27 @@ class _TraceTestApplication:
 
         test_app_env = os.environ.copy()
         test_app_env["LTTNG_HOME"] = str(environment.lttng_home_location)
+        if environment.lttng_rundir is not None:
+            test_app_env["LTTNG_UST_APP_PATH"] = str(environment.lttng_rundir)
+
         # Make sure the app is blocked until it is properly registered to
         # the session daemon.
         test_app_env["LTTNG_UST_REGISTER_TIMEOUT"] = "-1"
+        if type(binary_path) is str:
+            test_app_args = [str(binary_path)]
+        else:
+            # Assume it is a list
+            test_app_args = binary_path
 
-        test_app_args = [str(binary_path)]
-
+        test_app_env.update(extra_env_vars)
         self._process = subprocess.Popen(
-            test_app_args, env=test_app_env
+            test_app_args, env=test_app_env, **kwargs
         )  # type: subprocess.Popen
+        self._environment._log(
+            "Launched tested application '{}' with pid '{}'".format(
+                test_app_args, self._process.pid
+            )
+        )
 
     def wait_for_exit(self):
         # type: () -> None
@@ -608,8 +751,17 @@ class _TraceTestApplication:
         if self._process is not None and not self._has_returned:
             # This is potentially racy if the pid has been recycled. However,
             # we can't use pidfd_open since it is only available in python >= 3.9.
-            self._process.kill()
-            self._process.wait()
+            self._environment._log(
+                "Terminating test app pid '{}'".format(self._process.pid)
+            )
+            self._process.terminate()
+            try:
+                self._process.wait(timeout=self._environment.teardown_timeout)
+            except TimeoutException:
+                self._environment._log(
+                    "Killing test app pid '{}'".format(self._process.pid)
+                )
+                self._process.kill()
 
 
 class ProcessOutputConsumer(threading.Thread, logger._Logger):
@@ -627,7 +779,8 @@ class ProcessOutputConsumer(threading.Thread, logger._Logger):
     def run(self):
         # type: () -> None
         while self._process.poll() is None:
-            assert self._process.stdout
+            if not self._process.stdout:
+                break
             line = self._process.stdout.readline().decode("utf-8").replace("\n", "")
             if len(line) != 0:
                 self._log("{prefix}: {line}".format(prefix=self._prefix, line=line))
@@ -641,7 +794,8 @@ class SavingProcessOutputConsumer(ProcessOutputConsumer):
     def run(self):
         # type: () -> None
         while self._process.poll() is None:
-            assert self._process.stdout
+            if not self._process.stdout:
+                break
             line = self._process.stdout.readline().decode("utf-8").replace("\n", "")
             if len(line) != 0:
                 self._lines.append(line)
@@ -662,12 +816,14 @@ class _Environment(logger._Logger):
         extra_env_vars=dict(),  # type: dict
         skip_temporary_lttng_home=False,  # type: bool
         enable_kernel_domain=False,  # type: bool
+        skip_temporary_lttng_rundir=False,  # type: bool
+        sessiond_extra_args=None,  # type: Optional[List[str]]
     ):
         super().__init__(log)
         signal.signal(signal.SIGTERM, self._handle_termination_signal)
         signal.signal(signal.SIGINT, self._handle_termination_signal)
 
-        if os.getenv("LTTNG_TEST_VERBOSE_BABELTRACE", "0") == "1":
+        if os.getenv("LTTNG_TEST_VERBOSE_BABELTRACE", "0") != "0":
             # @TODO: Is there a way to feed the logging output to
             # the logger._Logger instead of directly to stderr?
             bt2.set_global_logging_level(bt2.LoggingLevel.TRACE)
@@ -697,21 +853,66 @@ class _Environment(logger._Logger):
                 | stat.S_IXOTH,
             )
 
+        # When starting a sessiond daemon with the root user, the rundir
+        # is global by default. Use LTTNG_RUNDIR to set it to a temporary
+        # directory to avoid trampling on parallel tests.
+        self._lttng_rundir = None
+        if os.getuid() == 0 and not skip_temporary_lttng_rundir:
+            self._lttng_rundir = TemporaryDirectory("lttng_test_env_rundir")
+            self._extra_env_vars["LTTNG_RUNDIR"] = str(self._lttng_rundir.path)
+
+        self._preserve_test_env = os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV", "0") != "1"
         self._relayd_control_port = 0
         self._relayd_data_port = 0
         self._relayd_live_port = 0
         self._relayd_env_vars = {}
+        self._relayd = None
+        self._relayd_output_consumer = None
+        self._dummy_users = {}  # type: Dictionary[int, string]
+        self.teardown_timeout = os.getenv("LTTNG_TEST_TEARDOWN_TIMEOUT", "60")
+        self._sessiond = None
+        self._sessiond_extra_args = sessiond_extra_args if sessiond_extra_args else []
+
+        # Start daemons as required
         self._relayd = (
             self._launch_lttng_relayd() if with_relayd else None
         )  # type: Optional[subprocess.Popen[bytes]]
-        self._relayd_output_consumer = None
-
         self._sessiond = (
             self._launch_lttng_sessiond(enable_kernel_domain) if with_sessiond else None
         )  # type: Optional[subprocess.Popen[bytes]]
 
-        self._dummy_users = {}  # type: Dictionary[int, string]
-        self._preserve_test_env = os.getenv("LTTNG_TEST_PRESERVE_TEST_ENV", "0") != "1"
+        # This is a bit of particularity for the testing infrastructure.
+        # When the sessiond is started, it will change the permissions of the
+        # set LTTNG_RUNDIR. When running tests that exercise a traced application
+        # run as a non-root user, it is convenient that the permissions are set
+        # so that the shm wait file in LTTNG_UST_APP_PATH can be created.
+        #
+        # The behaviour of a traced application is to disable the LTTNG_UST_APP_PATH
+        # connection if the shm wait file cannot be created, and subsequent retries
+        # only attempt connections to the global socket in /var/run/lttng/.
+        if self._lttng_rundir is not None:
+            os.chmod(
+                str(self._lttng_rundir.path),
+                stat.S_IRUSR
+                | stat.S_IWUSR
+                | stat.S_IXUSR
+                | stat.S_IROTH
+                | stat.S_IWOTH
+                | stat.S_IXOTH,
+            )
+
+    @property
+    def teardown_timeout(self):
+        return self._teardown_timeout
+
+    @teardown_timeout.setter
+    def teardown_timeout(self, value):
+        try:
+            self._teardown_timeout = int(value)
+        except TypeError:
+            self._teardown_timeout = None
+        except ValueError:
+            self._teardown_timeout = None
 
     @property
     def lttng_home_location(self):
@@ -720,6 +921,20 @@ class _Environment(logger._Logger):
             return self._lttng_home.path
         return None
 
+    @property
+    def lttng_log_dir(self):
+        path = os.getenv("LTTNG_TEST_LOG_DIR", "")
+        if path == "" or path is None or path == "-":
+            return None
+        return path
+
+    @property
+    def lttng_rundir(self):
+        # type: () -> pathlib.Path
+        if self._lttng_rundir is not None:
+            return self._lttng_rundir.path
+        return None
+
     @property
     def lttng_client_path(self):
         # type: () -> pathlib.Path
@@ -781,6 +996,18 @@ class _Environment(logger._Logger):
     def lttng_relayd_env_vars(self):
         return self._relayd_env_vars
 
+    @property
+    def lttng_relayd_output_path(self):
+        return os.path.join(
+            (
+                str(self.lttng_home_location)
+                if self.lttng_home_location
+                else os.getenv("HOME")
+            ),
+            "lttng-traces",
+            platform.node(),
+        )
+
     @property
     def preserve_test_env(self):
         # type: () -> bool
@@ -793,7 +1020,7 @@ class _Environment(logger._Logger):
 
     def create_dummy_user(self):
         # type: () -> (int, str)
-        # Create a dummy user. The uid and username will be eturned in a tuple.
+        # Create a dummy user. The uid and username will be returned in a tuple.
         # If the name already exists, an exception will be thrown.
         # The users will be removed when the environment is cleaned up.
         name = "".join([random.choice(string.ascii_lowercase) for x in range(10)])
@@ -829,15 +1056,15 @@ class _Environment(logger._Logger):
         self._dummy_users[entry[2]] = name
         return (entry[2], name)
 
-    def create_temporary_directory(self, prefix=None):
-        # type: (Optional[str]) -> pathlib.Path
+    def create_temporary_directory(self, prefix=None, dir=None):
+        # type: (Optional[str], Optional[str]) -> pathlib.Path
         # Simply return a path that is contained within LTTNG_HOME; it will
         # be destroyed when the temporary home goes out of scope.
+        if dir is None:
+            dir = str(self.lttng_home_location) if self.lttng_home_location else None
+
         return pathlib.Path(
-            tempfile.mkdtemp(
-                prefix="tmp" if prefix is None else prefix,
-                dir=str(self.lttng_home_location) if self.lttng_home_location else None,
-            )
+            tempfile.mkdtemp(prefix="tmp" if prefix is None else prefix, dir=dir)
         )
 
     @staticmethod
@@ -848,6 +1075,12 @@ class _Environment(logger._Logger):
             and os.getuid() == 0
         )
 
+    @staticmethod
+    def run_long_regression_tests():
+        # type: () -> bool
+        value = os.getenv("LTTNG_TOOLS_RUN_TESTS_LONG_REGRESSION")
+        return bool(value and value != "0")
+
     # Unpack a list of environment variables from a string
     # such as "HELLO=is_it ME='/you/are/looking/for'"
     @staticmethod
@@ -901,8 +1134,15 @@ class _Environment(logger._Logger):
         )
 
         verbose = []
-        if os.environ.get("LTTNG_TEST_VERBOSE_RELAYD") is not None:
+        if os.getenv("LTTNG_TEST_VERBOSE_RELAYD", "0") != "0":
             verbose = ["-vvv"]
+        self._relayd_log_file = None
+        if self.lttng_log_dir:
+            self._relayd_log_file = tempfile.NamedTemporaryFile(
+                prefix="relayd_",
+                dir=self.lttng_log_dir,
+                delete=False,
+            )
         process = subprocess.Popen(
             [
                 str(relayd_path),
@@ -915,7 +1155,9 @@ class _Environment(logger._Logger):
                 "tcp://localhost:{}".format(self.lttng_relayd_live_port),
             ]
             + verbose,
-            stdout=subprocess.PIPE,
+            stdout=(
+                self._relayd_log_file.file if self._relayd_log_file else subprocess.PIPE
+            ),
             stderr=subprocess.STDOUT,
             env=relayd_env,
         )
@@ -1038,13 +1280,28 @@ class _Environment(logger._Logger):
                 str(consumerd_path),
                 "--sig-parent",
             ]
-            if os.environ.get("LTTNG_TEST_VERBOSE_SESSIOND") is not None:
+            if os.getenv("LTTNG_TEST_VERBOSE_SESSIOND", "0") != "0":
                 sessiond_command.extend(["-vvv", "--verbose-consumer"])
             if not enable_kernel_domain:
                 sessiond_command.extend(["--no-kernel"])
+            if self._sessiond_extra_args:
+                sessiond_command.extend(self._sessiond_extra_args)
+            self._lttng_sessiond_log_file = None
+            if self.lttng_log_dir:
+                self._lttng_sessiond_log_file = tempfile.NamedTemporaryFile(
+                    prefix="sessiond_",
+                    dir=self.lttng_log_dir,
+                    delete=False,
+                )
+
+            self._log(" ".join(sessiond_command))
             process = subprocess.Popen(
                 sessiond_command,
-                stdout=subprocess.PIPE,
+                stdout=(
+                    self._lttng_sessiond_log_file.file
+                    if self._lttng_sessiond_log_file
+                    else subprocess.PIPE
+                ),
                 stderr=subprocess.STDOUT,
                 env=sessiond_env,
             )
@@ -1139,18 +1396,17 @@ class _Environment(logger._Logger):
         run_as=None,
         text_size=None,
         fill_text=False,
+        wait_before_last_event=False,
+        wait_before_last_event_file_path=None,
+        extra_env_vars=dict(),
+        emit_event_with_empty_field_name=False,
     ):
         # type: (int, int, bool, Optional[pathlib.Path], Optional[str]) -> _WaitTraceTestApplication
         """
         Launch an application that will wait before tracing `event_count` events.
         """
         return _WaitTraceTestApplication(
-            self._project_root
-            / "tests"
-            / "utils"
-            / "testapp"
-            / "gen-ust-events"
-            / "gen-ust-events",
+            self._project_root / "tests" / "utils" / "testapp" / "gen-ust-events",
             event_count,
             self,
             wait_time_between_events_us,
@@ -1159,22 +1415,78 @@ class _Environment(logger._Logger):
             run_as,
             text_size,
             fill_text,
+            wait_before_last_event,
+            wait_before_last_event_file_path,
+            extra_env_vars,
+            emit_event_with_empty_field_name,
         )
 
-    def launch_test_application(self, subpath):
+    def launch_multi_event_wait_trace_test_application(
+        self,
+        event_count,  # type: int
+        wait_time_between_events_us=0,
+        wait_before_exit=False,
+        wait_before_exit_file_path=None,
+        run_as=None,
+        wait_before_last_event=False,
+        wait_before_last_event_file_path=None,
+    ):
+        # type: (int, int, bool, Optional[pathlib.Path], Optional[str], bool, Optional[pathlib.Path]) -> _WaitTraceTestApplication
+        """
+        Launch the gen-many-event-classes application that will wait before tracing `event_count` iterations.
+        Each iteration emits events from multiple event classes.
+        Note: text_size and fill_text parameters are not supported by this application.
+        """
+        return _WaitTraceTestApplication(
+            self._project_root
+            / "tests"
+            / "utils"
+            / "testapp"
+            / "gen-many-event-classes"
+            / "gen-many-event-classes",
+            event_count,
+            self,
+            wait_time_between_events_us,
+            wait_before_exit,
+            wait_before_exit_file_path,
+            run_as,
+            text_size=None,
+            fill_text=False,
+            wait_before_last_event=wait_before_last_event,
+            wait_before_last_event_file_path=wait_before_last_event_file_path,
+        )
+
+    def launch_test_application(self, path, extra_env_vars=dict(), **kwargs):
         # type () -> TraceTestApplication
         """
-        Launch an application that will trace from within constructors.
+        Launch an application with it's environment set for being traced
+        by this test environment's session daemon.
         """
         return _TraceTestApplication(
-            self._project_root / "tests" / "utils" / "testapp" / subpath,
+            path,
             self,
+            extra_env_vars,
+            **kwargs,
         )
 
     def _terminate_relayd(self):
         if self._relayd and self._relayd.poll() is None:
-            self._relayd.terminate()
-            self._relayd.wait()
+            try:
+                self._relayd.terminate()
+                self._log(
+                    "Termination signal sent to relayd (pid = {})".format(
+                        self._relayd.pid
+                    )
+                )
+                self._relayd.wait(timeout=self.teardown_timeout)
+            except subprocess.TimeoutExpired:
+                self._relayd.kill()
+                self._log(
+                    "Kill signal sent to relayd (pid = {}) after waiting {}s".format(
+                        self._relayd.pid, self.teardown_timeout
+                    )
+                )
+
             if self._relayd_output_consumer:
                 self._relayd_output_consumer.join()
                 self._relayd_output_consumer = None
@@ -1186,14 +1498,21 @@ class _Environment(logger._Logger):
         # type: () -> None
         if self._sessiond and self._sessiond.poll() is None:
             # The session daemon is alive; kill it.
-            self._log(
-                "Killing session daemon (pid = {sessiond_pid})".format(
-                    sessiond_pid=self._sessiond.pid
+            try:
+                self._sessiond.terminate()
+                self._log(
+                    "Termination signal sent to session daemon (pid = {})".format(
+                        self._sessiond.pid
+                    )
+                )
+                self._sessiond.wait(timeout=self.teardown_timeout)
+            except subprocess.TimeoutExpired:
+                self._sessiond.kill()
+                self._log(
+                    "Kill signal sent to lttng-sessiond (pid = {}) after waiting {}s".format(
+                        self._sessiond.pid, self.teardown_timeout
+                    )
                 )
-            )
-
-            self._sessiond.terminate()
-            self._sessiond.wait()
             if self._sessiond_output_consumer:
                 self._sessiond_output_consumer.join()
                 self._sessiond_output_consumer = None
@@ -1250,6 +1569,15 @@ def getconf(name):
     return p.stdout.read().decode("utf-8").strip()
 
 
+def get_machine():
+    """
+    Returns a machine identifier
+    """
+    return "{}-{}-{}".format(
+        platform.machine(), platform.architecture()[0], platform.system().lower()
+    )
+
+
 def count_events(trace_path, ignore_exceptions=False):
     """
     Returns a tuple of (received events, discarded events) from the trace path.
@@ -1272,7 +1600,7 @@ def count_events(trace_path, ignore_exceptions=False):
                 if type(msg) is bt2._DiscardedEventsMessageConst:
                     # msg.count may be None when the value is indeterminate
                     discarded += msg.count if msg.count is not None else 1
-        except RuntimeError as e:
+        except Exception as e:
             if not ignore_exceptions:
                 raise e
 
@@ -1386,8 +1714,10 @@ def test_environment(
     extra_env_vars=dict(),
     skip_temporary_lttng_home=False,
     enable_kernel_domain=False,
+    skip_temporary_lttng_rundir=False,
+    sessiond_extra_args=None,
 ):
-    # type: (bool, Optional[Callable[[str], None]], bool) -> Iterator[_Environment]
+    # type: (bool, Optional[Callable[[str], None]], bool, dict, bool, bool, bool, Optional[List[str]]) -> Iterator[_Environment]
     env = _Environment(
         with_sessiond,
         log,
@@ -1395,6 +1725,8 @@ def test_environment(
         extra_env_vars,
         skip_temporary_lttng_home,
         enable_kernel_domain,
+        skip_temporary_lttng_rundir,
+        sessiond_extra_args,
     )
     try:
         yield env
