@@ -6,11 +6,13 @@
 
 
 from . import lttngctl, logger, environment
+
+import enum
 import os
-from typing import Callable, Optional, Type, Union, Iterator
 import shlex
 import subprocess
-import enum
+import tempfile
+from typing import Callable, Optional, Type, Union, Iterator
 import xml.etree.ElementTree
 
 """
@@ -77,7 +79,7 @@ def _get_context_type_name(context):
     else:
         raise Unsupported(
             "Context `{context_name}` is not supported by the LTTng client".format(
-                type(context).__name__
+                context_name=type(context).__name__
             )
         )
 
@@ -261,7 +263,7 @@ class _Channel(lttngctl.Channel):
                 client_args = client_args + " --all"
 
             if rule.filter_expression:
-                client_args = client_args + " " + rule.filter_expression
+                client_args = client_args + " --filter " + rule.filter_expression
 
             if getattr(rule, "log_level_rule", None):
                 if isinstance(rule.log_level_rule, lttngctl.LogLevelRuleAsSevereAs):
@@ -311,7 +313,7 @@ class _Channel(lttngctl.Channel):
     @property
     def recording_rules(self):
         # type: () -> Iterator[lttngctl.EventRule]
-        list_session_xml = self._client._run_cmd(
+        list_session_xml, _ = self._client._run_cmd(
             "list '{session_name}'".format(session_name=self._session.name),
             LTTngClient.CommandOutputFormat.MI_XML,
         )
@@ -517,16 +519,24 @@ class _Session(lttngctl.Session):
         buffer_allocation_policy=lttngctl.BufferAllocationPolicy.PerCPU,
         subbuf_size=None,
         subbuf_count=None,
+        tracefile_size=None,
+        tracefile_count=None,
+        event_record_loss_mode=None,
+        watchdog_timer_period_us=None,
+        buffer_preallocation_policy=None,
+        auto_reclaim_memory_older_than=None,
+        auto_reclaim_memory_consumed=False,
     ):
-        # type: (lttngctl.TracingDomain, Optional[str], lttngctl.BufferSharingPolicy), Optional[str], Optional[str] -> lttngctl.Channel
-        channel_name = lttngctl.Channel._generate_name()
+        # type: (lttngctl.TracingDomain, Optional[str], lttngctl.BufferSharingPolicy, lttngctl.BufferAllocationPolicy, Optional[int], Optional[int], Optional[int], Optional[int], Optional[lttngctl.EventRecordLossMode], Optional[int]) -> lttngctl.Channel
+        if channel_name is None:
+            channel_name = lttngctl.Channel._generate_name()
         domain_option_name = _get_domain_option_name(domain)
-        buffer_sharing_policy = (
+        buffer_sharing_policy_cli_arg = (
             "--buffer-ownership=user"
             if buffer_sharing_policy == lttngctl.BufferSharingPolicy.PerUID
             else "--buffer-ownership=process"
         )
-        buffer_allocation_policy = (
+        buffer_allocation_policy_cli_arg = (
             "--buffer-allocation=per-cpu"
             if buffer_allocation_policy == lttngctl.BufferAllocationPolicy.PerCPU
             else "--buffer-allocation=per-channel"
@@ -538,14 +548,37 @@ class _Session(lttngctl.Session):
             "--{}".format(domain_option_name),
             channel_name,
         ]
-        args.append(buffer_allocation_policy)
+        args.append(buffer_allocation_policy_cli_arg)
         if domain != lttngctl.TracingDomain.Kernel:
-            args.append(buffer_sharing_policy)
+            args.append(buffer_sharing_policy_cli_arg)
         if subbuf_size is not None:
             args.extend(["--subbuf-size", str(subbuf_size)])
         if subbuf_count is not None:
             args.extend(["--num-subbuf", str(subbuf_count)])
-
+        if tracefile_count is not None:
+            args.extend(["--tracefile-count", str(tracefile_count)])
+        if tracefile_size is not None:
+            args.extend(["--tracefile-size", str(tracefile_size)])
+        if event_record_loss_mode is not None:
+            args.append(
+                "--overwrite"
+                if event_record_loss_mode == lttngctl.EventRecordLossMode.Overwrite
+                else "--discard"
+            )
+        if watchdog_timer_period_us is not None:
+            args.append("--watchdog-timer={}".format(watchdog_timer_period_us))
+        if buffer_preallocation_policy:
+            args.append(
+                "--buffer-preallocation={}".format(buffer_preallocation_policy.as_arg())
+            )
+        if auto_reclaim_memory_older_than:
+            args.append(
+                "--auto-reclaim-memory=older-than:{}".format(
+                    auto_reclaim_memory_older_than
+                )
+            )
+        elif auto_reclaim_memory_consumed:
+            args.append("--auto-reclaim-memory=consumed")
         self._client._run_cmd(" ".join([shlex.quote(x) for x in args]))
         return _Channel(self._client, channel_name, domain, self)
 
@@ -570,9 +603,56 @@ class _Session(lttngctl.Session):
         # type: () -> None
         self._client._run_cmd("clear '{session_name}'".format(session_name=self.name))
 
-    def destroy(self):
-        # type: () -> None
-        self._client._run_cmd("destroy '{session_name}'".format(session_name=self.name))
+    def destroy(self, timeout_s=None):
+        # type: (Optional[float]) -> None
+        args = [
+            "destroy",
+            self.name,
+        ]
+        if timeout_s == 0:
+            args.append("--no-wait")
+            timeout_s = None
+
+        self._client._run_cmd(
+            " ".join([shlex.quote(x) for x in args]), timeout_s=timeout_s
+        )
+
+    def reclaim_memory(
+        self,
+        wait=True,
+        older_than_us=None,
+        session=None,
+        channels=None,
+        all_channels=False,
+    ):
+        args = [
+            "reclaim-memory",
+            "--userspace",
+        ]
+
+        if not wait:
+            args.append("--no-wait")
+
+        if older_than_us:
+
+            if not isinstance(older_than_us, int):
+                raise Exception(
+                    "Parameter `older_than_us={}` is not an integer".format(
+                        older_than_us
+                    )
+                )
+
+            args.append("--older-than={}".format(older_than_us))
+
+        if session:
+            args.append("--session={}".format(session))
+
+        if channels:
+            args.extend(channels)
+        elif all_channels:
+            args.append("--all")
+
+        self._client._run_cmd(" ".join([shlex.quote(arg) for arg in args]))
 
     def rotate(self, wait=True):
         # type: (bool) -> None
@@ -591,10 +671,22 @@ class _Session(lttngctl.Session):
         else:
             self._client.snapshot_record(self.name, None)
 
+    def regenerate(self, target):
+        # type: lttngctl.SessionRegenerateTarget
+        if not isinstance(target, lttngctl.SessionRegenerateTarget):
+            raise RuntimeError(
+                "Expected target of type `{}`, got `{}`".format(
+                    lttngctl.SessionRegenerateTarget, type(target)
+                )
+            )
+
+        args = ["regenerate", str(target), "-s", self.name]
+        self._client._run_cmd(" ".join([shlex.quote(x) for x in args]))
+
     @property
     def is_active(self):
         # type: () -> bool
-        list_session_xml = self._client._run_cmd(
+        list_session_xml, _ = self._client._run_cmd(
             "list '{session_name}'".format(session_name=self.name),
             LTTngClient.CommandOutputFormat.MI_XML,
         )
@@ -664,10 +756,12 @@ class LTTngClientError(lttngctl.ControlException):
     def __init__(
         self,
         command_args,  # type: str
+        output,  # type: str
         error_output,  # type: str
     ):
         self._command_args = command_args  # type: str
-        self._output = error_output  # type: str
+        self._output = output  # type: str
+        self._error_output = error_output  # type: str
 
 
 class LTTngClient(logger._Logger, lttngctl.Controller):
@@ -680,6 +774,7 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
         HUMAN = 1
 
     _MI_NS = "{https://lttng.org/xml/ns/lttng-mi}"
+    _timeout_s = None
 
     def __init__(
         self,
@@ -691,18 +786,44 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
         self._environment = test_environment  # type: environment._Environment
         self._extra_env_vars = extra_env_vars
 
+        # The client needs to know where to find the LTTng session configuration
+        # XSD file for validating session configuration files when `lttng-load`
+        # is used.
+        self._extra_env_vars.setdefault(
+            "LTTNG_SESSION_CONFIG_XSD_PATH",
+            str(self._environment._project_root / "src" / "common"),
+        )
+
     @staticmethod
     def _namespaced_mi_element(property):
         # type: (str) -> str
         return LTTngClient._MI_NS + property
 
-    def _run_cmd(self, command_args, output_format=CommandOutputFormat.MI_XML):
-        # type: (str, CommandOutputFormat) -> str
+    @property
+    def timeout(self):
+        return self._timeout_s
+
+    @timeout.setter
+    def timeout(self, value):
+        if value is None:
+            self._timeout_s = None
+        else:
+            self._timeout_s = int(value)
+
+    def _run_cmd(
+        self, command_args, output_format=CommandOutputFormat.MI_XML, timeout_s=None
+    ):
+        # type: (str, CommandOutputFormat, Optional[float]) -> tuple[str, str]
         """
         Invoke the `lttng` client with a set of arguments. The command is
         executed in the context of the client's test environment.
+
+        Returns a tuple containing (stdout, stderr) that has been decoded to
+        UTF-8.
         """
         args = [str(self._environment.lttng_client_path)]  # type: list[str]
+        if os.getenv("LTTNG_TEST_VERBOSE_CLIENT", "0") != "0":
+            args.extend(["-vvv"])
         if output_format == LTTngClient.CommandOutputFormat.MI_XML:
             args.extend(["--mi", "xml"])
 
@@ -713,27 +834,49 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
         client_env = os.environ.copy()  # type: dict[str, str]
         if self._environment.lttng_home_location is not None:
             client_env["LTTNG_HOME"] = str(self._environment.lttng_home_location)
+        if self._environment.lttng_rundir is not None:
+            client_env["LTTNG_RUNDIR"] = str(self._environment.lttng_rundir)
         client_env.update(self._extra_env_vars)
 
+        err_output = (
+            tempfile.NamedTemporaryFile(
+                prefix="lttng_", dir=self._environment.lttng_log_dir, delete=False
+            )
+            if self._environment.lttng_log_dir is not None
+            else None
+        )
         process = subprocess.Popen(
-            args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=client_env
+            args,
+            stdout=subprocess.PIPE,
+            stderr=err_output.file if err_output else subprocess.PIPE,
+            env=client_env,
         )
 
-        out = process.communicate()[0]
+        out, err = process.communicate(timeout=timeout_s or self.timeout)
+        out = out.decode("utf-8")
+        if not err_output:
+            err = err.decode("utf-8")
+        else:
+            err = open(err_output.name, "r", encoding="utf-8").read()
+
+        for error_line in err.splitlines():
+            self._log(error_line)
 
         if process.returncode != 0:
-            decoded_output = out.decode("utf-8")
-            for error_line in decoded_output.splitlines():
-                self._log(error_line)
-
-            raise LTTngClientError(command_args, decoded_output)
+            raise LTTngClientError(command_args, out, err)
         else:
-            return out.decode("utf-8")
+            return (out, err)
 
     def create_session(
-        self, name=None, output=None, live=False, snapshot=False, shm_path=None
+        self,
+        name=None,
+        output=None,
+        live=False,
+        snapshot=False,
+        shm_path=None,
+        trace_format=None,
     ):
-        # type: (Optional[str], Optional[lttngctl.SessionOutputLocation], bool, Optional[pathlib.Path]) -> lttngctl.Session
+        # type: (Optional[str], Optional[lttngctl.SessionOutputLocation], bool, bool, Optional[pathlib.Path], Optional[lttngctl.TraceFormat]) -> lttngctl.Session
         name = name if name else lttngctl.Session._generate_name()
         args = ["create", name]
 
@@ -747,11 +890,13 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
             raise TypeError("LTTngClient only supports local or no output")
 
         if live:
-            args.append("--live")
+            args.append("--live={}".format(live) if type(live) is int else "--live")
         if shm_path is not None:
             args.extend(["--shm-path", str(shm_path)])
         if snapshot:
             args.append("--snapshot")
+        if trace_format is not None:
+            args.extend(["--trace-format", trace_format.value])
         self._run_cmd(" ".join([shlex.quote(x) for x in args]))
         return _Session(self, name, output)
 
@@ -842,7 +987,7 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
 
     def list_sessions(self):
         # type () -> List[Session]
-        list_sessions_xml = self._run_cmd(
+        list_sessions_xml, _ = self._run_cmd(
             "list", LTTngClient.CommandOutputFormat.MI_XML
         )
 
@@ -876,7 +1021,7 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
         return ctl_sessions
 
     def list_session_raw(self, session):
-        list_sessions_xml = self._run_cmd(
+        list_sessions_xml, _ = self._run_cmd(
             "list {}".format(session), LTTngClient.CommandOutputFormat.MI_XML
         )
 
@@ -887,3 +1032,51 @@ class LTTngClient(logger._Logger, lttngctl.Controller):
         if len(sessions) > 0:
             return sessions[0]
         raise InvalidMI("Invalid empty 'sessions element in '{}".format(session_mi.tag))
+
+    def save_sessions(self, session_name=None, force=False, output_path=None):
+
+        cmd = [
+            "save",
+        ]
+
+        if force:
+            cmd.append("--force")
+
+        if output_path:
+            cmd.append("--output-path={}".format(output_path))
+        else:
+            cmd.append(
+                "--output-path={}".format(
+                    str(self._environment.lttng_home_location) + "/.lttng/sessions"
+                )
+            )
+
+        if session_name:
+            cmd.append(session_name)
+        else:
+            cmd.append("--all")
+
+        self._run_cmd(" ".join(cmd))
+
+    def load_sessions(self, session_name=None, force=False, input_path=None):
+
+        cmd = ["load"]
+
+        if force:
+            cmd.append("--force")
+
+        if input_path:
+            cmd.append("--input-path={}".format(input_path))
+        else:
+            cmd.append(
+                "--input-path={}".format(
+                    str(self._environment.lttng_home_location) + "/.lttng/sessions"
+                )
+            )
+
+        if session_name:
+            cmd.append(session_name)
+        else:
+            cmd.append("--all")
+
+        self._run_cmd(" ".join(cmd))
