@@ -118,6 +118,38 @@ class BufferAllocationPolicy(enum.Enum):
         return "<%s.%s>" % (self.__class__.__name__, self.name)
 
 
+@enum.unique
+class BufferPreAllocationPolicy(enum.Enum):
+    """Buffer preallocation policy."""
+
+    PreAllocate = "Pre-allocate allocation"
+    OnDemand = "On-demand allocation"
+
+    def __repr__(self):
+        return "<%s.%s>" % (self.__class__.__name__, self.name)
+
+    def as_arg(self):
+        """
+        Return the string value for lttng cli
+        """
+        if self.name == "PreAllocate":
+            return "preallocate"
+        elif self.name == "OnDemand":
+            return "on-demand"
+        assert False
+
+
+@enum.unique
+class EventRecordLossMode(enum.Enum):
+    """Event record loss mode."""
+
+    Discard = "Per-CPU allocation"
+    Overwrite = "Per-Channel allocation"
+
+    def __repr__(self):
+        return "<%s.%s>" % (self.__class__.__name__, self.name)
+
+
 class EventRule(abc.ABC):
     """Event rule base class, see LTTNG-EVENT-RULE(7)."""
 
@@ -138,6 +170,26 @@ class LogLevel(enum.Enum):
     pass
 
 
+@enum.unique
+class SessionRegenerateTarget(enum.Enum):
+    """Session regeneration target data type"""
+
+    Metadata = "metadata"
+    Statedump = "statedump"
+
+    def __str__(self):
+        return self.value
+
+    def __repr__(self):
+        return "<%s.%s>" % (self.__class__.__name__, self.name)
+
+
+@enum.unique
+class TraceFormat(enum.Enum):
+    CTF_1_8 = "ctf-1.8"
+    CTF_2 = "ctf-2"
+
+
 @enum.unique
 class UserLogLevel(LogLevel):
     EMERGENCY = 0
@@ -637,8 +689,14 @@ class Session(abc.ABC):
         channel_name=None,
         buffer_sharing_policy=BufferSharingPolicy.PerUID,
         buffer_allocation_policy=BufferAllocationPolicy.PerCPU,
+        subbuf_size=None,
+        subbuf_count=None,
+        tracefile_size=None,
+        tracefile_count=None,
+        event_record_loss_mode=None,
+        watchdog_timer_period_us=None,
     ):
-        # type: (TracingDomain, Optional[str], BufferSharingPolicy) -> Channel
+        # type: (TracingDomain, Optional[str], BufferSharingPolicy, BufferAllocationPolicy, Optional[int], Optional[int], Optional[int], Optional[int], Optional[EventRecordLossMode], Optional[int]) -> Channel
         """Add a channel with default attributes to the session."""
         raise NotImplementedError
 
@@ -652,9 +710,19 @@ class Session(abc.ABC):
         # type: () -> None
         raise NotImplementedError
 
-    @abc.abstractmethod
-    def destroy(self):
-        # type: () -> None
+    def destroy(self, timeout_s=None):
+        # type: (Optional[float]) -> None
+        """
+        Destroy the session.
+
+        If `timeout_s` is specified, wait for the
+        session to be destroyed for up to `timeout_s` seconds. Specifying `0`
+        invokes a non-blocking destroy operation, which will return immediately
+        and the session will be destroyed asynchronously.
+
+        If `timeout_s` is not specified, wait indefinitely for the session to
+        be destroyed.
+        """
         raise NotImplementedError
 
     @abc.abstractmethod
@@ -663,8 +731,8 @@ class Session(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def rotate(self):
-        # type: () -> None
+    def rotate(self, wait=True):
+        # type: (bool) -> None
         raise NotImplementedError
 
     @abc.abstractmethod
@@ -737,9 +805,15 @@ class Controller(abc.ABC):
 
     @abc.abstractmethod
     def create_session(
-        self, name=None, output=None, live=False, snapshot=False, shm_path=None
+        self,
+        name=None,
+        output=None,
+        live=False,
+        snapshot=False,
+        shm_path=None,
+        trace_format=None,
     ):
-        # type: (Optional[str], Optional[lttngctl.SessionOutputLocation], bool, Optional[pathlib.Path]) -> lttngctl.Session
+        # type: (Optional[str], Optional[lttngctl.SessionOutputLocation], bool, bool, Optional[pathlib.Path], Optional[TraceFormat]) -> lttngctl.Session
         """
         Create a session with an output. Don't specify an output
         to create a session without an output.
